VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DSMapBook"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

' Copyright 2006 ESRI
'
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
'
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
'
' See use restrictions at /arcgis/developerkit/userestrictions.

Option Explicit

Implements IDSMapBook
Implements IPersistVariant

Private m_ContentColl As Collection
Private m_bEnableBook As Boolean
Private m_dJAMBVersionNumber As Double 'Added 02/26/2009
Private m_dMapVersionNumber As Double 'Added 02/26/2009

Const MIN_MAP_VERSION_NUMBER = 0.5

Private Sub Class_Initialize()
'MsgBox "DSMapBook: Starting Class Initialize"
26:   Set m_ContentColl = New Collection
  
  'Read the version number.
29:   m_dJAMBVersionNumber = GetVersionNumber
30:   m_dMapVersionNumber = GetVersionNumber
'MsgBox "DSMapBook: Ending Class Initialize"
End Sub

Private Sub Class_Terminate()
'MsgBox "DSMapBook: Starting Class Terminate"
36:   Set m_ContentColl = Nothing
'MsgBox "DSMapBook: Ending Class Terminate"
End Sub

Private Function GetVersionNumber() As Double
  Dim dVersionNumber As Double
  
43:   If App.Minor > 9 Then MsgBox "Warning: Minor version number is greater than 9.  JAMB will not record the proper version number."
44:   If App.Revision > 9 Then MsgBox "Warning: Revision version number is greater than 9.  JAMB will not record the proper version number."
  
46:   dVersionNumber = App.Major + (App.Minor * 0.1) + (App.Revision * 0.01)
47:   GetVersionNumber = dVersionNumber
End Function

Private Sub IDSMapBook_AddContent(ByVal Content As Object)
51:   m_ContentColl.Add Content
End Sub

Private Property Get IDSMapBook_ContentCount() As Long
55:   IDSMapBook_ContentCount = m_ContentColl.Count
End Property

Private Property Get IDSMapBook_ContentItem(index As Long) As Object
59:   If index > -1 And index < m_ContentColl.Count Then
60:     Set IDSMapBook_ContentItem = m_ContentColl.Item(index + 1)
61:   Else
62:     Set IDSMapBook_ContentItem = Nothing
63:   End If
End Property

Private Property Let IDSMapBook_EnableBook(ByVal RHS As Boolean)
67:   m_bEnableBook = RHS
End Property

Private Property Get IDSMapBook_EnableBook() As Boolean
71:   IDSMapBook_EnableBook = m_bEnableBook
End Property

Private Sub IDSMapBook_RemoveContent(index As Long)
75:   If index > -1 And index < m_ContentColl.Count Then
76:     m_ContentColl.Remove index + 1
77:   End If
End Sub

Private Property Get IPersistVariant_ID() As esriSystem.IUID
  Dim id As New UID
82:   id = "JAMB_Back_End.DSMapBook"
83:   Set IPersistVariant_ID = id
End Property

Private Sub IPersistVariant_Load(ByVal Stream As esriSystem.IVariantStream)
'Load the persisted parameters of the renderer
On Error GoTo ErrHand
  Dim lLoop As Long, lCount As Long, pMapSeries As IDSMapSeries
  Dim vCount As Variant, lPropCount As Long, Props As Collection
  
  'Added count variable 02/26/2009 to make the list of properties dynamic.
93:   vCount = Stream.Read
  
95:   If UCase(TypeName(vCount)) = "BOOLEAN" Then 'If this is a boolean, this version pre-dates 02/26/2009.
96:     m_bEnableBook = vCount
97:     m_dMapVersionNumber = MIN_MAP_VERSION_NUMBER
98:   Else 'Otherwise, the version is from 02/26/2009 or later and contains a property count variable.
99:     lPropCount = vCount
'MsgBox "Map Book - lPropCount: " & lPropCount & vbNewLine & "vCount: " & CStr(vCount)
101:     If lPropCount < 1 Then Err.Raise 0, "", "Property count was less than 1."
102:     Set Props = New Collection
    Dim lx As Long
104:     For lx = 1 To lPropCount
105:       Props.Add Stream.Read
106:     Next lx
107:     If lPropCount >= 2 Then
108:       m_bEnableBook = Props.Item(1)
109:       m_dMapVersionNumber = Props.Item(2)
110:     Else
111:       m_bEnableBook = True
112:       m_dMapVersionNumber = MIN_MAP_VERSION_NUMBER
113:     End If
114:   End If
'MsgBox "MapVersionNumber: " & m_dMapVersionNumber & vbNewLine & "MIN_MAP_VERSION_NUMBER: " & MIN_MAP_VERSION_NUMBER
116:   If m_dMapVersionNumber <= MIN_MAP_VERSION_NUMBER Then
'MsgBox "This map was saved with JAMB version 0.5 or earlier."
118:   Else
'MsgBox "This map was saved with version " & m_dMapVersionNumber & " of JAMB."
120:   End If

    
  'm_bEnableBook = Stream.Read
124:   lCount = Stream.Read
125:   Set m_ContentColl = New Collection
126:   If lCount > 0 Then
127:     For lLoop = 1 To lCount
128:       Set pMapSeries = Stream.Read
129:       m_ContentColl.Add pMapSeries
130:     Next lLoop
131:   End If
        
  'Exit if the number of map series is 0
  If (lCount = 0) Or (m_ContentColl.Count = 0) Then Exit Sub
    
  'Warn the user if the map book was made with a newer version of the application.
137:   If m_dMapVersionNumber > m_dJAMBVersionNumber Then
138:     MsgBox "Warning: This map was saved with a version of JAMB that is newer than your current version." _
    & vbNewLine & "If you save the map, it will be downgraded to your current version." _
    & vbNewLine & "You Current JAMB Version: " & m_dJAMBVersionNumber _
    & vbNewLine & "The map was saved with version: " & m_dMapVersionNumber
142:   End If
  
  Exit Sub
ErrHand:
146:   MsgBox "MapBook - IPersistVariant_Load - " & Erl & " - " & Err.Description
End Sub

Private Sub IPersistVariant_Save(ByVal Stream As esriSystem.IVariantStream)
'Write it all out
On Error GoTo ErrHand
  Dim lLoop As Long
  Dim lPropCount As Long
154:   lPropCount = 2
155:   Stream.Write lPropCount 'Save a property count
  'Save the Map Book Properties:
157:   Stream.Write m_bEnableBook 'Enabled
158:   Stream.Write m_dJAMBVersionNumber 'Version Number
  
  'Save the collection of Maps Series
161:   Stream.Write m_ContentColl.Count
162:   If m_ContentColl.Count > 0 Then
163:     For lLoop = 1 To m_ContentColl.Count
164:       Stream.Write m_ContentColl.Item(lLoop)
165:     Next lLoop
166:   End If
    
  Exit Sub
ErrHand:
170:   MsgBox "MapBook - IPersistVariant_Save - " & Erl & " - " & Err.Description
End Sub
