VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DSMapPage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

' Copyright 2006 ESRI
'
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
'
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
'
' See use restrictions at /arcgis/developerkit/userestrictions.

Option Explicit

Implements IDSMapPage
Implements IPersistVariant

Implements IDSMapSeriesGraphProps 'Added 02/12/2009 (Used to store any page-specific data graph info.)

Dim m_PageItemColl As Collection
Dim m_sPageName As String
Dim m_bPrintPage As Boolean
Dim m_dPageRotation As Double
Dim m_dPageScale As Double
Dim m_dLastOutputted As Date
Dim m_pPageShape As IPolygon
Dim m_lPageNumber As Long
Dim m_routePolyline As IPolyline
Dim m_lFromStat As Double
Dim m_lToStat As Double

'Added these new properties which track information about a data graph. (Added 02/12/2009)
Private m_bHasDataGraphPosition As Boolean
Private m_dDataGraphXMin As Double
Private m_dDataGraphYMin As Double
Private m_dDataGraphWidth As Double
Private m_dDataGraphHeight As Double

Const m_HIDE_LOCATION_X As Long = -9999999
Const m_HIDE_LOCATION_Y As Long = -9999999

' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\development\Source\DSMapBook\DSMapPage.cls"

Private Sub Class_Initialize()
46:   Set m_PageItemColl = New Collection

  'Initialize Data Graph size properties
49:   m_bHasDataGraphPosition = False
50:   m_dDataGraphXMin = 0
51:   m_dDataGraphYMin = 0
52:   m_dDataGraphWidth = 0
53:   m_dDataGraphHeight = 0
End Sub

Private Sub Class_Terminate()
'MsgBox "DSMapPage: Starting Class Terminate"
57:   Set m_PageItemColl = Nothing
'MsgBox "DSMapPage: Ending Class Terminate"
End Sub

Private Sub IDSMapPage_AddPageItem(ByVal PageItem As IElement)
61:   m_PageItemColl.Add PageItem
End Sub

Private Sub IDSMapPage_DrawPage(pDoc As IMxDocument, pDSMapSeries As IDSMapSeries, bRefreshFlag As Boolean)
On Error GoTo ErrHand
  Dim pMap As IMap, lLoop As Long, pEnv As IEnvelope, lIndex As Long
  Dim pFeatLayer As IFeatureLayer, pActive As IActiveView, pTempLayer As ILayer
  Dim pGraphicsCont As IGraphicsContainer, pTempColl As Collection, pElemProps As IElementProperties
  Dim pTextSym As ISimpleTextSymbol, pClone As IClone, pSeriesOpts2 As IDSMapSeriesOptions2
  Dim pSeriesProps As IDSMapSeriesProps, pSeriesOpts As IDSMapSeriesOptions
  Dim pSeriesOpts3 As IDSMapSeriesOptions3
72:   Set pSeriesProps = pDSMapSeries
  
  'Check if pDSMapSeries is nothing
75:   If pDSMapSeries Is Nothing Then
76:     MsgBox "Warning: pDSMapSeries is Nothing, in IDSMapPage_DrawPage"
    Exit Sub
78:   End If
  
  'Find the data frame
81:   Set pMap = FindDataFrame(pDoc, pSeriesProps.DataFrameName)
82:   If pMap Is Nothing Then
83:     MsgBox "Could not find map in DrawPage routine!!!"
    Exit Sub
85:   End If
   
'MsgBox "pSeriesProps.IndexLayerName: " & pSeriesProps.IndexLayerName

  'Find the Index layer
90:   Set pFeatLayer = GetFeatureLayerByName(pSeriesProps.IndexLayerName, pMap)
91:   If pFeatLayer Is Nothing Then
92:     MsgBox "Could not find index layer in DrawPage routine!!!"
    Exit Sub
94:   End If
  
  'Switch to the Layout view if we are not already there
97:   If Not TypeOf pDoc.ActiveView Is IPageLayout Then
98:     Set pDoc.ActiveView = pDoc.PageLayout
99:   End If
  
  'Remove any previous neighbor labels.
102:   Set pGraphicsCont = pDoc.ActiveView
103:   pGraphicsCont.Reset
104:   Set pTempColl = New Collection
105:   Set pElemProps = pGraphicsCont.Next
106:   Do While Not pElemProps Is Nothing
107:     If pElemProps.name = "DSMAPBOOK TEXT" Then
108:       pTempColl.Add pElemProps
109:     End If
110:     Set pElemProps = pGraphicsCont.Next
111:   Loop
112:   For lLoop = 1 To pTempColl.Count
113:     pGraphicsCont.DeleteElement pTempColl.Item(lLoop)
114:   Next lLoop
115:   Set pTempColl = Nothing
  
  'Rotate the frame if necessary
118:   Set pActive = pMap
119:   Set pSeriesOpts = pSeriesProps
120:   Set pSeriesOpts2 = pSeriesOpts
121:   If pSeriesOpts.RotateFrame Then
'    If m_dPageRotation > 0 Then
123:       pActive.ScreenDisplay.DisplayTransformation.Rotation = m_dPageRotation
'    End If
125:   End If
  
  'Set the extent and possibly the scale for the map
128:   SetMapExtent pSeriesOpts, pActive
    
  'Set the clip property
  'Updated 6/18/03 to support cross hatching of area outside the clip
  Select Case pSeriesOpts2.ClipData
  Case 0   'No clipping
'    pMap.ClipGeometry = Nothing
  Case 1   'Clipping only
136:     pMap.ClipGeometry = m_pPageShape
  Case 2   'clipping with cross hatching of area outside the clip
138:     pMap.ClipGeometry = Nothing
139:     CreateClipElement pDoc, pActive, pFeatLayer
140:   End Select
  
  'Check for indicator maps and update those also
143:   RefreshIndicators pDoc, pSeriesProps, bRefreshFlag
  
  'Check for Date and Title elements
146:   UpdateTaggedElements pDoc, m_sPageName, bRefreshFlag, pDSMapSeries
  
  'Label neighboring tiles if necessary
149:   If pSeriesOpts.LabelNeighbors Then
150:     Set pClone = pSeriesOpts.LabelSymbol
151:     Set pTextSym = pClone.Clone
152:     lIndex = pFeatLayer.FeatureClass.FindField(pSeriesProps.IndexFieldName)
153:     If lIndex >= 0 Then
154:       LabelNeighbors pDoc, pFeatLayer, pTextSym, lIndex, pSeriesProps.DataFrameName
155:     End If
156:   End If
  
   '--------------------------------
  ' DetailExtension:
  '
  ' Update detail maps by executing "Create maps" button.
  ' Added 6/17/2004 to support inset creation from a separate tool.
  '
  Dim pDocument As IDocument
  Dim pUID As IUID
  Dim pCommandItem As ICommandItem
  Dim pCommand As ICommand, bFlag As Boolean
  
169:   Set pUID = New UID
170:   bFlag = False
On Error GoTo NoDetails
172:   pUID.Value = "DetailAreaExt.CreateDetailsCmd"
On Error GoTo ErrHand
   
175:    If Not bFlag Then
176:     Set pDocument = pDoc
177:     Set pCommandItem = pDocument.CommandBars.Find(pUID)
178:     If Not pCommandItem Is Nothing Then
179:       If TypeOf pCommandItem Is ICommand Then
180:         Set pCommand = pCommandItem
181:         If pCommand.Enabled Then
182:           pCommandItem.Execute
183:         End If
184:       End If
185:     End If
186:   End If
  
  '  End of DetailExtension additions
  '-------------------------------
  
  'Select the tile if this option is selected - Added 11/23/2004 by LY
  Dim pFeatSel As IFeatureSelection, pQuery As IQueryFilter
193:   Set pSeriesOpts3 = pDSMapSeries
194:   If pSeriesOpts3.SelectTile Then
195:     Set pQuery = New QueryFilter
196:     pQuery.WhereClause = pSeriesProps.IndexFieldName & " = '" & m_sPageName & "'"
197:     Set pFeatSel = pFeatLayer
198:     pFeatSel.SelectFeatures pQuery, esriSelectionResultNew, True
199:   End If
  '-------------------------------------------------------------------------
  
202:   If bRefreshFlag Then
203:     pDoc.ActiveView.Refresh
204:   End If
  Exit Sub
  
NoDetails:
208:   bFlag = True
209:   Resume Next
  
ErrHand:
212:   MsgBox "IDSMapPage_DrawPage - " & Erl & " - " & Err.Description
End Sub

Private Property Let IDSMapPage_FromStation(RHS As Double)
216:     m_lFromStat = RHS
End Property

Private Property Get IDSMapPage_FromStation() As Double
220:     IDSMapPage_FromStation = m_lFromStat
End Property

Private Function IDSMapPage_IndexPage(pIndexLayer As IFeatureLayer, sFieldName As String) As Collection
On Error GoTo ErrHand
  Dim pFilter As ISpatialFilter, pIndex As Collection, lFieldIndex As Long
  Dim pCursor As IFeatureCursor, pFeat As IFeature, sValue As String, lLoop As Long
  
  'Check for a valid index layer
229:   Set IDSMapPage_IndexPage = Nothing
230:   If pIndexLayer Is Nothing Then
231:     MsgBox "You did not send a valid index layer to the IndexPage function!!"
    Exit Function
233:   End If
  
  'Check for a valid field name
236:   If sFieldName = "" Or sFieldName = " " Then
237:     MsgBox "You did not send a valid field name to the IndexPage function!!"
    Exit Function
239:   End If
  
  'Check for field name in the layer
242:   lFieldIndex = pIndexLayer.FeatureClass.FindField(sFieldName)
243:   If lFieldIndex < 0 Then
244:     MsgBox "Could not find field name in the index layer of the IndexPage function!!"
    Exit Function
246:   End If
  
  'Perform the query of the index layer using the page shape
249:   Set pFilter = New SpatialFilter
250:   pFilter.AddField sFieldName
251:   pFilter.WhereClause = sFieldName & " is not null"
'237:   pFilter.WhereClause = sFieldName & " <> '' and " & sFieldName & " <> ' '"
253:   Set pFilter.Geometry = m_pPageShape
254:   pFilter.GeometryField = pIndexLayer.FeatureClass.ShapeFieldName
255:   pFilter.SpatialRel = esriSpatialRelIntersects
256:   Set pCursor = pIndexLayer.Search(pFilter, True)
  
  'Populate the collection with the results of the query
259:   Set pIndex = New Collection
260:   Set pFeat = pCursor.NextFeature
261:   Do While Not pFeat Is Nothing
262:     sValue = pFeat.Value(lFieldIndex)
263:     If pIndex.Count > 0 Then
264:       For lLoop = 1 To pIndex.Count
265:         If sValue < pIndex.Item(lLoop) Then
266:           pIndex.Add sValue, sValue, lLoop
267:           Exit For
268:         ElseIf sValue = pIndex.Item(lLoop) Then
269:           Exit For
270:         End If
271:         If lLoop = pIndex.Count Then
272:           pIndex.Add sValue, sValue
273:         End If
274:       Next lLoop
275:     Else
276:       pIndex.Add sValue, sValue
277:     End If
    
279:     Set pFeat = pCursor.NextFeature
280:   Loop
  
282:   Set IDSMapPage_IndexPage = pIndex

  Exit Function
  
ErrHand:
287:   MsgBox "IDSMapPage_IndexPage - " & Err.Description
End Function

Private Property Let IDSMapPage_LastOutputted(RHS As Date)
291:  m_dLastOutputted = RHS
End Property

Private Property Get IDSMapPage_LastOutputted() As Date
295:   IDSMapPage_LastOutputted = m_dLastOutputted
End Property

Private Property Get IDSMapPage_PageItem(index As Long) As IElement
299:   If index > -1 And index < m_PageItemColl.Count Then
300:     Set IDSMapPage_PageItem = m_PageItemColl.Item(index + 1)
301:   Else
302:     Set IDSMapPage_PageItem = Nothing
303:   End If
End Property

Private Property Get IDSMapPage_PageItemCount() As Long
307:   IDSMapPage_PageItemCount = m_PageItemColl.Count
End Property

Private Property Let IDSMapPage_PageName(RHS As String)
311:   m_sPageName = RHS
End Property

Private Property Get IDSMapPage_PageName() As String
315:   IDSMapPage_PageName = m_sPageName
End Property

Private Property Let IDSMapPage_EnablePage(RHS As Boolean)
319:   m_bPrintPage = RHS
End Property

Private Property Get IDSMapPage_EnablePage() As Boolean
323:   IDSMapPage_EnablePage = m_bPrintPage
End Property

Private Property Let IDSMapPage_PageNumber(RHS As Long)
327:   m_lPageNumber = RHS
End Property

Private Property Get IDSMapPage_PageNumber() As Long
331:   IDSMapPage_PageNumber = m_lPageNumber
End Property

Private Property Let IDSMapPage_PageRotation(RHS As Double)
335:   m_dPageRotation = RHS
End Property

Private Property Get IDSMapPage_PageRotation() As Double
339:   IDSMapPage_PageRotation = m_dPageRotation
End Property

Private Property Let IDSMapPage_PageScale(RHS As Double)
343:   m_dPageScale = RHS
End Property

Private Property Get IDSMapPage_PageScale() As Double
347:   IDSMapPage_PageScale = m_dPageScale
End Property

Private Property Set IDSMapPage_PageShape(RHS As IPolygon)
351:   Set m_pPageShape = RHS
End Property

Private Property Get IDSMapPage_PageShape() As IPolygon
355:   Set IDSMapPage_PageShape = m_pPageShape
End Property

Private Sub IDSMapPage_RemovePageItem(index As Long)
359:   If index > -1 And index < m_PageItemColl.Count Then
360:     m_PageItemColl.Remove index + 1
361:   End If
End Sub

Private Property Let IDSMapPage_routeLine(RHS As esriGeometry.IPolyline)
365:     Set m_routePolyline = RHS
End Property

Private Property Let IDSMapPage_ToStation(RHS As Double)
369:     m_lToStat = RHS
End Property

Private Property Get IDSMapPage_ToStation() As Double
373:     IDSMapPage_ToStation = m_lToStat
End Property

Private Property Let IDSMapSeriesGraphProps_DataGraphHeight(RHS As Double)
377:   m_dDataGraphHeight = RHS
End Property

Private Property Get IDSMapSeriesGraphProps_DataGraphHeight() As Double
381:   IDSMapSeriesGraphProps_DataGraphHeight = m_dDataGraphHeight
End Property

Private Property Let IDSMapSeriesGraphProps_DataGraphWidth(RHS As Double)
385:   m_dDataGraphWidth = RHS
End Property

Private Property Get IDSMapSeriesGraphProps_DataGraphWidth() As Double
389:   IDSMapSeriesGraphProps_DataGraphWidth = m_dDataGraphWidth
End Property

Private Property Let IDSMapSeriesGraphProps_DataGraphXMin(RHS As Double)
393:   m_dDataGraphXMin = RHS
End Property

Private Property Get IDSMapSeriesGraphProps_DataGraphXMin() As Double
397:   IDSMapSeriesGraphProps_DataGraphXMin = m_dDataGraphXMin
End Property

Private Property Let IDSMapSeriesGraphProps_DataGraphYMin(RHS As Double)
401:   m_dDataGraphYMin = RHS
End Property

Private Property Get IDSMapSeriesGraphProps_DataGraphYMin() As Double
405:   IDSMapSeriesGraphProps_DataGraphYMin = m_dDataGraphYMin
End Property

Private Property Let IDSMapSeriesGraphProps_HasDataGraphPosition(RHS As Boolean)
409:   m_bHasDataGraphPosition = RHS
End Property

Private Property Get IDSMapSeriesGraphProps_HasDataGraphPosition() As Boolean
413:   IDSMapSeriesGraphProps_HasDataGraphPosition = m_bHasDataGraphPosition
End Property

Private Property Get IPersistVariant_ID() As esriSystem.IUID
  Dim id As New UID
418:   id = "JAMB_Back_End.DSMapPage"
419:   Set IPersistVariant_ID = id
End Property

Private Sub IPersistVariant_Load(ByVal Stream As esriSystem.IVariantStream)
'Load the persisted parameters of the renderer
On Error GoTo ErrHand
'MsgBox "Starting DSMapPage.IPersistVariant_Load"
  Dim lLoop As Long, lCount As Long, pElem As IElement, sFirstItem As String
  Dim lPropCount As Long

  'Added 2/18/04 to make the list of persisted properties more dynamic
430:   sFirstItem = Stream.Read
'MsgBox "sFirstItem: " + CStr(sFirstItem)
432:   If UCase(Left(sFirstItem, 18)) = "PAGEPROPERTYCOUNT-" Then
433:     lPropCount = Mid(sFirstItem, 19) - 1 'Subtract one from property count
434:     m_sPageName = Stream.Read
435:   Else
436:     lPropCount = 5
437:     m_sPageName = sFirstItem
438:   End If
'MsgBox "Map Page - lPropCount: " & lPropCount & vbNewLine _
'& "sFirstItem: " & sFirstItem
'MsgBox "lPropCount: " + CStr(lPropCount)
'MsgBox "m_sPageName: " + CStr(m_sPageName)
  
  'Raise an error if property count is less than 1.
445:   If lPropCount < 1 Then Err.Raise 0, "", "Property count was less than 1."
  
  'Read all of the properties into a collection
  Dim Props As Collection, lx As Long
449:   Set Props = New Collection
450:   For lx = 1 To lPropCount
451:     Props.Add Stream.Read
452:   Next lx

  'Original page properties
455:   If lPropCount >= 5 Then
456:     m_bPrintPage = Props.Item(1)
457:     m_dPageRotation = Props.Item(2)
458:     m_dPageScale = Props.Item(3)
459:     m_dLastOutputted = Props.Item(4)
460:     Set m_pPageShape = Props.Item(5)
461:   End If

  'Additional properties added after 2/18/04
464:   If lPropCount >= 6 Then    'Checking for page number
465:     m_lPageNumber = Props.Item(6)
466:   Else
467:     m_lPageNumber = -1
468:   End If
'MsgBox "m_lPageNumber: " + CStr(m_lPageNumber)

  'Additional properties added to track FROM and TO station values.
472:   If lPropCount >= 8 Then
473:       m_lFromStat = Props.Item(7)
474:       m_lToStat = Props.Item(8)
475:   End If
'MsgBox "m_lFromStat: " + CStr(m_lFromStat)
'MsgBox "m_lToStat: " + CStr(m_lToStat)

  'Read the 5 new properties that were added 2/11/2009
  'Current Page Number and Data Graph Properties added 02/11/2009
481:   If lPropCount >= 13 Then
'MsgBox "Map Page: lPropCount >= 13"
483:     m_bHasDataGraphPosition = Props.Item(9)
484:     m_dDataGraphXMin = Props.Item(10)
485:     m_dDataGraphYMin = Props.Item(11)
486:     m_dDataGraphWidth = Props.Item(12)
487:     m_dDataGraphHeight = Props.Item(13)
488:   Else
'MsgBox "Map Page: NOT TRUE: lPropCount >= 13"
490:     m_bHasDataGraphPosition = False
491:     m_dDataGraphXMin = 0
492:     m_dDataGraphYMin = 0
493:     m_dDataGraphWidth = 0
494:     m_dDataGraphHeight = 0
495:   End If

'  'Added two parameters for testing purposes
'561:   If lPropCount >= 15 Then
'    Dim sFirstTest As String, sSecondTest As String
'563:     sFirstTest = Props.Item(14)
'564:     sSecondTest = Props.Item(15)
'565:     MsgBox "First Test: " & sFirstTest & vbNewLine & "Second Test: " & sSecondTest
'566:   Else
'567:     MsgBox "The two test parameters were not found."
'568:   End If

  'More original properties.  Writen out below the new properties because they are of variable length
508:   lCount = Stream.Read
509:   If lCount > 0 Then
510:     For lLoop = 1 To lCount
511:       Set pElem = Stream.Read
512:       m_PageItemColl.Add pElem
513:     Next lLoop
514:   End If
'MsgBox "Ending DSMapPage.IPersistVariant_Load without errors."
  Exit Sub
ErrHand:
518:   MsgBox "MapPage - IPersistVariant_Load - " & Erl & " - " & Err.Description
End Sub

Private Sub IPersistVariant_Save(ByVal Stream As esriSystem.IVariantStream)
'Write it all out
On Error GoTo ErrHand
  Dim lLoop As Long
  'Added 2/18/04 to make the list of persisted properties more dynamic
  'Increased to 14 to record information about the Data Graph (added 02/11/2009)
  'Moved the extra properties to the end 2/17/2009
  'Stream.Write "PAGEPROPERTYCOUNT-9"
529:   Stream.Write "PAGEPROPERTYCOUNT-14"
'690:   Stream.Write "PAGEPROPERTYCOUNT-16" 'Increase by 2 to include 2 test variables.

532:   Stream.Write m_sPageName
533:   Stream.Write m_bPrintPage
534:   Stream.Write m_dPageRotation
535:   Stream.Write m_dPageScale
536:   Stream.Write m_dLastOutputted
537:   Stream.Write m_pPageShape
538:   Stream.Write m_lPageNumber   'Added 2/18/04
              
  'Additional properties added to track FROM and TO station values.
541:   Stream.Write m_lFromStat
542:   Stream.Write m_lToStat
  
  'Record information about the Data Graph (added 02/11/2009)
545:   Stream.Write m_bHasDataGraphPosition
546:   Stream.Write m_dDataGraphXMin
547:   Stream.Write m_dDataGraphYMin
548:   Stream.Write m_dDataGraphWidth
549:   Stream.Write m_dDataGraphHeight
  
''Add extra test variables
'712: Stream.Write "test1"
'713: Stream.Write "test2"
  
555:   Stream.Write m_PageItemColl.Count
556:   If m_PageItemColl.Count > 0 Then
557:     For lLoop = 1 To m_PageItemColl.Count
558:       Stream.Write m_PageItemColl.Item(lLoop)
559:     Next lLoop
560:   End If
    
  Exit Sub
ErrHand:
564:   MsgBox "MapPage - IPersistVariant_Save - " & Err.Description
End Sub

Private Sub LabelNeighbors(pDoc As IMxDocument, pFLayer As IFeatureLayer, pTextSym As ISimpleTextSymbol, _
 lIndex As Long, sFrameName As String)
'Routine for loop through the tiles that are touching are selected tile
On Error GoTo ErrHand
  Dim pElem As IElement, pTextElem As ITextElement, pMap As IMap
  Dim pGraphSel As IGraphicsContainerSelect
  Dim pSpatial As ISpatialFilter, pFeatCursor As IFeatureCursor
  Dim pFeats As IFeature, pActive As IActiveView, sText As String
575:   Set pMap = pDoc.FocusMap
576:   Set pActive = pDoc.ActiveView
  
578:   Set pSpatial = New SpatialFilter
579:   Set pSpatial.Geometry = m_pPageShape
580:   pSpatial.GeometryField = pFLayer.FeatureClass.ShapeFieldName
581:   pSpatial.SpatialRel = esriSpatialRelTouches
582:   Set pFeatCursor = pFLayer.Search(pSpatial, False)
583:   Set pFeats = pFeatCursor.NextFeature
584:   Do While Not pFeats Is Nothing
    'If there is a value for the selected adjacent tile, then get it and label the feature
586:     If Not IsNull(pFeats.Value(lIndex)) Then
587:       sText = pFeats.Value(lIndex)
588:       LabelAdjacent pFeats, pMap, pActive, pTextSym, sText, sFrameName
589:     End If
590:     Set pFeats = pFeatCursor.NextFeature
591:   Loop
592:   Set pGraphSel = pActive
593:   pGraphSel.UnselectAllElements

  Exit Sub
ErrHand:
597:   MsgBox "LabelNeighbors - " & Err.Description
End Sub

Sub LabelAdjacent(pLabelFeat As IFeature, pMap As IMap, pActive As IActiveView, pTextSym As ITextSymbol, _
 sText As String, sFrameName As String)
'Routine for labeling the outside edge of our data frame based on the relative
'position of the tile feature being sent in (pLabelFeat) to the selected tile (pIndexFeat)
'The relative position is determined by finding the midpoint of the common line between
'the selected tile and the tile to be labeled.  The midpoint is then combined with the
'center of the extent of the map to create a line that is extended to the edge of the
'map extent.  The location along the map extent is then extrapolated out to a point on
'the boundary of our map frame.  This location is then labeled with the name of the tile.
On Error GoTo ErrHand
  Dim pCommonGeom As IGeometry, pTopoOp As ITopologicalOperator, pPolyline2 As IPolyline
  Dim pMidPt As IPoint, pPolyLine As IPolyline, pEnv As IEnvelope, pTmpFrame As IMapFrame
  Dim pCenterPt As IPoint, pMapView As IActiveView, pMulti As IPointCollection
  Dim pGraph As IGraphicsContainer, lLoop As Long, pElem As IElement
  Dim pElemProps As IElementProperties, pTrans2d As ITransform2D
  Dim pPt As IConstructPoint, pNewPt As IPoint, pTextElem As ITextElement
  Dim pMapFrame As IMapFrame, pMapEnv As IEnvelope, pFramePoly As IPointCollection
  Dim pLine As ILine, dDist As Double, iSeg As Integer, pEndPt As IPoint
  Dim pProx As IProximityOperator, dTmpDist As Double, pCurve2 As ICurve
  Dim pOutPt As IPoint, dAlong As Double, dFrom As Double, bSide As Boolean
  Dim pPoints As IPointCollection, pPoints2 As IPointCollection, dAngle As Double
  Dim pIntPoints As IPointCollection, pIntTopo As ITopologicalOperator

623:   Set pTopoOp = m_pPageShape
624:   Set pCommonGeom = pTopoOp.Intersect(pLabelFeat.Shape, esriGeometry1Dimension)
625:   If pCommonGeom.IsEmpty Then
626:     Set pCommonGeom = pTopoOp.Intersect(pLabelFeat.Shape, esriGeometry0Dimension)
627:     Set pMulti = pCommonGeom
628:     Set pMidPt = pMulti.Point(0)
629:   Else
630:     Set pPolyLine = pCommonGeom
631:     Set pMidPt = New esriGeometry.Point
632:     pPolyLine.QueryPoint esriNoExtension, 0.5, True, pMidPt
633:   End If
    
  'Find center point of map frame
636:   Set pCenterPt = New esriGeometry.Point
637:   Set pMapView = pMap
638:   Set pEnv = pMapView.Extent
639:   pCenterPt.X = pEnv.XMin + ((pEnv.XMax - pEnv.XMin) / 2)
640:   pCenterPt.Y = pEnv.YMin + ((pEnv.YMax - pEnv.YMin) / 2)

  'Get the geometry of the map frame
643:   Set pGraph = pActive
644:   pGraph.Reset
645:   Set pElem = pGraph.Next
646:   Do While Not pElem Is Nothing
647:     If TypeOf pElem Is IMapFrame Then
648:       Set pTmpFrame = pElem
649:       If pTmpFrame.Map.name = sFrameName Then
650:         Set pMapFrame = pElem
651:         Exit Do
652:       End If
653:     End If
654:     Set pElem = pGraph.Next
655:   Loop
  If pMapFrame Is Nothing Then Exit Sub
  
658:   Set pMapEnv = pMapFrame.MapBounds
659:   Set pFramePoly = pElem.Geometry
  
  'Create curves and intersect them
662:   Set pPoints = New Polyline
663:   pPoints.AddPoint pMapEnv.LowerLeft
664:   pPoints.AddPoint pMapEnv.LowerRight
665:   pPoints.AddPoint pMapEnv.UpperRight
666:   pPoints.AddPoint pMapEnv.UpperLeft
667:   pPoints.AddPoint pMapEnv.LowerLeft
  
669:   Set pPoints2 = RotatedAndExtendedLine(pCenterPt, pMidPt)
  'If for some reason nothing is returned, go with a rotation of 0
  If pPoints2 Is Nothing Then Exit Sub
672:   Set pPolyline2 = pPoints2
  If pPolyline2.IsEmpty Then Exit Sub
    
  'Find the intersection point of the line we created and the map extent boundary
676:   Set pIntTopo = pPoints2
677:   Set pIntPoints = pIntTopo.Intersect(pPoints, esriGeometry0Dimension)
  If pIntPoints.PointCount = 0 Then Exit Sub
  
680:   Set pEndPt = pIntPoints.Point(0)
  
  'Extrapolate the point on the extent to a point on the outside of the map frame
  'Figure out which segment we are closest to
684:   Set pProx = pEndPt
685:   dDist = 999999
686:   iSeg = -1
687:   For lLoop = 0 To 3
688:     Set pLine = New esriGeometry.Line
    Select Case lLoop
    Case 0
691:       pLine.PutCoords pMapEnv.LowerLeft, pMapEnv.UpperLeft
    Case 1
693:       pLine.PutCoords pMapEnv.UpperLeft, pMapEnv.UpperRight
    Case 2
695:       pLine.PutCoords pMapEnv.UpperRight, pMapEnv.LowerRight
    Case Else
697:       pLine.PutCoords pMapEnv.LowerRight, pMapEnv.LowerLeft
698:     End Select
    
700:     dTmpDist = pProx.ReturnDistance(pLine)
701:     If dTmpDist < dDist Then
702:       dDist = dTmpDist
703:       iSeg = lLoop
704:       Set pCurve2 = pLine
705:     End If
706:   Next lLoop
707:   Set pOutPt = New esriGeometry.Point
708:   pCurve2.QueryPointAndDistance esriNoExtension, pEndPt, True, pOutPt, dAlong, dFrom, bSide
  
  'We know have the segment and ratio length on that segment, so we can transfer that
  'information to the frame geometry and find the corresponding point there
712:   Set pPt = New esriGeometry.Point
713:   Set pLine = New esriGeometry.Line
  Select Case iSeg
  Case 0
716:     pLine.PutCoords pFramePoly.Point(0), pFramePoly.Point(1)
717:     pTextSym.HorizontalAlignment = esriTHACenter
718:     pTextSym.VerticalAlignment = esriTVABottom
  Case 1
720:     pLine.PutCoords pFramePoly.Point(1), pFramePoly.Point(2)
721:     pTextSym.HorizontalAlignment = esriTHACenter
722:     pTextSym.VerticalAlignment = esriTVABottom
  Case 2
724:     pLine.PutCoords pFramePoly.Point(2), pFramePoly.Point(3)
725:     pTextSym.HorizontalAlignment = esriTHACenter
726:     pTextSym.VerticalAlignment = esriTVABottom
  Case 3
728:     pLine.PutCoords pFramePoly.Point(3), pFramePoly.Point(0)
729:     pTextSym.HorizontalAlignment = esriTHACenter
730:     pTextSym.VerticalAlignment = esriTVATop
731:   End Select
732:   pPt.ConstructAlong pLine, esriNoExtension, dAlong, True
733:   Set pNewPt = pPt
  
  'Now that we have a point along the data frame, we can place the label based on
  'that point and which side of the frame it is on
737:   Set pTextElem = New TextElement
738:   Set pElem = pTextElem
739:   pTextElem.Symbol = pTextSym
740:   pElem.Geometry = pNewPt
741:   Set pElemProps = pElem
742:   pElemProps.name = "DSMAPBOOK TEXT"
743:   pTextElem.Text = sText
744:   Set pTrans2d = pTextElem
  Select Case iSeg
  Case 0
747:     dAngle = 90 * (3.14159265358979 / 180)
  Case 1
749:     dAngle = 0
  Case 2
751:     dAngle = 270 * (3.14159265358979 / 180)
  Case 3
753:     dAngle = 0
754:   End Select
755:   pTrans2d.Rotate pNewPt, dAngle
756:   pGraph.AddElement pElem, 0
  
  Exit Sub
ErrHand:
760:   MsgBox "LabelAdjacent - " & Err.Description
End Sub

Private Function RotatedAndExtendedLine(pCenterPt As IPoint, pMidPt As IPoint) As IPolyline
On Error GoTo ErrHand
  Dim pPoints As IPointCollection, pPolyLine As IPolyline, pLine As ILine, pNewPt As IConstructPoint
  Dim dOrigAngle As Double, dNewAngle As Double, dLength As Double, dRadAngle As Double
  Dim pNewPoints As IPointCollection, pNewPoint As IConstructPoint, dA As Double
  
  'Create a line so we can get the current angle and distance
770:   Set pLine = New esriGeometry.Line
771:   pLine.PutCoords pCenterPt, pMidPt
772:   dLength = pLine.Length
  
774:   If m_dPageRotation = 0 Then
    'Create another point at the same angle to make sure our line crosses the extent boundary
776:     Set pNewPt = New esriGeometry.Point
777:     pNewPt.ConstructAngleDistance pMidPt, pLine.Angle, dLength * 100
778:     Set pPoints = New Polyline
779:     pPoints.AddPoint pCenterPt
780:     pPoints.AddPoint pMidPt
781:     pPoints.AddPoint pNewPt
782:     Set RotatedAndExtendedLine = pPoints
    Exit Function
784:   End If
  
  'If the page is rotated, then we have to rotate the labeling of adjacent tiles also
787:   dOrigAngle = pLine.Angle * (180 / 3.14159265358979)
788:   dA = dOrigAngle
789:   If dOrigAngle < 0 Then
790:     dOrigAngle = 360 - Abs(dOrigAngle)
791:   End If
792:   dNewAngle = dOrigAngle + m_dPageRotation
793:   If dNewAngle >= 360 Then
794:     dNewAngle = dNewAngle - 360
795:   End If
796:   dRadAngle = dNewAngle * (3.14159265358979 / 180)
  
  'Make a new esrigeometry.line at the rotated angle we just calculated.  The new esrigeometry.line is made shorter than the original
  'to ensure the line does not extend past the map bounds we need to intersect it with in the next stage
800:   Set pNewPoint = New esriGeometry.Point
801:   Set pNewPoints = New Polyline
802:   pNewPoint.ConstructAngleDistance pCenterPt, dRadAngle, dLength * 100
803:   pNewPoints.AddPoint pCenterPt
804:   pNewPoints.AddPoint pNewPoint
805:   Set RotatedAndExtendedLine = pNewPoints
  
  Exit Function
ErrHand:
809:   MsgBox "RotatedLine - " & Err.Description
End Function

Private Sub RefreshIndicators(pDoc As IMxDocument, pSeriesProps As IDSMapSeriesProps, bRefreshFlag As Boolean)
'Routine for updating any identicator maps there might be
On Error GoTo ErrHand
  Dim pGridLayer As IFeatureLayer, pGridSel As IFeatureSelection
  Dim lLoop As Long, pActive As IActiveView, pSpatial As ISpatialFilter
  Dim pFeature As IFeature, pCursor As IFeatureCursor, pEnv As IEnvelope
  Dim pQuery As IQueryFilter, lLoop2 As Long, pMap As IMap, pSelEvents As ISelectionEvents

  'Check for indicator maps and update those also
821:   For lLoop = 0 To pDoc.Maps.Count - 1
822:     If pDoc.Maps.Item(lLoop).name = "Global Indicator" Or pDoc.Maps.Item(lLoop).name = "Local Indicator" Then
823:       Set pMap = pDoc.Maps.Item(lLoop)
      'Find the Index layer
825:       For lLoop2 = 0 To pMap.LayerCount - 1
826:         If pMap.Layer(lLoop2).name = "Identifier Layer" Then
827:           Set pGridLayer = pMap.Layer(lLoop2)
828:           Exit For
829:         End If
830:       Next lLoop2
831:       If pGridLayer Is Nothing Then
832:         MsgBox "Could not find layer called Identifier Layer, can not redraw " & pMap.name & " frame!!!"
        Exit Sub
834:       End If
      
'      Set pGridLayer = pDoc.Maps.Item(lLoop).Layer(0)
837:       Set pGridSel = pGridLayer
838:       Set pQuery = New QueryFilter
839:       pQuery.WhereClause = pSeriesProps.IndexFieldName & " = '" & m_sPageName & "'"
840:       pGridSel.Clear
841:       pGridSel.SelectFeatures pQuery, esriSelectionResultNew, True
        
843:       If pMap.name = "Global Indicator" Then
844:         Set pActive = pDoc.Maps.Item(lLoop)
845:         If bRefreshFlag Then pActive.Refresh
846:       ElseIf pMap.name = "Local Indicator" Then
847:         Set pSpatial = New SpatialFilter
848:         Set pSpatial.Geometry = m_pPageShape
849:         pSpatial.GeometryField = pGridLayer.FeatureClass.ShapeFieldName
850:         pSpatial.SpatialRel = esriSpatialRelIntersects
851:         Set pCursor = pGridLayer.Search(pSpatial, False)
852:         Set pFeature = pCursor.NextFeature
853:         Do While Not pFeature Is Nothing
854:           If pEnv Is Nothing Then
855:             Set pEnv = pFeature.Shape.Envelope
856:           Else
857:             pEnv.Union pFeature.Shape.Envelope
858:           End If
859:           Set pFeature = pCursor.NextFeature
860:         Loop
861:         Set pActive = pMap
862:         pActive.Extent = pEnv
863:         If bRefreshFlag Then pActive.Refresh
864:       End If
865:       Set pSelEvents = pMap
866:       pSelEvents.SelectionChanged
      
868:       Set pGridLayer = Nothing
869:     End If
870:   Next lLoop

  Exit Sub
ErrHand:
874:   MsgBox "RefreshIndicators - " & Err.Description
End Sub

Private Sub UpdateTaggedElements(pDoc As IMxDocument, sTileName As String, bRefreshFlag As Boolean, _
 pDSMapSeries As IDSMapSeries)
'Routine for updating text elements tagged as Date or Title elements
On Error GoTo ErrHand
  'Check if pDSMapSeries is nothing
882:   If pDSMapSeries Is Nothing Then
883:     MsgBox "Warning: pDSMapSeries is Nothing, in UpdateTaggedElements"
    Exit Sub
885:   End If

  Dim pGraphicsCont As IGraphicsContainer, pElemProps As IElementProperties
  Dim pTextElement As ITextElement, pActive As IActiveView, pElem As IElement
  Dim pEnv As IEnvelope, pEnv2 As IEnvelope, sText As String, bUpdate As Boolean
890:   Set pGraphicsCont = pDoc.PageLayout
891:   Set pActive = pGraphicsCont
892:   pGraphicsCont.Reset
893:   Set pElemProps = pGraphicsCont.Next
894:   Do While Not pElemProps Is Nothing
895:     Set pElem = pElemProps
896:     bUpdate = False
        
    'If this is a text element
899:     If TypeOf pElemProps Is ITextElement Then
900:       bUpdate = True
      Select Case pElemProps.name
      Case TAG_CONST_DATE
903:         sText = Format(Date, "mmm dd, yyyy")
      Case TAG_CONST_TITLE
905:         sText = sTileName
      Case TAG_CONST_PAGENUMBER
907:         sText = CStr(m_lPageNumber)
      Case TAG_CONST_EXTRAITEM
909:         sText = GetExtraItemValue(pDoc, sTileName, pElemProps.Type, pDSMapSeries)
      Case Else
911:         bUpdate = False
912:       End Select
      'If the element name is recognized, update the text element
914:       If bUpdate Then
915:         Set pElem = pElemProps
916:         Set pEnv = New Envelope
917:         pElem.QueryBounds pActive.ScreenDisplay, pEnv
918:         Set pTextElement = pElemProps
919:         pTextElement.Text = sText
920:         pGraphicsCont.UpdateElement pTextElement
921:         Set pEnv2 = New Envelope
922:         pElem.QueryBounds pActive.ScreenDisplay, pEnv2
923:         pEnv.Union pEnv2
924:         If bRefreshFlag Then pActive.PartialRefresh esriViewGraphics, Nothing, pEnv
925:       End If 'End of if pUpdate
926:     End If 'End of if ITextElement
        
    'If this is a DataGraph Element
929:     If TypeOf pElemProps Is IDataGraphTElement Then 'If DataGraphT element
      'If this is the tagged DataGraphElement
931:       If pElemProps.Type = TAG_CONST_DATAGRAPH Then
        'Msgbox information about the DatagraphT Element
'        MsgBox "pElemProps.Name: " & pElemProps.Name & vbNewLine _
'        & "pElemProps.Type: " & pElemProps.Type
'        If pElemProps.CustomProperty Is Nothing Then
'          MsgBox "pElemProps.CustomProperty is nothing"
'        Else
'          MsgBox "pElemProps.CustomProperty is not nothing"
'          'MsgBox "CStr(pElemProps.CustomProperty: " & CStr(pElemProps.CustomProperty)
'        End If
      
        Dim dataGraphs As IDataGraphs
        Dim dataGraphElement As IDataGraphTElement
        Dim dataGraphAxisBottom As IDataGraphTAxisProperties
        Dim dataGraphAxisTop As IDataGraphTAxisProperties
        Dim dataGraph As IDataGraphT
        Dim sDataGraphFormatBottom As String
        Dim sDataGraphFormatTop As String
        
950:         Set dataGraphElement = pElemProps
951:         Set dataGraph = dataGraphElement.DataGraphT
952:         Set dataGraphAxisBottom = dataGraph.AxisProperties(2)
953:         Set dataGraphAxisTop = dataGraph.AxisProperties(3)
        
        'Update the graph by setting the To and From station
        'In order to avoid a bug in the ESRI DatagraphT Object,
        'we must set the minimum value to a dummy value that is GREATER THAN
        'the real minimum value that we want to set.  We first update then
        'graph with the dummy minimum value, then update again with the real
        'minumum value.  In this case, I chose to use the maximum value as my
        'dummy minimum value.
962:         dataGraphAxisBottom.AutomaticMaximum = False
963:         dataGraphAxisBottom.AutomaticMinimum = False
964:         dataGraphAxisBottom.Minimum = m_lToStat
965:         dataGraphAxisBottom.Maximum = m_lToStat
966:         dataGraphAxisTop.AutomaticMaximum = False
967:         dataGraphAxisTop.AutomaticMinimum = False
968:         dataGraphAxisTop.Minimum = m_lToStat
969:         dataGraphAxisTop.Maximum = m_lToStat
        'Update the data graph
        Dim pCancel As ITrackCancel
972:         Set pCancel = New CancelTracker
973:         pCancel.CancelOnClick = False
974:         pCancel.CancelOnKeyPress = False
975:         dataGraph.Update pCancel
        
        'Set the true minimum Station value for the top and bottom axis.
978:         dataGraphAxisTop.Minimum = m_lFromStat
979:         dataGraphAxisBottom.Minimum = m_lFromStat
        'Update the data graph
        Dim pCancel2 As ITrackCancel
982:         Set pCancel2 = New CancelTracker
983:         pCancel2.CancelOnClick = False
984:         pCancel2.CancelOnKeyPress = False
985:         dataGraph.Update pCancel2
        
        'Get the saved size and position.
        Dim XMin As Double, YMin As Double, Width As Double, Height As Double
        Dim bHasSavedPosition As Boolean
        Dim MSGP As IDSMapSeriesGraphProps
991:         Set MSGP = pDSMapSeries
                
        'If the current page has a saved size and position, get it.
994:         If m_bHasDataGraphPosition = True Then
995:           XMin = m_dDataGraphXMin
996:           YMin = m_dDataGraphYMin
997:           Width = m_dDataGraphWidth
998:           Height = m_dDataGraphHeight
999:           bHasSavedPosition = True
        'Otherwise, if the map series has saved graph size and position information, get it.
1001:         ElseIf MSGP.HasDataGraphPosition = True Then
1002:           XMin = MSGP.DataGraphXMin
1003:           YMin = MSGP.DataGraphYMin
1004:           Width = MSGP.DataGraphWidth
1005:           Height = MSGP.DataGraphHeight
1006:           bHasSavedPosition = True
'MsgBox "XMin, YMin, Width, Height: " & XMin & ", " & YMin & ", " & Width & ", " & Height
1008:         End If
        
        'If the map series has saved graph size and position information, move the graph to the appropriate location.
1011:         If bHasSavedPosition = True Then
1012:           MoveElementToPosition pElem, pActive, XMin, YMin, Width, Height
1013:         End If
1014:       End If
1015:     End If
              
    'If this element is tagged as "THIS PAGE ONLY"
1018:     If (pElemProps.name = TAG_CONST_THISPAGEONLY) _
    Or (pElemProps.name = TAG_CONST_ALLPAGESBUTTHIS) Then
1020:       bUpdate = True
      'Get the geometry
      Dim pElementTemp As IElement
      Dim pGeom As IGeometry
      Dim pTrans2d As ITransform2D
1025:       Set pElementTemp = pElemProps
1026:       Set pGeom = pElementTemp.Geometry
1027:       Set pTrans2d = pGeom

      'Determine if the element is hidden or visible.
      Dim bHidden As Boolean
1031:       bHidden = pElemProps.CustomProperty
      'Handle error case here, pElemProps.CustomProperty is not a boolean
      'If TypeOf pElemProps.CustomProperty Is blah Then
      
      'Determine if the page numbers match
      Dim bPageNumbersMatch As Boolean
1037:       bPageNumbersMatch = (pElemProps.Type = CStr(m_lPageNumber))
      
      'Determine if we are showing or hiding.
      Dim bShow As Boolean
1041:       bShow = False
1042:       If (pElemProps.name = TAG_CONST_THISPAGEONLY) _
      And (bPageNumbersMatch) Then bShow = True
1044:       If (pElemProps.name = TAG_CONST_ALLPAGESBUTTHIS) _
      And (bPageNumbersMatch = False) Then bShow = True
      
      'Hide or show the element.
1048:       If bShow Then 'Show the element
1049:         If bHidden Then 'If the element is hidden, show it
1050:           pTrans2d.Move (m_HIDE_LOCATION_X * -1), (m_HIDE_LOCATION_Y * -1)
1051:           pElemProps.CustomProperty = False
1052:         Else 'If the element is already visible, do nothing
1053:           bUpdate = False
1054:         End If
1055:       Else 'Hide the element off of the page
1056:         If bHidden Then 'If the element is already hidden, do nothing
1057:           bUpdate = False
1058:         Else ' If the elment is visible, hide it
1059:           pTrans2d.Move m_HIDE_LOCATION_X, m_HIDE_LOCATION_Y
1060:           pElemProps.CustomProperty = True
1061:         End If
1062:       End If
    
1064:       If bUpdate Then 'Update the element
1065:         Set pElem = pElemProps
1066:         Set pEnv = New Envelope
1067:         pElem.QueryBounds pActive.ScreenDisplay, pEnv
1068:         pElem.Geometry = pTrans2d
1069:         pGraphicsCont.UpdateElement pElemProps
1070:         Set pEnv2 = New Envelope
1071:         pElem.QueryBounds pActive.ScreenDisplay, pEnv2
1072:         pEnv.Union pEnv2
1073:         If bRefreshFlag Then pActive.PartialRefresh esriViewGraphics, Nothing, pEnv
1074:       End If 'End of if pUpdate
1075:     End If 'End of pElemProps.Name = TAG_CONST_THISPAGEONLY
    
    'Move to the next element
1078:     Set pElemProps = pGraphicsCont.Next
1079:   Loop

  Exit Sub
ErrHand:
1083:   MsgBox "UpdateTaggedElements - " & Erl & " - " & Err.Description
End Sub

Private Function GetExtraItemValue(pDoc As IMxDocument, sTileName As String, sFieldName As String, _
 pSeriesProps As IDSMapSeriesProps) As String
On Error GoTo ErrHand
  Dim pIndexLayer As IFeatureLayer, pQuery As IQueryFilter, pFCursor As IFeatureCursor
  Dim pFeat As IFeature, lIndex As Long, pMap As IMap, lIndex2 As Long
  
  'Find the data frame
1093:   Set pMap = FindDataFrame(pDoc, pSeriesProps.DataFrameName)
1094:   If pMap Is Nothing Then
1095:     MsgBox "(In GetExtraItem routine): Error, the main data frame '" & pSeriesProps.DataFrameName & "' was removed or renamed. " _
    & "Please rename the main data frame name back to its original name.", vbExclamation
1097:     GetExtraItemValue = "missing"
    Exit Function
1099:   End If
  
  'Find the Index layer
1102:   Set pIndexLayer = GetFeatureLayerByName(pSeriesProps.IndexLayerName, pMap)
1103:   If pIndexLayer Is Nothing Then
1104:     MsgBox "(In GetExtraItem routine): Error, the index layer '" & pSeriesProps.IndexLayerName & "' was removed, renamed, over moved to a different data frame. " _
      & "Please add the index layer back into the main data frame.", vbExclamation
1106:     GetExtraItemValue = "missing"
    Exit Function
1108:   End If
  
  'Find the field in the index layer
1111:   lIndex = pIndexLayer.FeatureClass.FindField(sFieldName)
1112:   If lIndex < 0 Then
1113:     MsgBox "Warning: I could not update the tagged text element because the '" & sFieldName & "' field is missing from " _
& "the index layer '" & pSeriesProps.IndexLayerName & "'.  Check to see if you removed or renamed this field.  To fix this error, " _
& " restore the field or un-tag the text element. I will change the text to read 'missing'.", vbExclamation
1116:     GetExtraItemValue = "missing"
    Exit Function
1118:   End If
  
  'Find the tile name field in the index layer
1121:   lIndex2 = pIndexLayer.FeatureClass.FindField(pSeriesProps.IndexFieldName)
1122:   If lIndex2 < 0 Then
1123:     MsgBox "Warning: I could not update the tagged text element because the '" & pSeriesProps.IndexFieldName & "' field is missing from " _
& "the index layer '" & pSeriesProps.IndexLayerName & "'.  Check to see if you removed or renamed this field.  To fix this error, " _
& " restore the field or un-tag the text element. I will change the text to read 'missing'.", vbExclamation
1126:     GetExtraItemValue = "missing"
    Exit Function
1128:   End If
  
  'Create the query object then select the appropriate tile from the index layer
1131:   Set pQuery = New QueryFilter
1132:   pQuery.WhereClause = pSeriesProps.IndexFieldName & " = '" & sTileName & "'"
1133:   Set pFCursor = pIndexLayer.Search(pQuery, False)
1134:   Set pFeat = pFCursor.NextFeature
1135:   If pFeat Is Nothing Then
1136:     MsgBox "Warning: I could not update the tagged text element because no features were found where '" & pSeriesProps.IndexFieldName & "' field is equal to " _
& "'" & sTileName & "' in the index layer '" & pSeriesProps.IndexLayerName & "'.  Check to see if you deleted or renamed the feature.  To fix this error, " _
& " restore the feature or un-tag the text element. I will change the text to read 'missing'.", vbExclamation
1139:     GetExtraItemValue = "missing"
    Exit Function
1141:   End If
  
  'Send back the value of the field
1144:   If IsNull(pFeat.Value(lIndex)) Then
1145:     GetExtraItemValue = " "
1146:   Else
1147:     GetExtraItemValue = pFeat.Value(lIndex)
1148:   End If

  Exit Function
ErrHand:
1152:   MsgBox "GetExtraItemValue - " & Erl & " - " & Err.Description
End Function

Private Sub CreateClipElement(pDoc As IMxDocument, pActive As IActiveView, _
 pFeatLayer As IFeatureLayer)
'Added 6/18/03 to support cross hatching of area outside the clip
On Error GoTo ErrHand
  Dim pPoly As IPolygon, pTopoOp As ITopologicalOperator
  Dim pGraphs As IGraphicsContainer, pElem As IElement, pNewElem As IElement
  Dim pNewPoly As IPointCollection, pElemProps As IElementProperties
  Dim pFinalGeom As IPolygon, pPoly2 As IPolygon, lLoop As Long
  
  'Search for an existing clip element and delete it when found
'  Set pGraphs = pDoc.FocusMap
1166:   Set pGraphs = pActive
1167:   pGraphs.Reset
1168:   Set pElemProps = pGraphs.Next
1169:   Do While Not pElemProps Is Nothing
1170:     If TypeOf pElemProps Is IPolygonElement Then
1171:       If UCase(pElemProps.name) = "DSMAPBOOK CLIP ELEMENT" Then
1172:         pGraphs.DeleteElement pElemProps
1173:         Exit Do
1174:       End If
1175:     End If
1176:     Set pElemProps = pGraphs.Next
1177:   Loop
  
1179:   Set pElem = New PolygonElement
1180:   Set pPoly = m_pPageShape
1181:   Set pNewElem = New PolygonElement
1182:   Set pNewPoly = New Polygon
1183:   pNewPoly.AddPoint pFeatLayer.AreaOfInterest.LowerLeft
1184:   pNewPoly.AddPoint pFeatLayer.AreaOfInterest.UpperLeft
1185:   pNewPoly.AddPoint pFeatLayer.AreaOfInterest.UpperRight
1186:   pNewPoly.AddPoint pFeatLayer.AreaOfInterest.LowerRight
1187:   Set pPoly2 = pNewPoly
1188:   pPoly2.Close
  
  Dim pLineSym As ISimpleLineSymbol, pLineFillSym As ILineFillSymbol
  Dim pFillShape As IFillShapeElement, pColor As IGrayColor
1192:   Set pColor = New GrayColor
1193:   pColor.Level = 150
1194:   Set pLineSym = New SimpleLineSymbol
1195:   pLineSym.Color = pColor
1196:   Set pLineFillSym = New LineFillSymbol
1197:   pLineFillSym.Angle = 45
1198:   pLineFillSym.Color = pColor
1199:   pLineFillSym.Outline = pLineSym
1200:   Set pLineFillSym.LineSymbol = pLineSym
1201:   pLineFillSym.Separation = 5
  
1203:   Set pTopoOp = pPoly2
1204:   Set pFinalGeom = pTopoOp.Difference(pPoly)
1205:   pNewElem.Geometry = pFinalGeom
1206:   Set pFillShape = pNewElem
1207:   pFillShape.Symbol = pLineFillSym
1208:   Set pElemProps = pFillShape
1209:   pElemProps.name = "DSMapBook Clip Element"
1210:   pGraphs.AddElement pNewElem, 0

  Exit Sub
ErrHand:
1214:   MsgBox "CreateClipElement - " & Erl & " - " & Err.Description
End Sub

Private Sub SetMapExtent(pSeriesOpts As IDSMapSeriesOptions, pActiveView As IActiveView)
On Error GoTo ErrHand
'Routine for calculating the extent of the tile to be displayed in the layout
  Dim dMult As Double, pEnv As IEnvelope, pMap As IMap
  
1222:   Set pMap = pActiveView
  Select Case pSeriesOpts.ExtentType
  Case 0  'Variable
1225:     If pSeriesOpts.Margin > 0 Then
1226:       Set pEnv = m_pPageShape.Envelope
      Select Case pSeriesOpts.MarginType
      Case 0  'Percent
1229:         dMult = 1 + (pSeriesOpts.Margin / 100)
1230:         pEnv.Expand dMult, dMult, True
      Case 1  'mapunits
1232:         pEnv.Expand pSeriesOpts.Margin, pSeriesOpts.Margin, False
1233:       End Select
1234:       pActiveView.Extent = pEnv
1235:     Else
1236:       pActiveView.Extent = m_pPageShape.Envelope
1237:     End If
  Case 1  'Fixed
1239:     pActiveView.Extent = m_pPageShape.Envelope
1240:     pMap.MapScale = pSeriesOpts.FixedScale
  Case 2  'DataDriven
1242:     pActiveView.Extent = m_pPageShape.Envelope
1243:     pMap.MapScale = m_dPageScale
1244:   End Select

  Exit Sub
ErrHand:
1248:   MsgBox "SetMapExtent - " & Err.Description
End Sub

Public Function FindDataFrame(pDoc As IMxDocument, sFrameName As String) As IMap
On Error GoTo ErrHand

1254:   Set FindDataFrame = modSharedFunctions.FindDataFrame(pDoc, sFrameName)

'  Dim lLoop As Long, pMap As IMap
'
'  'Find the data frame
'1248:   For lLoop = 0 To pDoc.Maps.Count - 1
'1249:     If pDoc.Maps.Item(lLoop).name = sFrameName Then
'1250:       Set pMap = pDoc.Maps.Item(lLoop)
'1251:       Exit For
'1252:     End If
'1253:   Next lLoop
'1254:   If Not pMap Is Nothing Then
'1255:     Set FindDataFrame = pMap
'1256:   End If

  Exit Function
ErrHand:
1271:   MsgBox "FindDataFrame - " & Err.Description
End Function

'Private Function CompositeLayer1(pCompLayer As ICompositeLayer, sIndexName As String) As IFeatureLayer
'On Error GoTo ErrHand
'  Dim lLoop As Long, pFeatLayer As IFeatureLayer
'1266:   For lLoop = 0 To pCompLayer.Count - 1
'1267:     If TypeOf pCompLayer.Layer(lLoop) Is ICompositeLayer Then
'1268:       Set pFeatLayer = CompositeLayer1(pCompLayer.Layer(lLoop), sIndexName)
'1269:       If Not pFeatLayer Is Nothing Then
'1270:         Set CompositeLayer1 = pFeatLayer
'        Exit Function
'1272:       End If
'1273:     Else
'1274:       If pCompLayer.Layer(lLoop).name = sIndexName Then
'1275:         Set CompositeLayer1 = pCompLayer.Layer(lLoop)
'        Exit Function
'1277:       End If
'1278:     End If
'1279:   Next lLoop
'
'1281:   Set CompositeLayer1 = Nothing
'
'  Exit Function
'ErrHand:
'1285:   MsgBox "CompositeLayer - " & Err.Description
'End Function

'Private Function FindLayer(sLayerName As String, pMap As IMap) As IFeatureLayer
'' Routine for finding a layer based on a name and then returning that layer as
'' a IFeatureLayer
'On Error GoTo ErrHand
'  Dim lLoop As Integer
'  Dim pFLayer As IFeatureLayer
'
'1305:   For lLoop = 0 To pMap.LayerCount - 1
'1306:     If TypeOf pMap.Layer(lLoop) Is ICompositeLayer Then
'1307:       Set pFLayer = FindCompositeLayer(pMap.Layer(lLoop), sLayerName, pMap)
'1308:       If Not pFLayer Is Nothing Then
'1309:         Set FindLayer = pFLayer
'        Exit Function
'1311:       End If
'1312:     ElseIf TypeOf pMap.Layer(lLoop) Is IFeatureLayer Then
'1313:       Set pFLayer = pMap.Layer(lLoop)
'1314:       If UCase(pFLayer.name) = UCase(sLayerName) Then
'1315:         Set FindLayer = pFLayer
'        Exit Function
'1317:       End If
'1318:     End If
'1319:   Next lLoop
'
'1321:   Set FindLayer = Nothing
'
'  Exit Function
'
'ErrHand:
'1326:   MsgBox "FindLayer - " & Erl & " - " & Err.Description
'End Function
'
'Private Function FindCompositeLayer(pCompLayer As ICompositeLayer, sLayerName As String, pMap As IMap) As IFeatureLayer
'On Error GoTo ErrHand
'  Dim lLoop As Long, pFeatLayer As IFeatureLayer
'1332:   For lLoop = 0 To pCompLayer.Count - 1
'1333:     If TypeOf pCompLayer.Layer(lLoop) Is ICompositeLayer Then
'1334:       Set pFeatLayer = FindCompositeLayer(pCompLayer.Layer(lLoop), sLayerName, pMap)
'1335:       If Not pFeatLayer Is Nothing Then
'1336:         Set FindCompositeLayer = pFeatLayer
'        Exit Function
'1338:       End If
'1339:     Else
'1340:       If TypeOf pCompLayer.Layer(lLoop) Is IFeatureLayer Then
'1341:         If UCase(pCompLayer.Layer(lLoop).name) = UCase(sLayerName) Then
'1342:           Set FindCompositeLayer = pCompLayer.Layer(lLoop)
'          Exit Function
'1344:         End If
'1345:       End If
'1346:     End If
'1347:   Next lLoop
'
'  Exit Function
'ErrHand:
'1351:   MsgBox "CompositeLayer - " & Erl & " - " & Err.Description
'End Function

'Move and resize a map layout element.
Private Sub MoveElementToPosition(ByRef pElement As IElement, pActive As IActiveView, XMin, YMin, Width, Height)
On Error GoTo ErrHand
  'MsgBox "MoveElementToPosition"
  
  'Get the geometry
  Dim pGeom As IGeometry
  Dim pTrans2d As ITransform2D
1363:   Set pGeom = pElement.Geometry
1364:   Set pTrans2d = pGeom

  'Determine the scaling factors
  Dim dDeltaWidth As Double
  Dim dDeltaHeight As Double
1369:   dDeltaWidth = Width / pGeom.Envelope.Width
1370:   dDeltaHeight = Height / pGeom.Envelope.Height
  
  'Determine the lower left point of the current envelope
  Dim pPoint As IPoint
1374:   Set pPoint = pGeom.Envelope.LowerLeft
  
  'Resize/Scale the element
  'For VB, Scale must be enclosed within a With block (according to ArcGIS Developer Help)
1378:   With pTrans2d
1379:     .Scale pPoint, dDeltaWidth, dDeltaHeight
1380:   End With
  
  'Determine how far to move the element from it's existing location to the new location.
  Dim dDeltaX As Double, dDeltaY As Double
1384:   dDeltaX = XMin - pGeom.Envelope.XMin
1385:   dDeltaY = YMin - pGeom.Envelope.YMin
    
  'Move the element to its new location.
1388:   pTrans2d.Move dDeltaX, dDeltaY

  'Update the element geometry and refresh the active view
  Dim pEnv As IEnvelope, pEnv2 As IEnvelope
1392:   Set pEnv = New Envelope
1393:   pElement.QueryBounds pActive.ScreenDisplay, pEnv
1394:   pElement.Geometry = pTrans2d
  Dim pGraphicsCont As IGraphicsContainer
1396:   Set pGraphicsCont = pActive
1397:   pGraphicsCont.UpdateElement pElement
1398:   Set pEnv2 = New Envelope
1399:   pElement.QueryBounds pActive.ScreenDisplay, pEnv2
1400:   pEnv.Union pEnv2
1401:   pActive.PartialRefresh esriViewGraphics, Nothing, pEnv

  Exit Sub
ErrHand:
1405:   MsgBox "MoveElementToPosition - " & Erl & " - " & Err.Description
End Sub
