VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCreateGrids"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True

' Copyright 2006 ESRI
'
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
'
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
'
' See use restrictions at /arcgis/developerkit/userestrictions.

Option Explicit

'----------------------------------------------
' Properties
' - DestinationFeatureClass (IFeatureClass, r/w)
' - MapScale (Double, r/w)
' - FrameWidthInPageUnits (Double, r/w)
' - FrameHeightInPageUnits (Double, r/w)
' - IdentifierOrder (mgGridIdentifierOrderType, r/w)
' - RowIDType (mgGridIdentifierValueType, r/w)
' - ColIDType (mgGridIdentifierValueType, r/w)
' - StartingCoordinateLL_X (Double, r/w)
' - StartingCoordinateLL_Y (Double, r/w)
' - EndingCoordinateUR_X (Double, r/w)
' - EndingCoordinateUR_Y (Double, r/w)
' - UseUnderscore (Boolean, r/w)
'----------------------------------------------
' Methods
' - GenerateGrids(pApp as IApplication)
'    : Generates the grids using the values added.
' - RunStandardGUI(pApp as IApplication)
'    : To open the form without having the button
'      added to ArcMap's GUI.
'----------------------------------------------
' Enumerated Types
Public Enum mgGridIdentifierOrderType
    Row_Column = 0
    Column_Row = 1
End Enum
Public Enum mgGridIdentifierValueType
    Alphabetical = 0
    Numerical = 1
End Enum
Public Enum mgGridIdentifierStartPositionType
    TopLeft = 0
    LowerLeft = 1
End Enum
'----------------------------------------------
' Local Global Declarations
Private m_DestFL As IFeatureLayer
Private m_DestFC As IFeatureClass
Private m_dMapScale As Double
Private m_dFrameWidthInPageUnits As Double
Private m_dFrameHeightInPageUnits As Double
Private m_IDOrderType As mgGridIdentifierOrderType
Private m_RowIDType As mgGridIdentifierValueType
Private m_ColIDType As mgGridIdentifierValueType
Private m_StartIDType As mgGridIdentifierStartPositionType
Private m_StartX As Double
Private m_StartY As Double
Private m_EndX As Double
Private m_EndY As Double
Private m_UseUnderscore As Boolean
Private m_colLayerNamesForData As Collection
Private m_FldID As String
Private m_FldRowNum As String
Private m_FldColNum As String
Private m_FldScale As String
Private m_RemoveGrids As Boolean
Private m_NoEmptyGrids As Boolean
Private m_pProgress As IModelessFrame

Public Property Set DestinationFeatureLayer(pFL As IFeatureLayer)
76:     If pFL.FeatureClass.ShapeType = esriGeometryPolygon Then
77:         Set m_DestFL = pFL
78:         Set m_DestFC = pFL.FeatureClass
79:     Else
80:         Err.Raise vbObjectError, "MapGridManager_Set_DestinationFeatureLayer", _
            "Not a polygon feature layer"
82:     End If
End Property

Public Property Get DestinationFeatureLayer() As IFeatureLayer
86:     Set DestinationFeatureLayer = m_DestFL
End Property

Public Property Let FrameWidthInPageUnits(dWidth As Double)
90:     m_dFrameWidthInPageUnits = dWidth
End Property

Public Property Get FrameWidthInPageUnits() As Double
94:     FrameWidthInPageUnits = m_dFrameWidthInPageUnits
End Property

Public Property Let FrameHeightInPageUnits(dHeight As Double)
98:     m_dFrameHeightInPageUnits = dHeight
End Property

Public Property Get FrameHeightInPageUnits() As Double
102:     FrameHeightInPageUnits = m_dFrameHeightInPageUnits
End Property

Public Property Let MapScale(dScale As Double)
106:     m_dMapScale = dScale
End Property

Public Property Get MapScale() As Double
110:     MapScale = m_dMapScale
End Property

Public Property Let IdentifierOrder(mgType As mgGridIdentifierOrderType)
114:     m_IDOrderType = mgType
End Property

Public Property Get IdentifierOrder() As mgGridIdentifierOrderType
118:     IdentifierOrder = m_IDOrderType
End Property

Public Property Let RowIDType(mgIDStyle As mgGridIdentifierValueType)
122:     m_RowIDType = mgIDStyle
End Property

Public Property Get RowIDType() As mgGridIdentifierValueType
126:     RowIDType = m_RowIDType
End Property

Public Property Let ColIDType(mgIDStyle As mgGridIdentifierValueType)
130:     m_ColIDType = mgIDStyle
End Property

Public Property Get ColIDType() As mgGridIdentifierValueType
134:     ColIDType = m_ColIDType
End Property

Public Property Let IDStartPositionType(mgStartPos As mgGridIdentifierStartPositionType)
138:     m_StartIDType = mgStartPos
End Property

Public Property Get IDStartPositionType() As mgGridIdentifierStartPositionType
142:     IDStartPositionType = m_StartIDType
End Property

Public Property Let StartingCoordinateLL_X(X As Double)
146:     m_StartX = X
End Property

Public Property Get StartingCoordinateLL_X() As Double
150:     StartingCoordinateLL_X = m_StartX
End Property

Public Property Let StartingCoordinateLL_Y(Y As Double)
154:     m_StartY = Y
End Property

Public Property Get StartingCoordinateLL_Y() As Double
158:     StartingCoordinateLL_Y = m_StartY
End Property

Public Property Let EndingCoordinateUR_X(X As Double)
162:     m_EndX = X
End Property

Public Property Get EndingCoordinateUR_X() As Double
166:     EndingCoordinateUR_X = m_EndX
End Property

Public Property Let EndingCoordinateUR_Y(Y As Double)
170:     m_EndY = Y
End Property

Public Property Get EndingCoordinateUR_Y() As Double
174:     EndingCoordinateUR_Y = m_EndY
End Property

Public Property Let FieldNameGridID(FieldName As String)
178:     m_FldID = FieldName
End Property

Public Property Let FieldNameRowNum(FieldName As String)
182:     m_FldRowNum = FieldName
End Property

Public Property Let FieldNameColNum(FieldName As String)
186:     m_FldColNum = FieldName
End Property

Public Property Let FieldNameScale(FieldName As String)
190:     m_FldScale = FieldName
End Property

Public Property Let UseUnderscore(UseUnderscoreInID As Boolean)
194:     m_UseUnderscore = UseUnderscoreInID
End Property

Public Property Get UseUnderscore() As Boolean
198:     UseUnderscore = m_UseUnderscore
End Property

Public Property Let RemoveCurrentGrids(RemoveGrids As Boolean)
202:     m_RemoveGrids = RemoveGrids
End Property

Public Property Get RemoveCurrentGrids() As Boolean
206:     RemoveCurrentGrids = m_RemoveGrids
End Property

Public Property Let NoEmptyGrids(NoEmptyGridPolygons As Boolean)
210:     m_NoEmptyGrids = NoEmptyGridPolygons
End Property

Public Property Get NoEmptyGrids() As Boolean
214:     NoEmptyGrids = m_NoEmptyGrids
End Property

Public Sub AddRequiredDataLayer(sLayerName As String)
218:     m_colLayerNamesForData.Add sLayerName
End Sub

Public Property Get RequiredDataLayer(LayerNumber As Long) As String
222:     If LayerNumber <= m_colLayerNamesForData.count Then
223:         RequiredDataLayer = m_colLayerNamesForData.Item(LayerNumber - 1)
224:     Else
225:         If m_colLayerNamesForData.count = 0 Then
226:             RequiredDataLayer = "There are no Required Data Layers."
227:         Else
228:             RequiredDataLayer = "Invalid layer number.  Valid range: 0 - " & (m_colLayerNamesForData.count - 1)
229:         End If
230:     End If
End Property

Public Property Get RequiredDataLayerCount() As Long
234:     RequiredDataLayerCount = m_colLayerNamesForData.count
End Property

Public Sub ClearRequiredDataLayers()
238:     Set m_colLayerNamesForData = New Collection
End Sub

Private Sub Class_Initialize()
    ' Set the default grid identifier (row-col, alpha/numeric => eg: "B02")
243:     m_IDOrderType = Row_Column
244:     m_RowIDType = Alphabetical
245:     m_ColIDType = Numerical
246:     Set m_colLayerNamesForData = New Collection
247:     Set m_pProgress = New ModelessFrame
End Sub

Private Function CalculateID(ByVal lRow As Long, ByVal lCol As Long, _
                             ByVal iRowIDLen As Integer, ByVal iColIDLen As Integer) As String
    Dim sRowID As String
    Dim sColID As String
    Dim sNumericFormat As String
    Dim lLoop As Long
    Dim lTmp As Long, lCalc As Long, lTmp2 As Long

    ' Row ---------------------------------------------
259:     sNumericFormat = ""
260:     For lLoop = 1 To iRowIDLen
261:         sNumericFormat = sNumericFormat & "0"
262:     Next
263:     If m_RowIDType = Alphabetical Then
264:         For lLoop = 1 To iRowIDLen
265:             lTmp = 26 ^ lLoop
266:             lTmp2 = (26 ^ (lLoop - 1))
267:             If lRow >= lTmp2 Then
268:                 lCalc = ((((lRow - lTmp2) / lTmp) * 26) + 1) Mod 26
269:                 sRowID = Chr(Asc("A") + lCalc) & sRowID
270:                 lRow = lRow - (lCalc * lTmp2)
271:             Else
272:                 sRowID = "A" & sRowID
273:             End If
274:         Next
275:     Else
276:         sRowID = Format(lRow + 1, sNumericFormat)
277:     End If
    ' Col ---------------------------------------------
279:     sNumericFormat = ""
280:     For lLoop = 1 To iColIDLen
281:         sNumericFormat = sNumericFormat & "0"
282:     Next
283:     If m_ColIDType = Alphabetical Then
284:         For lLoop = 1 To iColIDLen
285:             lTmp = 26 ^ lLoop
286:             lTmp2 = (26 ^ (lLoop - 1))
287:             If lCol >= lTmp2 Then
288:                 lCalc = ((((lCol - lTmp2) / lTmp) * 26) + 1) Mod 26
289:                 sColID = Chr(Asc("A") + lCalc) & sColID
290:                 lCol = lCol - (lCalc * lTmp2)
291:             Else
292:                 sColID = "A" & sColID
293:             End If
294:         Next
295:     Else
296:         sColID = Format(lCol + 1, sNumericFormat)
297:     End If
    ' Join --------------------------------------------
299:     If m_IDOrderType = Row_Column Then
300:         If m_UseUnderscore Then
301:             CalculateID = sRowID & "_" & sColID
302:         Else
303:             CalculateID = sRowID & sColID
304:         End If
305:     Else
306:         If m_UseUnderscore Then
307:             CalculateID = sColID & "_" & sRowID
308:         Else
309:             CalculateID = sColID & sRowID
310:         End If
311:     End If
End Function

Private Function GetMinimumStringLength(lValue As Long, lBase As Long) As Integer
    Dim lTmp As Long, lIndex As Long
    
    On Error GoTo eh
    ' ROW
319:     lTmp = lBase
320:     lIndex = 1
321:     While lValue > (lTmp - 1)
322:         lTmp = lTmp * lBase
323:         lIndex = lIndex + 1
324:     Wend
325:     GetMinimumStringLength = lIndex
    Exit Function
eh:
328:     Err.Raise Err.Number, "GetMinimumStringLength", "Error in GetMinimumStringLength: " & Erl & " - " & Err.Description
End Function

Public Sub RunStandardGUI(pApp As IApplication)
332:     Set frmGridSettings.m_Application = pApp
333:     frmGridSettings.Tickle
334:     SetWindowLong frmGridSettings.hwnd, GWL_HWNDPARENT, pApp.hwnd
335:     frmGridSettings.Show vbModeless
End Sub

'Private Function CreateGridPoly(pStartPoint As IPoint, lRow As Long, lCol As Long, _
'                                dGridWidth As Double, dGridHeight As Double) As IPolygon
'    Dim pPntColl As IPointCollection
'    Dim pPoint As IPoint
'    Dim dX As Double, dY As Double
'
'    Set CreateGridPoly = New Polygon
'    Set pPntColl = CreateGridPoly
'    Set pPoint = new esrigeometry.point
'    pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
'    pPntColl.AddPoint pPoint
'    Set pPoint = new esrigeometry.point
'    pPoint.PutCoords pStartPoint.X + ((lCol + 1) * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
'    pPntColl.AddPoint pPoint
'    Set pPoint = new esrigeometry.point
'    pPoint.PutCoords pStartPoint.X + ((lCol + 1) * dGridWidth), pStartPoint.Y + ((lRow + 1) * dGridHeight)
'    pPntColl.AddPoint pPoint
'    Set pPoint = new esrigeometry.point
'    pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + ((lRow + 1) * dGridHeight)
'    pPntColl.AddPoint pPoint
'    Set pPoint = new esrigeometry.point
'    pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
'    pPntColl.AddPoint pPoint
'
'    'Set CreateGridPoly = pPntColl
'    Debug.Print CreateGridPoly.IsClosed
'    'Debug.Print CreateGridPoly.Envelope.XMin & "," & CreateGridPoly.Envelope.YMin
'
'End Function

Public Sub GenerateGrids2(Application As IApplication) ', _
                         'Optional bRemoveEmptyGrids As Boolean = False, _
                         'Optional bReplaceExistingGrids As Boolean = False)
    Dim pEditor As IEditor
    Dim pUID As New UID
    Dim pWorkspaceEdit As IWorkspaceEdit
    Dim lLoop As Long
    Dim pFeatDataset As IFeatureDataset
    Dim pFeature As IFeature
    Dim pFeatCur As IFeatureCursor
    Dim pSourcePolygon As IPolygon
    Dim pGridPolygon As IPolygon
    Dim pPointColl As IPointCollection
    Dim pStartingCoord As IPoint
    Dim pPoint As IPoint
    Dim lRow As Long
    Dim lCol As Long
    Dim lRowCount As Long
    Dim lColCount As Long
    Dim pClone As IClone
    Dim dGridSizeW As Double
    Dim dGridSizeH As Double
    Dim pTransform As ITransform2D
    Dim bOKToAdd As Boolean
    Dim iStringLengthRow As Integer
    Dim iStringLengthCol As Integer
    Dim pDataset As IDataset
    Dim lBase As Long
    Dim dDataFrameWidth As Double
    Dim dDataFrameHeight As Double
    Dim dConvertPageToMapUnits As Double
    Dim dIncrement As Double
    Dim pInsertFeatureBuffer As IFeatureBuffer
    Dim pInsertFeatureCursor As IFeatureCursor
    Dim pMx As IMxDocument
    Dim pFL As IFeatureLayer
    Dim pFC As IFeatureClass
    Dim pProgress As frmProgress
    
    On Error GoTo eh
    
    ' Set mouse pointer
410:     Screen.MousePointer = vbArrowHourglass
    
    ' Init
413:     Set pMx = Application.Document
414:     For lLoop = 0 To pMx.FocusMap.LayerCount - 1
415:         If TypeOf pMx.FocusMap.Layer(lLoop) Is IFeatureLayer Then
416:             If UCase(pMx.FocusMap.Layer(lLoop).Name) = UCase(m_DestFL.Name) Then
417:                 Set pFL = pMx.FocusMap.Layer(lLoop)
418:                 Exit For
419:             End If
420:         End If
421:     Next
422:     If pFL Is Nothing Then
423:         MsgBox "No match in the Map for layer '" & m_DestFL.Name & "'."
        Exit Sub
425:     End If
426:     Set pFC = pFL.FeatureClass
    ' Check for required fields - that the field exists
    Dim bErrorWithFields As Boolean
429:     bErrorWithFields = (pFC.FindField(m_FldID) < 0)
430:     If Len(m_FldRowNum) > 0 Then bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldRowNum) < 0)
431:     If Len(m_FldColNum) > 0 Then bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldColNum) < 0)
432:     If Len(m_FldScale) > 0 Then bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldScale) < 0)
    ' If error
434:     If bErrorWithFields Then
435:         Err.Raise vbObjectError, "GenerateGrids", "Could not find all the given field names in " & pFL.Name & "." _
            & vbCrLf & " - " & m_FldID & ", " & m_FldRowNum & ", " & m_FldColNum & ", " & m_FldScale
437:     End If
    ' Check the field types
439:     bErrorWithFields = (pFC.Fields.Field(pFC.FindField(m_FldID)).Type <> esriFieldTypeString)
440:     If Len(m_FldRowNum) > 0 Then
441:         bErrorWithFields = bErrorWithFields Or _
           ((pFC.Fields.Field(pFC.FindField(m_FldRowNum)).Type <> esriFieldTypeDouble) And _
            (pFC.Fields.Field(pFC.FindField(m_FldRowNum)).Type <> esriFieldTypeInteger) And _
            (pFC.Fields.Field(pFC.FindField(m_FldRowNum)).Type <> esriFieldTypeSingle) And _
            (pFC.Fields.Field(pFC.FindField(m_FldRowNum)).Type <> esriFieldTypeSmallInteger))
446:     End If
447:     If Len(m_FldColNum) > 0 Then
448:         bErrorWithFields = bErrorWithFields Or _
           ((pFC.Fields.Field(pFC.FindField(m_FldColNum)).Type <> esriFieldTypeDouble) And _
            (pFC.Fields.Field(pFC.FindField(m_FldColNum)).Type <> esriFieldTypeInteger) And _
            (pFC.Fields.Field(pFC.FindField(m_FldColNum)).Type <> esriFieldTypeSingle) And _
            (pFC.Fields.Field(pFC.FindField(m_FldColNum)).Type <> esriFieldTypeSmallInteger))
453:     End If
454:     If Len(m_FldScale) > 0 Then
455:         bErrorWithFields = bErrorWithFields Or _
           ((pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeDouble) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeInteger) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeSingle) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeSmallInteger))
460:     End If
    ' if error
462:     If bErrorWithFields Then
463:         Err.Raise vbObjectError, "GenerateGrids", "Given field names are not of the correct type." _
            & vbCrLf & "Grid ID field must be a Text field, all others must be numeric fields."
465:     End If
    ' Get the dataset and workspace (to start editing upon)
467:     Set pFeatDataset = pFC.FeatureDataset
468:     If Not pFeatDataset Is Nothing Then
469:         Set pWorkspaceEdit = pFeatDataset.Workspace
470:     Else
        ' Is a shapefile, go via just IDataset
472:         Set pDataset = pFC
473:         Set pWorkspaceEdit = pDataset.Workspace
474:     End If
475:     dDataFrameWidth = m_dFrameWidthInPageUnits
476:     dDataFrameHeight = m_dFrameHeightInPageUnits
    ' Start Editing
478:     pWorkspaceEdit.StartEditing False
479:     pWorkspaceEdit.StartEditOperation
    
    ' If replacing, delete all existing polygons
482:     Set pProgress = New frmProgress
483:     m_pProgress.Create pProgress
484:     If m_RemoveGrids Then
        Dim pFCu As IFeatureCursor
        Dim pT As ITable
487:         Set pFCu = m_DestFL.Search(Nothing, False)
488:         Set pT = m_DestFL.FeatureClass
489:         pProgress.ProgressBar1.Min = 0
490:         pProgress.ProgressBar1.Max = 100
491:         If pT.RowCount(Nothing) = 0 Then
492:             dIncrement = 99
493:         Else
494:             dIncrement = 100 / pT.RowCount(Nothing)
495:         End If
496:         pProgress.ProgressBar1.value = 0
497:         pProgress.lblInformation.Caption = "Deleting previous grids..."
498:         pProgress.cmdCancel.Visible = False        ' User cannot cancel this step
        
500:         m_pProgress.Visible = True
501:         Set pFeature = pFCu.NextFeature
502:         While Not pFeature Is Nothing
503:             pFeature.Delete
504:             If (pProgress.ProgressBar1.value + dIncrement) <= pProgress.ProgressBar1.Max Then
505:                 pProgress.ProgressBar1.value = pProgress.ProgressBar1.value + dIncrement
506:             Else
507:                 pProgress.lblInformation.Caption = "Warning: Val > Max (" & pProgress.ProgressBar1.Max & ")"
508:             End If
509:             Set pFeature = pFCu.NextFeature
510:         Wend
511:         m_pProgress.Visible = False
512:     End If
    
    ' Calc the row/column extents, grid size (map units), ID lengths and starting coordinate
515:     Set pStartingCoord = New esriGeometry.Point
516:     pStartingCoord.PutCoords m_StartX, m_StartY
517:     dConvertPageToMapUnits = CalculatePageToMapRatio(Application)
518:     dGridSizeW = ((m_dMapScale * dDataFrameWidth) / dConvertPageToMapUnits)
519:     dGridSizeH = ((m_dMapScale * dDataFrameHeight) / dConvertPageToMapUnits)
520:     If Not (pFL.FeatureClass.FeatureDataset Is Nothing) Then
521:         CalculateRowColCounts m_StartX, m_StartY, m_EndX, m_EndY, _
            dGridSizeW, dGridSizeH, GetValidExtentForLayer(m_DestFL), lColCount, lRowCount
523:     Else
524:         CalculateRowColCounts m_StartX, m_StartY, m_EndX, m_EndY, _
            dGridSizeW, dGridSizeH, Nothing, lColCount, lRowCount
526:     End If
527:     If lColCount = 0 Or lRowCount = 0 Then
528:         Err.Raise vbObjectError, "GenerateGrids", "CalculateRowColCounts returned zero row/columns"
529:     End If
530:     If m_ColIDType = Alphabetical Then
531:         lBase = 26
532:     Else
533:         lBase = 10
534:     End If
535:     iStringLengthCol = GetMinimumStringLength(lColCount, lBase)
536:     If m_RowIDType = Alphabetical Then
537:         lBase = 26
538:     Else
539:         lBase = 10
540:     End If
541:     iStringLengthRow = GetMinimumStringLength(lRowCount, lBase)
    
    ' Create and add the new Grid polygon features
544:     bOKToAdd = True
545:     pProgress.ProgressBar1.Min = 0
546:     pProgress.ProgressBar1.Max = 100
547:     pProgress.ProgressBar1.value = 0
548:     pProgress.lblInformation.Caption = "Creating new Grid features..."
549:     pProgress.cmdCancel.Visible = True
550:     m_pProgress.Visible = True
551:     dIncrement = 99.9 / (lRowCount * lColCount)
552:     Set pInsertFeatureCursor = pFC.Insert(True)
553:     Set pInsertFeatureBuffer = pFC.CreateFeatureBuffer
554:     For lRow = 0 To lRowCount - 1
555:         For lCol = 0 To lColCount - 1
            ' Create the source polygon
557:             Set pGridPolygon = CreateGridPoly2(pStartingCoord, lRow, lCol, dGridSizeW, dGridSizeH)
            ' If required, check for containing features
559:             If m_NoEmptyGrids Then
560:                 bOKToAdd = HasFeatures(pGridPolygon, pMx.FocusMap)
561:             End If
562:             If bOKToAdd Then
                ' Create new grid cell feature
564:                 Set pInsertFeatureBuffer.Shape = pGridPolygon
565:                 If m_StartIDType = TopLeft Then
566:                     pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldID)) = _
                            CalculateID((lRowCount - 1) - lRow, lCol, iStringLengthRow, iStringLengthCol)
568:                     If Len(m_FldRowNum) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldRowNum)) = (lRowCount - lRow)
569:                 Else
570:                     pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldID)) = _
                            CalculateID(lRow, lCol, iStringLengthRow, iStringLengthCol)
572:                     If Len(m_FldRowNum) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldRowNum)) = (lRow + 1)
573:                 End If
574:                 If Len(m_FldColNum) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldColNum)) = (lCol + 1)
575:                 If Len(m_FldScale) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldScale)) = m_dMapScale
576:                 pInsertFeatureCursor.InsertFeature pInsertFeatureBuffer
577:             End If
578:             If pProgress.ProgressBar1.value < pProgress.ProgressBar1.Max Then
579:                 pProgress.ProgressBar1.value = pProgress.ProgressBar1.value + dIncrement
580:             Else
581:                 pProgress.lblInformation.Caption = "Warning: Val > Max (" & pProgress.ProgressBar1.Max & ")"
582:             End If
583:             If ((lRow * lColCount) + lCol) Mod 20 = 0 Then
584:                 DoEvents
585:                 pInsertFeatureCursor.Flush
586:             End If
587:             pProgress.Refresh
588:             If pProgress.Cancelled Then
                Dim vUserChoice
590:                 pProgress.Cancelled = False       ' Reset the form
591:                 vUserChoice = MsgBox("Operation cancelled." _
                    & "  Save the edits made thus far?" & vbCrLf & vbCrLf _
                    & "(Click Cancel to continue processing)", _
                                vbYesNoCancel, "Generate Grids")
595:                 If vUserChoice <> vbCancel Then
596:                     GoTo CancelledGenerateGrids     'Sorry for GoTo usage - in a hurry
597:                 End If
598:             End If
599:         Next
600:     Next
601:     pInsertFeatureCursor.Flush
602:     m_pProgress.Visible = False
    
    ' Stop editing
605:     pWorkspaceEdit.StopEditOperation
606:     pWorkspaceEdit.StopEditing True
    ' ----------------------------------------------------
    
609:     Screen.MousePointer = vbDefault
610:     pMx.ActiveView.Refresh
    
    Exit Sub
    
CancelledGenerateGrids:
615:     m_pProgress.Visible = False
616:     If vUserChoice = vbYes Then
617:         pInsertFeatureCursor.Flush
618:         pWorkspaceEdit.StopEditOperation
619:         pWorkspaceEdit.StopEditing True
620:     Else
621:         pWorkspaceEdit.StopEditOperation
622:         pWorkspaceEdit.StopEditing False
623:     End If
624:     Screen.MousePointer = vbDefault
625:     pMx.ActiveView.Refresh
    Exit Sub
    
628:     Resume
eh:
630:     m_pProgress.Visible = False
631:     Screen.MousePointer = vbDefault
632:     If Not pWorkspaceEdit Is Nothing Then
633:         If pWorkspaceEdit.IsBeingEdited Then
634:             pWorkspaceEdit.StopEditOperation
635:             pWorkspaceEdit.StopEditing False
636:         End If
637:     End If
638:     MsgBox "GenerateGrids - " & Erl & " - " & Err.Description
End Sub


Private Function CreateGridPoly2(pStartPoint As IPoint, lRow As Long, lCol As Long, _
                                dGridWidth As Double, dGridHeight As Double) As IPolygon
    Dim pPntColl As IPointCollection
    Dim pPoint As IPoint
    Dim dX As Double, dY As Double
    
648:     Set CreateGridPoly2 = New Polygon
649:     Set pPntColl = CreateGridPoly2
650:     Set pPoint = New esriGeometry.Point
651:     pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
652:     pPntColl.AddPoint pPoint
653:     Set pPoint = New esriGeometry.Point
654:     pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + ((lRow + 1) * dGridHeight)
655:     pPntColl.AddPoint pPoint
656:     Set pPoint = New esriGeometry.Point
657:     pPoint.PutCoords pStartPoint.X + ((lCol + 1) * dGridWidth), pStartPoint.Y + ((lRow + 1) * dGridHeight)
658:     pPntColl.AddPoint pPoint
659:     Set pPoint = New esriGeometry.Point
660:     pPoint.PutCoords pStartPoint.X + ((lCol + 1) * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
661:     pPntColl.AddPoint pPoint
662:     Set pPoint = New esriGeometry.Point
663:     pPoint.PutCoords pStartPoint.X + (lCol * dGridWidth), pStartPoint.Y + (lRow * dGridHeight)
664:     pPntColl.AddPoint pPoint
    
End Function

Private Sub CalculateRowColCounts(ByVal XStart As Double, ByVal YStart As Double, _
                                  ByVal XEnd As Double, ByVal YEnd As Double, _
                                  ByVal XGridWidth As Double, ByVal YGridHeight As Double, _
                                  ByVal LayerExtent As IEnvelope, _
                                  ByRef ReturnColCount As Long, ByRef ReturnRowCount As Long)
    Dim XRange As Double
    Dim YRange As Double
    Dim dTemp As Double
    Dim lMultiplierX As Long
    Dim lMultiplierY As Long
    
    On Error GoTo eh
    
    ' Protect against div by zero (where the grid width/height is less than 0.5 => ie: with Geographic Data)
682:     lMultiplierX = 1
683:     dTemp = XGridWidth
684:     While dTemp < 10
685:         dTemp = dTemp * 10
686:         lMultiplierX = lMultiplierX * 10
687:     Wend
688:     lMultiplierY = 1
689:     dTemp = YGridHeight
690:     While dTemp < 10
691:         dTemp = dTemp * 10
692:         lMultiplierY = lMultiplierY * 10
693:     Wend
    
    ' Init
696:     XRange = XEnd - XStart
697:     YRange = YEnd - YStart
    'X ------------------------------------------
699:     If Not (LayerExtent Is Nothing) Then
700:         If XStart < LayerExtent.XMin Then
701:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Starting X (" & XStart & ") is outside the valid layer extent (" & LayerExtent.XMin & ")"
703:         ElseIf XEnd > LayerExtent.XMax Then
704:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Ending X (" & XStart & ") is outside the valid layer extent (" & LayerExtent.XMax & ")"
706:         ElseIf (XStart + XGridWidth) > LayerExtent.XMax Then
707:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Adding a single Grid (width " & XGridWidth & ") would exceed valid X range for layer (" & LayerExtent.XMax & ")"
709:         End If
710:     End If
711:     If XRange < XGridWidth Then
712:         Err.Raise vbObjectError, "CalculateRowColCounts", _
            "Grid width " & XGridWidth & " exceeds range specified (" & XStart & " - " & XEnd & ")"
714:     ElseIf (XStart + XGridWidth) > XEnd Then
715:         Err.Raise vbObjectError, "CalculateRowColCounts", _
            "Adding a single Grid (width " & XGridWidth & ") would exceed specified X range (" & XEnd & ")"
717:     End If
718:     ReturnColCount = (XRange * lMultiplierX) \ (XGridWidth * lMultiplierX)
    'Y ------------------------------------------
720:     If Not (LayerExtent Is Nothing) Then
721:         If YStart < LayerExtent.YMin Then
722:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Starting Y (" & YStart & ") is outside the valid layer extent (" & LayerExtent.YMin & ")"
724:         ElseIf YEnd > LayerExtent.YMax Then
725:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Ending Y (" & YStart & ") is outside the valid layer extent (" & LayerExtent.YMax & ")"
727:         ElseIf (YStart + YGridHeight) > LayerExtent.YMax Then
728:             Err.Raise vbObjectError, "CalculateRowColCounts", _
                "Adding a single Grid (height " & YGridHeight & ") would exceed valid Y range for layer (" & LayerExtent.YMax & ")"
730:         End If
731:     End If
732:     If YRange < YGridHeight Then
733:         Err.Raise vbObjectError, "CalculateRowColCounts", _
            "Grid height " & YGridHeight & " exceeds range specified (" & YStart & " - " & YEnd & ")"
735:     ElseIf (XStart + XGridWidth) > XEnd Then
736:         Err.Raise vbObjectError, "CalculateRowColCounts", _
            "Adding a single Grid (height " & YGridHeight & ") would exceed specified Y range (" & YEnd & ")"
738:     End If
739:     ReturnRowCount = (YRange * lMultiplierY) \ (YGridHeight * lMultiplierY)
    '--------------------------------------------
    
    Exit Sub
eh:
744:     Err.Raise Err.Number, Err.source, "Error in CalculateRowColCounts: " & Erl & " - " & Err.Description
End Sub

Private Function CalculatePageToMapRatio(pApp As IApplication) As Double
    Dim pMx As IMxDocument
    Dim pPage As IPage
    Dim pPageUnits As esriUnits
    Dim pSR As ISpatialReference
    Dim pSRI As ISpatialReferenceInfo
    Dim pPCS As IProjectedCoordinateSystem
    Dim dMetersPerUnit As Double
    Dim dCurrScale As Double
    Dim pExtentEnv As IEnvelope
    
    On Error GoTo eh
    
    ' Init
761:     Set pMx = pApp.Document
762:     Set pSR = pMx.FocusMap.SpatialReference
    ' If a Projected coord system
764:     If TypeOf pSR Is IProjectedCoordinateSystem Then
        ' Use meters per unit as the conversion
766:         Set pPCS = pSR
767:         dMetersPerUnit = pPCS.CoordinateUnit.MetersPerUnit
        ' Now convert this into page (ie: paper) units
769:         Set pPage = pMx.PageLayout.Page
770:         pPageUnits = pPage.Units
        Select Case pPageUnits
            Case esriInches: CalculatePageToMapRatio = dMetersPerUnit / (1 / 12 * 0.304800609601219)
            Case esriFeet: CalculatePageToMapRatio = dMetersPerUnit / (0.304800609601219)
            Case esriCentimeters: CalculatePageToMapRatio = dMetersPerUnit / (1 / 100)
            Case esriMeters: CalculatePageToMapRatio = dMetersPerUnit / (1)
            Case Else:
777:                 MsgBox "Warning: Only the following Page (Layout) Units are supported by this tool:" _
                    & vbCrLf & " - Inches, Feet, Centimeters, Meters" _
                    & vbCrLf & vbCrLf & "Calculating as though Page Units are in Inches..."
780:                 CalculatePageToMapRatio = dMetersPerUnit / (1 / 12 * 0.304800609601219)
781:         End Select
    ' Otherwise
783:     Else
        ' If not projected, we can only do a "flat" conversion -> that is, use the current scale and extent
        '  as a ratio to be applied to the map grid scale.
        ' NOTE: We MUST be in Layout mode to make this calculation, as the scale in Map View and Layout View
        '  are not the same (as the extent envelope and data frame envelope can be different shapes).  The
        '  test for being in Layout Mode is made in the clsMapGridButton.ICommand_Enabled property.
789:         Set pExtentEnv = pMx.ActiveView.Extent
790:         dCurrScale = pMx.FocusMap.MapScale
791:         If ((m_EndX - m_StartX) / m_dFrameWidthInPageUnits) > ((m_EndY - m_StartY) / m_dFrameHeightInPageUnits) Then
792:             CalculatePageToMapRatio = m_dFrameWidthInPageUnits / ((m_EndX - m_StartX) / dCurrScale)
793:         Else
794:             CalculatePageToMapRatio = m_dFrameHeightInPageUnits / ((m_EndY - m_StartY) / dCurrScale)
795:         End If
796:     End If
    
    Exit Function
eh:
800:     CalculatePageToMapRatio = 1
801:     MsgBox "Error in CalculatePageToMapRatio" & vbCrLf & Erl & " - " & Err.Description
End Function

Private Function HasFeatures(pPolygon As IPolygon, pMap As IMap) As Boolean
    Dim lLoop As Long
    Dim pFL As IFeatureLayer
    Dim pSF As ISpatialFilter
    Dim pFC As IFeatureCursor
    Dim pF As IFeature
    
811:     HasFeatures = False
812:     Set pSF = New SpatialFilter
813:     Set pSF.Geometry = pPolygon
814:     pSF.SpatialRel = esriSpatialRelIntersects
815:     For lLoop = 0 To (pMap.LayerCount - 1)
816:         If TypeOf pMap.Layer(lLoop) Is IFeatureLayer Then
817:             Set pFL = pMap.Layer(lLoop)
818:             If pFL.Name <> m_DestFL.Name And IsARequiredLayer(pFL.Name) Then
819:                 pSF.GeometryField = pFL.FeatureClass.shapeFieldName
820:                 Set pFC = pFL.Search(pSF, False)
821:                 Set pF = pFC.NextFeature
822:                 If Not pF Is Nothing Then
823:                     HasFeatures = True
824:                     Exit For
825:                 End If
826:             End If
827:         End If
828:     Next
End Function

Private Function IsARequiredLayer(sLayerName As String) As Boolean
    Dim lLoop As Long
833:     For lLoop = 1 To m_colLayerNamesForData.count
834:         If UCase(m_colLayerNamesForData.Item(lLoop)) = UCase(sLayerName) Then
835:             IsARequiredLayer = True
836:             Exit For
837:         End If
838:     Next
End Function

Private Sub Class_Terminate()
842:     Set m_DestFL = Nothing
843:     Set m_DestFC = Nothing
844:     Set m_pProgress = Nothing
End Sub

