VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCreateStripMap"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

' Copyright 2006 ESRI
'
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
'
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
'
' See use restrictions at /arcgis/developerkit/userestrictions.

Option Explicit

'----------------------------------------------
' Properties
' - DestinationFeatureClass (IFeatureClass, r/w)
' - MapScale (Double, r/w)
' - FrameWidthInPageUnits (Double, r/w)
' - FrameHeightInPageUnits (Double, r/w)
' - StripMapRoute (IPolyline, r/w)
'----------------------------------------------
' Methods
' - GenerateGrids(pApp as IApplication)
'    : Generates the grids using the values added.
' - RunStandardGUI(pApp as IApplication)
'    : To open the form without having the button
'      added to ArcMap's GUI.
'----------------------------------------------
' Local Global Declarations
Private m_DestFL As IFeatureLayer
Private m_DestFC As IFeatureClass
Private m_Polyline As IPolyline
Private m_dMapScale As Double
Private m_dFrameWidthInPageUnits As Double
Private m_dFrameHeightInPageUnits As Double
Private m_FldStripName As String
Private m_FldNumInSeries As String
Private m_FldMapAngle As String
Private m_FldScale As String

'Add Private Variable to store "From Station", "To Station", and "In Grid" field names -SF 05/21/2009
Private m_FldFromStation As String
Private m_FldToStation As String
Private m_FldInGrid As String

Private m_RemoveGrids As Boolean
Private m_Flip As Boolean
Private m_GridWidth As Double
Private m_GridHeight As Double
Private m_StripMapName As String
Private m_pProgress As IModelessFrame
'Added properties for option to load station values from a table.
Private m_LoadStationsFromTable As Boolean
Private m_StationValuesTable As String
Private m_StationValuesFROMColumn As String
Private m_StationValuesTOColumn As String
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\development\Source\DSMapBook\clsCreateStripMap.cls"

'******Added properties for option to load station values from a table.
Public Property Let LoadStationsFromTable(bLoadStationsFromTable As Boolean)
63:   m_LoadStationsFromTable = bLoadStationsFromTable
End Property

Public Property Get LoadStationsFromTable() As Boolean
67:   LoadStationsFromTable = m_LoadStationsFromTable
End Property

Public Property Let StationValuesTable(sStationValuesTable As String)
71:   m_StationValuesTable = sStationValuesTable
End Property

Public Property Get StationValuesTable() As String
75:   StationValuesTable = m_StationValuesTable
End Property

Public Property Let StationValuesFROMColumn(sStationValuesFROMColumn As String)
79:   m_StationValuesFROMColumn = sStationValuesFROMColumn
End Property

Public Property Get StationValuesFROMColumn() As String
83:   StationValuesFROMColumn = m_StationValuesFROMColumn
End Property

Public Property Let StationValuesTOColumn(sStationValuesTOColumn As String)
87:   m_StationValuesTOColumn = sStationValuesTOColumn
End Property

Public Property Get StationValuesTOColumn() As String
91:   StationValuesTOColumn = m_StationValuesTOColumn
End Property
'******

Public Property Set DestinationFeatureLayer(pFL As IFeatureLayer)
96:     If pFL.FeatureClass.ShapeType = esriGeometryPolygon Then
97:         Set m_DestFL = pFL
98:         Set m_DestFC = pFL.FeatureClass
99:     Else
100:         Err.Raise vbObjectError, "MapGridManager_Set_DestinationFeatureLayer", _
            "Not a polygon feature layer"
102:     End If
End Property

Public Property Get DestinationFeatureLayer() As IFeatureLayer
106:     Set DestinationFeatureLayer = m_DestFL
End Property

Public Property Set StripMapRoute(RoutePolyline As IPolyline)
110:     Set m_Polyline = RoutePolyline
End Property

Public Property Get StripMapRoute() As IPolyline
114:     Set StripMapRoute = m_Polyline
End Property

Public Property Let FrameWidthInPageUnits(dWidth As Double)
118:     m_dFrameWidthInPageUnits = dWidth
End Property

Public Property Get FrameWidthInPageUnits() As Double
122:     FrameWidthInPageUnits = m_dFrameWidthInPageUnits
End Property

Public Property Let FrameHeightInPageUnits(dHeight As Double)
126:     m_dFrameHeightInPageUnits = dHeight
End Property

Public Property Get FrameHeightInPageUnits() As Double
130:     FrameHeightInPageUnits = m_dFrameHeightInPageUnits
End Property

Public Property Let MapScale(dScale As Double)
134:     m_dMapScale = dScale
End Property

Public Property Get MapScale() As Double
138:     MapScale = m_dMapScale
End Property

Public Property Let StripMapName(MapName As String)
142:     m_StripMapName = MapName
End Property

Public Property Get StripMapName() As String
146:     StripMapName = m_StripMapName
End Property


Public Property Let FieldNameStripMapName(FieldName As String)
151:     m_FldStripName = FieldName
End Property

Public Property Get FieldNameStripMapName() As String
155:     FieldNameStripMapName = m_FldStripName
End Property

Public Property Let FieldNameNumberInSeries(FieldName As String)
159:     m_FldNumInSeries = FieldName
End Property

Public Property Get FieldNameNumberInSeries() As String
163:     FieldNameNumberInSeries = m_FldNumInSeries
End Property

Public Property Let FieldNameMapAngle(FieldName As String)
167:     m_FldMapAngle = FieldName
End Property

Public Property Get FieldNameMapAngle() As String
171:     FieldNameMapAngle = m_FldMapAngle
End Property

Public Property Let FieldNameScale(FieldName As String)
175:     m_FldScale = FieldName
End Property

Public Property Get FieldNameScale() As String
179:     FieldNameScale = m_FldScale
End Property

'*********Added public variables to get/set "From Station", "To Station", and "In Grid" field names -SF 05/21/2009*********
Public Property Let FieldNameFromStat(FieldName As String)
184:     m_FldFromStation = FieldName
End Property

Public Property Get FieldNameFromStat() As String
188:     FieldNameFromStat = m_FldFromStation
End Property

Public Property Let FieldNameToStat(FieldName As String)
192:     m_FldToStation = FieldName
End Property

Public Property Get FieldNameToStat() As String
196:     FieldNameToStat = m_FldToStation
End Property

Public Property Let FieldNameInGrid(FieldName As String)
200:     m_FldInGrid = FieldName
End Property

Public Property Get FieldNameInGrid() As String
204:     FieldNameInGrid = m_FldInGrid
End Property
'**************(End of) -SF 05/21/2009*********

Public Property Let RemoveCurrentGrids(RemoveGrids As Boolean)
209:     m_RemoveGrids = RemoveGrids
End Property

Public Property Get RemoveCurrentGrids() As Boolean
213:     RemoveCurrentGrids = m_RemoveGrids
End Property

Public Property Let FlipPolyline(Flip As Boolean)
217:     m_Flip = Flip
End Property

Public Property Get FlipPolyline() As Boolean
221:     FlipPolyline = m_Flip
End Property

Private Sub Class_Initialize()
    ' Set the defaults
226:     Set m_pProgress = New ModelessFrame
End Sub

Public Sub RunStandardGUI(pApp As IApplication)
230:     Set frmSMapSettings.m_Application = pApp
231:     frmSMapSettings.Tickle
232:     SetWindowLong frmSMapSettings.hwnd, GWL_HWNDPARENT, pApp.hwnd
233:     frmSMapSettings.Show vbModeless
End Sub

Private Function CalculatePageToMapRatio(pApp As IApplication) As Double
    Dim pMx As IMxDocument
    Dim pPage As IPage
    Dim pPageUnits As esriUnits
    Dim pSR As ISpatialReference
    Dim pSRI As ISpatialReferenceInfo
    Dim pPCS As IProjectedCoordinateSystem
    Dim dMetersPerUnit As Double
    Dim dCurrScale As Double
    Dim pExtentEnv As IEnvelope
    Dim dEndX As Double, dEndY As Double
    Dim dStartX As Double, dStartY As Double
    
    On Error GoTo eh
    
    ' Init
252:     Set pMx = pApp.Document
253:     Set pSR = pMx.FocusMap.SpatialReference
    ' If a Projected coord system
255:     If TypeOf pSR Is IProjectedCoordinateSystem Then
        ' Use meters per unit as the conversion
257:         Set pPCS = pSR
258:         dMetersPerUnit = pPCS.CoordinateUnit.MetersPerUnit
        ' Now convert this into page (ie: paper) units
260:         Set pPage = pMx.PageLayout.Page
261:         pPageUnits = pPage.Units
        Select Case pPageUnits
            Case esriInches: CalculatePageToMapRatio = dMetersPerUnit / (1 / 12 * 0.304800609601219)
            Case esriFeet: CalculatePageToMapRatio = dMetersPerUnit / (0.304800609601219)
            Case esriCentimeters: CalculatePageToMapRatio = dMetersPerUnit / (1 / 100)
            Case esriMeters: CalculatePageToMapRatio = dMetersPerUnit / (1)
            Case Else:
268:                 MsgBox "Warning: Only the following Page (Layout) Units are supported by this tool:" _
                    & vbCrLf & " - Inches, Feet, Centimeters, Meters" _
                    & vbCrLf & vbCrLf & "Calculating as though Page Units are in Inches..."
271:                 CalculatePageToMapRatio = dMetersPerUnit / (1 / 12 * 0.304800609601219)
272:         End Select
    ' Otherwise
274:     Else
        ' If not projected, we can only do a "flat" conversion -> that is, use the current scale and extent
        '  as a ratio to be applied to the map grid scale.
        ' NOTE: We MUST be in Layout mode to make this calculation, as the scale in Map View and Layout View
        '  are not the same (as the extent envelope and data frame envelope can be different shapes).  The
        '  test for being in Layout Mode is made in the clsMapGridButton.ICommand_Enabled property.
280:         Set pExtentEnv = pMx.ActiveView.Extent
281:         dStartX = pExtentEnv.XMin
282:         dStartY = pExtentEnv.YMin
283:         dEndX = pExtentEnv.XMax
284:         dEndY = pExtentEnv.YMax
        
286:         dCurrScale = pMx.FocusMap.MapScale
287:         If ((dEndX - dStartX) / m_dFrameWidthInPageUnits) > ((dEndY - dStartY) / m_dFrameHeightInPageUnits) Then
288:             CalculatePageToMapRatio = m_dFrameWidthInPageUnits / ((dEndX - dStartX) / dCurrScale)
289:         Else
290:             CalculatePageToMapRatio = m_dFrameHeightInPageUnits / ((dEndY - dStartY) / dCurrScale)
291:         End If
292:     End If
    
    Exit Function
eh:
296:     CalculatePageToMapRatio = 1
297:     MsgBox "Error in CalculatePageToMapRatio" & vbCrLf & Erl & " - " & Err.Description
End Function

Private Sub Class_Terminate()
301:     Set m_DestFL = Nothing
302:     Set m_DestFC = Nothing
303:     Set m_pProgress = New ModelessFrame
End Sub

Public Sub GenerateStripMap(Application As IApplication)
    Dim pMx As IMxDocument
    Dim pWorkspaceEdit As IWorkspaceEdit
    Dim lLoop As Long
    Dim pFeatDataset As IFeatureDataset
    Dim pFeature As IFeature
    Dim pDataset As IDataset
    Dim dIncrement As Double
    Dim pFL As IFeatureLayer
    Dim pFC As IFeatureClass
    Dim pProgress As frmProgress
    
    On Error GoTo eh
    ' Set mouse pointer
320:     Screen.MousePointer = vbArrowHourglass

    ' Init
323:     Set pMx = Application.Document
324:     For lLoop = 0 To pMx.FocusMap.LayerCount - 1
325:         If TypeOf pMx.FocusMap.Layer(lLoop) Is IFeatureLayer Then
326:             If UCase(pMx.FocusMap.Layer(lLoop).Name) = UCase(m_DestFL.Name) Then
327:                 Set pFL = pMx.FocusMap.Layer(lLoop)
328:                 Exit For
329:             End If
330:         End If
331:     Next
332:     If pFL Is Nothing Then
333:         MsgBox "No match in the Map for layer '" & m_DestFL.Name & "'."
        Exit Sub
335:     End If
336:     Set pFC = pFL.FeatureClass
    ' Check for required fields - that the field exists
    Dim bErrorWithFields As Boolean
339:     bErrorWithFields = (pFC.FindField(m_FldStripName) < 0)
340:     bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldNumInSeries) < 0)
341:     bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldMapAngle) < 0)
342:     If Len(m_FldScale) > 0 Then bErrorWithFields = bErrorWithFields Or (pFC.FindField(m_FldScale) < 0)
    ' If error
344:     If bErrorWithFields Then
345:         Err.Raise vbObjectError, "GenerateStripMap", "Could not find all the given field names in " & pFL.Name & "." _
            & vbCrLf & " - " & m_FldStripName & ", " & m_FldNumInSeries & ", " & m_FldMapAngle & ", " & m_FldScale
347:     End If
    ' Check the field types
349:     bErrorWithFields = (pFC.Fields.Field(pFC.FindField(m_FldStripName)).Type <> esriFieldTypeString)
350:     bErrorWithFields = bErrorWithFields Or _
       ((pFC.Fields.Field(pFC.FindField(m_FldNumInSeries)).Type <> esriFieldTypeDouble) And _
        (pFC.Fields.Field(pFC.FindField(m_FldNumInSeries)).Type <> esriFieldTypeInteger) And _
        (pFC.Fields.Field(pFC.FindField(m_FldNumInSeries)).Type <> esriFieldTypeSingle) And _
        (pFC.Fields.Field(pFC.FindField(m_FldNumInSeries)).Type <> esriFieldTypeSmallInteger))
355:     bErrorWithFields = bErrorWithFields Or _
       ((pFC.Fields.Field(pFC.FindField(m_FldMapAngle)).Type <> esriFieldTypeDouble) And _
        (pFC.Fields.Field(pFC.FindField(m_FldMapAngle)).Type <> esriFieldTypeInteger) And _
        (pFC.Fields.Field(pFC.FindField(m_FldMapAngle)).Type <> esriFieldTypeSingle) And _
        (pFC.Fields.Field(pFC.FindField(m_FldMapAngle)).Type <> esriFieldTypeSmallInteger))
360:     If Len(m_FldScale) > 0 Then
361:         bErrorWithFields = bErrorWithFields Or _
           ((pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeDouble) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeInteger) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeSingle) And _
            (pFC.Fields.Field(pFC.FindField(m_FldScale)).Type <> esriFieldTypeSmallInteger))
366:     End If
    ' if error
368:     If bErrorWithFields Then
369:         Err.Raise vbObjectError, "GenerateStripMap", "Given field names are not of the correct type." _
            & vbCrLf & "Strip Map Name field must be a Text field, all others must be numeric fields."
371:     End If
    ' Get the dataset and workspace (to start editing upon)
373:     Set pFeatDataset = pFC.FeatureDataset
374:     If Not pFeatDataset Is Nothing Then
375:         Set pWorkspaceEdit = pFeatDataset.Workspace
376:     Else
        ' Is a shapefile, go via just IDataset
378:         Set pDataset = pFC
379:         Set pWorkspaceEdit = pDataset.Workspace
380:     End If
    
'    ' If replacing, delete all existing polygons
383:     Set pProgress = New frmProgress
384:     m_pProgress.Create pProgress
385:     pProgress.ProgressBar1.Min = 0
386:     pProgress.ProgressBar1.Max = 100
387:     pProgress.ProgressBar1.value = 0
388:     If m_RemoveGrids Then
        Dim pFCu As IFeatureCursor
        Dim pT As ITable
391:         Set pFCu = m_DestFL.Search(Nothing, False)
392:         Set pT = m_DestFL.FeatureClass
393:         If pT.RowCount(Nothing) = 0 Then
394:             dIncrement = 99
395:         Else
396:             dIncrement = 100 / pT.RowCount(Nothing)
397:         End If
398:         pProgress.lblInformation.Caption = "Deleting previous grids..."
399:         pProgress.cmdCancel.Visible = False        ' User cannot cancel this step
400:         m_pProgress.Visible = True
401:         Set pFeature = pFCu.NextFeature
402:         While Not pFeature Is Nothing
403:             pFeature.Delete
404:             If (pProgress.ProgressBar1.value + dIncrement) <= pProgress.ProgressBar1.Max Then
405:                 pProgress.ProgressBar1.value = pProgress.ProgressBar1.value + dIncrement
406:             Else
407:                 pProgress.lblInformation.Caption = "Warning: Val > Max (" & pProgress.ProgressBar1.Max & ")"
408:             End If
409:             Set pFeature = pFCu.NextFeature
410:         Wend
411:         m_pProgress.Visible = False
412:     End If
  
    ' Init strip map stuff
    
    ' Flip, if required
417:     If m_Flip Then
418:         m_Polyline.ReverseOrientation
419:     End If
    ' Get the progress bar ready
421:     pProgress.ProgressBar1.Min = 0
422:     pProgress.ProgressBar1.Max = 101
423:     pProgress.ProgressBar1.value = 0
424:     pProgress.lblInformation.Caption = "Creating strip map..."
425:     pProgress.cmdCancel.Visible = True        ' User cannot cancel this step
    ' Get map units size for grids
    Dim dConvertPageToMapUnits As Double
428:     dConvertPageToMapUnits = CalculatePageToMapRatio(Application)
429:     m_GridWidth = ((m_dMapScale * m_dFrameWidthInPageUnits) / dConvertPageToMapUnits)
430:     m_GridHeight = ((m_dMapScale * m_dFrameHeightInPageUnits) / dConvertPageToMapUnits)
    
  'Call one of the helper functions to Generate the Strip Map
433:   If m_LoadStationsFromTable = True Then
434:     GenerateStripMapHelper2_GenerateFromStationing pFC, pProgress, pWorkspaceEdit, Application
435:   Else
436:     GenerateStripMapHelper pFC, pProgress, pWorkspaceEdit, m_Polyline
437:   End If
  
  'Refresh the Table of Contents
440:   pMx.CurrentContentsView.Refresh Null
  'Refresh the Active View
442:   pMx.ActiveView.Refresh
  'Restore the mouse pointer
444:   Screen.MousePointer = vbDefault
  Exit Sub
eh:
447:     MsgBox "Error in GenerateStripMap - " & Erl & " - " & Err.Description
End Sub

'This function uses stationing values from a table to generate a strip map series.
Private Sub GenerateStripMapHelper2_GenerateFromStationing(pFC As IFeatureClass, _
pProgress As frmProgress, pWorkspaceEdit As IWorkspaceEdit, pApp As IApplication)
On Error GoTo eh
  'Check to make sure that the Table name and column names are set. (If not, raise an error.)
  Dim bErrorWithStationsTable As Boolean
456:   bErrorWithStationsTable = Len(m_StationValuesTable) < 0
457:   bErrorWithStationsTable = bErrorWithStationsTable Or Len(m_StationValuesFROMColumn) < 0
458:   bErrorWithStationsTable = bErrorWithStationsTable Or Len(m_StationValuesTOColumn) < 0
459:   If bErrorWithStationsTable Then
460:     Err.Raise vbObjectError, "GenerateStripMapHelper2_GenerateFromStationing", _
    "One or more of the following names were not set:" & vbCrLf _
    & "Station Table: " & m_StationValuesTable & "   FROM Column: " & m_StationValuesFROMColumn _
    & "   TO Column: " & m_StationValuesTOColumn
464:   End If
  
  'Check if the stationing table is found in the map. (If not, raise an error.)
  Dim pStationingDataset As IDataset
468:   Set pStationingDataset = modGeneralFunctions.FindDatasetByName(m_StationValuesTable, pApp)
469:   If pStationingDataset Is Nothing Then
470:     Err.Raise vbObjectError, "GenerateStripMapHelper2_GenerateFromStationing", _
    "The stationing table was not found in the map:" & vbCrLf _
    & "Station Table: " & m_StationValuesTable
473:   End If
  
  'Check if the stationing table contains the FROM and TO columns. (If not, raise an error.)
  Dim pTable As ITable
  Dim pFields As IFields
  Dim lFROMColumnIndex As Long
  Dim lTOColumnIndex As Long
480:   Set pTable = pStationingDataset
481:   Set pFields = pTable.Fields
482:   lFROMColumnIndex = pFields.FindField(m_StationValuesFROMColumn)
483:   If lFROMColumnIndex = -1 Then
484:     Err.Raise vbObjectError, "GenerateStripMapHelper2_GenerateFromStationing", _
    "The FROM Column was not found in the Stationing Dataset." & vbCrLf _
    & "FROM Column: " & m_StationValuesFROMColumn
487:   End If
488:   lTOColumnIndex = pFields.FindField(m_StationValuesTOColumn)
489:   If lTOColumnIndex = -1 Then
490:     Err.Raise vbObjectError, "GenerateStripMapHelper2_GenerateFromStationing", _
    "The TO Column was not found in the Stationing Dataset." & vbCrLf _
    & "TO Column: " & m_StationValuesTOColumn
493:   End If

  'Create Feature Buffer and Feature Cursor
  Dim pInsertFeatureBuffer As IFeatureBuffer
  Dim pInsertFeatureCursor As IFeatureCursor
498:   Set pInsertFeatureCursor = pFC.Insert(True)
499:   Set pInsertFeatureBuffer = pFC.CreateFeatureBuffer

501:   m_pProgress.Visible = True 'Display the progress bar dialog

  'Loop through each of the records in the stationing table,
  'and create a corresponding record in the index polygon feature class.
  Dim pCursor As ICursor
  Dim pRow As IRow
  Dim dFromStation As Double, dToStation As Double
  Dim lPageNumber As Long
  Dim vUserChoice As Variant
  Dim lTotalNumPages As Long
  
  'Query the number of records in the stations table.
513:   lTotalNumPages = GetNumberOfRows(m_StationValuesTable, pApp)
  
515:   lPageNumber = 1
516:   Set pCursor = pTable.Search(Nothing, False)
517:   Set pRow = pCursor.NextRow
518:   Do Until pRow Is Nothing
519:     dFromStation = pRow.value(pRow.Fields.FindField(m_StationValuesFROMColumn))
520:     dToStation = pRow.value(pRow.Fields.FindField(m_StationValuesTOColumn))
    
    'Call GenerateIndexPolygonFeature_FromStationTable subroutine
    'to generate the index polygon.
524:     GenerateIndexPolygonFeature2_GenerateFromStationing pFC, dFromStation, dToStation, pInsertFeatureBuffer, lPageNumber
    
    'Insert the new Index Polygon Feature
527:     pInsertFeatureCursor.InsertFeature pInsertFeatureBuffer
    
    'Update the progress bar
530:     If pProgress.ProgressBar1.value <= pProgress.ProgressBar1.Max Then
        'Use integer division to calculate an integer between 1 and 100 that represents % completion.
        Dim iPercentComplete As Integer
533:         iPercentComplete = ((lPageNumber * 100) \ lTotalNumPages)
534:         pProgress.lblInformation.Caption = "Creating strip map... " & iPercentComplete & "% complete"
535:         pProgress.ProgressBar1.value = iPercentComplete
536:     Else
537:         pProgress.lblInformation.Caption = "Warning: Val > Max (" & pProgress.ProgressBar1.Max & ")"
538:         pProgress.ProgressBar1.value = pProgress.ProgressBar1.Max
539:     End If

    'Give some CPU time to the user interface so that the user can click cancel.
542:     DoEvents

    'Commit the updates to the database after every 20 pages.
545:     If (lPageNumber Mod 20 = 0) Then
546:         pInsertFeatureCursor.Flush
547:     End If

    'Refresh the progress bar
550:     pProgress.Refresh
    
    'Check if the user clicked cancel.
553:     If pProgress.Cancelled Then
554:         pProgress.Cancelled = False       ' Reset the form
555:         vUserChoice = MsgBox("Operation cancelled." _
            & "  Save the edits made thus far?" & vbCrLf & vbCrLf _
            & "(Click Cancel to continue processing)", _
                        vbYesNoCancel, "Generate Strip Map")
559:         If vUserChoice <> vbCancel Then
560:             GoTo CancelledGenerateGrids
561:         End If
562:     End If

    'Increment the cursor and page number
565:     Set pRow = pCursor.NextRow
566:     lPageNumber = lPageNumber + 1
567:   Loop
  
  ' Stop editing
570:   pWorkspaceEdit.StopEditOperation
571:   pWorkspaceEdit.StopEditing True
  Exit Sub
CancelledGenerateGrids:
574:   m_pProgress.Visible = False
575:   If vUserChoice = vbYes Then
576:       pInsertFeatureCursor.Flush
577:       pWorkspaceEdit.StopEditOperation
578:       pWorkspaceEdit.StopEditing True
579:   Else
580:       pWorkspaceEdit.StopEditOperation
581:       pWorkspaceEdit.StopEditing False
582:   End If
  Exit Sub
eh:
585:   MsgBox "Error in GenerateStripMapHelper2_GenerateFromStationing - " & Erl & " - " & Err.Description
End Sub

'This function retrieves a point along a polyline at a certain M value.
'Z values are ignored.
Private Function GetPointAlongPolyline(pPolyline As IPolyline, dM As Double) As IPoint
On Error GoTo eh
  Dim pOutVertex As IPoint
  Dim pBeforePoint As IPoint
  Dim pAfterPoint As IPoint
  Dim pReturnPoint As IPoint
  Dim lOutPart As Long
  Dim lOutVertex As Long
  Dim pPointCollection As IPointCollection
  Dim pEnumVertex As IEnumVertex2
  Dim bExactMatch As Boolean
  Dim bBeforePointFound As Boolean
  Dim bAfterPointFound As Boolean
  
604:   bExactMatch = False
605:   bBeforePointFound = False
606:   bAfterPointFound = False
  
608:   Set pOutVertex = New esriGeometry.Point 'Create a new point.  This may not be a required step.
609:   Set pPointCollection = pPolyline
610:   Set pEnumVertex = pPointCollection.EnumVertices
  
  'Reset the Enum
613:   pEnumVertex.SetAt 0, 0
'  pEnumVertex.Reset
  
'  Dim iPart As Integer, iVertex As Integer

  'Get the first vertex
619:   pEnumVertex.Next pOutVertex, lOutPart, lOutVertex
'  iPart = lOutPart
'  iVertex = lOutVertex
'  MsgBox "iPart: " & iPart & vbNewLine & "iVertex: " & iVertex & vbNewLine & "dM: " & Str(dM) & vbNewLine & "pOutVertex.m: " & pOutVertex.m
  
  'Do While (Not pOutVertex Is Nothing) And (iVertex < 10)
625:   Do While (Not pOutVertex Is Nothing)
626:     If (Abs(pOutVertex.m - dM) < 0.000001) Then
627:       bExactMatch = True
      'Set pReturnPoint = pOutVertex
      'Exit Do
630:       Set GetPointAlongPolyline = pOutVertex 'Return the point
      Exit Function
      
633:     ElseIf (pOutVertex.m < dM) Then
      
635:       If bBeforePointFound = False Then
636:         Set pBeforePoint = pOutVertex
637:         bBeforePointFound = True
638:       ElseIf pOutVertex.m > pBeforePoint.m Then 'Only replace the before point with the new point if the new point has a greater M value.
639:         Set pBeforePoint = pOutVertex
640:       End If
    
642:     ElseIf (pOutVertex.m > dM) Then
      
644:       If bAfterPointFound = False Then
645:         Set pAfterPoint = pOutVertex
646:         bAfterPointFound = True
647:       ElseIf pOutVertex.m < pAfterPoint.m Then 'Only replace the afte point with the new point if the new point has a smaller M value.
648:         Set pAfterPoint = pOutVertex
649:       End If
      
      'MsgBox "After Point:" & vbNewLine & "dM: " & Str(dM) & vbNewLine & "pOutVertex.m: " & pOutVertex.m
      'Exit Do
653:     End If
    'Increment to the next vertex.
655:     pEnumVertex.Next pOutVertex, lOutPart, lOutVertex
    
'    'Check which part we are in and what vertex we are in
'573:     If (lOutPart <> iPart) Then
'574:       iPart = lOutPart
'575:       iVertex = lOutVertex
'576:       MsgBox "iPart: " & iPart & vbNewLine & "iVertex: " & iVertex & vbNewLine & "dM: " & Str(dM) & vbNewLine & "pOutVertex.m: " & pOutVertex.m
'577:     End If
    
664:   Loop
  
666:   If bExactMatch Then
667:   Else
668:     If bAfterPointFound = False Then 'If the point falls after the end point.
669:       Set pReturnPoint = pBeforePoint
670:     ElseIf bBeforePointFound = False Then 'If the point falls before the start point
671:       Set pReturnPoint = pAfterPoint
672:     Else 'If the point falls somewhere along the line
      Dim dBeforeM As Double, dAfterM As Double
      Dim dWeighted_Distance As Double
      Dim dInterpX As Double, dInterpY As Double
676:       dBeforeM = pBeforePoint.m
677:       dAfterM = pAfterPoint.m
678:       dWeighted_Distance = (dM - dBeforeM) / (dAfterM - dBeforeM)
679:       dInterpX = pBeforePoint.X + (dWeighted_Distance * (pAfterPoint.X - pBeforePoint.X))
680:       dInterpY = pBeforePoint.Y + (dWeighted_Distance * (pAfterPoint.Y - pBeforePoint.Y))
      
682:       Set pReturnPoint = New esriGeometry.Point
683:       pReturnPoint.X = dInterpX
684:       pReturnPoint.Y = dInterpY
685:       pReturnPoint.m = dM
686:     End If
687:   End If

'    'Interpolate to find a point between the upper and lower bounding points
'    MsgBox "pBeforePoint(M,X,Y): " & pBeforePoint.m & ", " & pBeforePoint.X & ", " & pBeforePoint.Y & vbNewLine _
'    & "pAfterPoint(M,X,Y): " & pAfterPoint.m & ", " & pAfterPoint.X & ", " & pAfterPoint.Y & vbNewLine _
'    & "pInterpPoint(M,X,Y): " & pInterpPoint.m & ", " & pInterpPoint.X & ", " & pInterpPoint.Y

694:   Set GetPointAlongPolyline = pReturnPoint 'Return the point
  Exit Function
eh:
697:   MsgBox "Error in GetPointAlongPolyline - " & Erl & " - " & Err.Description
End Function


'This function generates in index polygon based on a FROM station and a TO station value.
Private Sub GenerateIndexPolygonFeature2_GenerateFromStationing(pFC As IFeatureClass, dFromStation As Double, dToStation As Double, _
pInsertFeatureBuffer As IFeatureBuffer, ByRef lPageNumber As Long)
On Error GoTo eh
  Dim pSubPolyline As IPolyline
  Dim pFromPoint As IPoint
  Dim pToPoint As IPoint
      
709:   Set pFromPoint = GetPointAlongPolyline(m_Polyline, dFromStation)
710:   Set pToPoint = GetPointAlongPolyline(m_Polyline, dToStation)
      
  'Get X and Y coordinates from the FROM and TO points
  Dim dFromX As Double
  Dim dFromY As Double
  Dim dFromZ As Double
  Dim dFromM As Double
  Dim dToX As Double
  Dim dToY As Double
  Dim dToZ As Double
  Dim dToM As Double
    
722:   dFromX = pFromPoint.X
723:   dFromY = pFromPoint.Y
724:   dFromZ = pFromPoint.Z
725:   dFromM = pFromPoint.m
726:   dToX = pToPoint.X
727:   dToY = pToPoint.Y
728:   dToZ = pToPoint.Z
729:   dToM = pToPoint.m
    
  'Calculate the segment straight-line length
  Dim dSegmentLength As Double
  Dim dDX As Double, dDY As Double
734:   dDX = dFromX - dToX
735:   dDY = dFromY - dToY
736:   dSegmentLength = Sqr((dDX * dDX) + (dDY * dDY))
  
  'Build a grid Polygon (and get the rotation angle).
  Dim pPolygon As IPolygon
  Dim dRotationAngle As Double
741:   CreateAngledGridPolygon2_Buffered pFromPoint, pToPoint, pPolygon, dRotationAngle, m_GridHeight, m_GridWidth
  
  'Test if the polyline segment fits entirely within the grid polygon
  Dim bSegmentFitsInsideGrid As Boolean
  Dim sSegmentFitsInsideGrid As String
746:   bSegmentFitsInsideGrid = DoesSegmentFitInsideGrid(m_Polyline, pPolygon, pFromPoint, pToPoint)
747:   If bSegmentFitsInsideGrid = False Then
748:     sSegmentFitsInsideGrid = "N"
'MsgBox "The segment does not fit within the grid polygon." & vbNewLine _
& "PageNumber: " & lPageNumber & vbNewLine _
& "From: " & dFromStation & vbNewLine & "To: " & dToStation
752:   Else
753:     sSegmentFitsInsideGrid = "Y"
754:   End If
  
'  MsgBox "lPageNumber: " & lPageNumber & vbNewLine _
'  & "FROM X, Y, Z, M: " & dFromX & ", " & dFromY & ", " & dFromZ & ", " & dFromM & vbNewLine _
'  & "TO X,Y,Z,M: " & dToX & ", " & dToY & ", " & dToZ & ", " & dToM & vbNewLine _
'  & "dRotationAngle: " & dRotationAngle & "   dSegmentLength: " & dSegmentLength
  
  'Get the page number and left-pad it with zeros.
  Dim sMapCount As String
763:   sMapCount = CStr(lPageNumber)
764:   While (Len(sMapCount) < 4) '4 digits supports up to 9999 pages.
765:     sMapCount = "0" + sMapCount
766:   Wend
  
  'Create the new index polygon feature
769:   Set pInsertFeatureBuffer.Shape = pPolygon
770:   pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldStripName)) = m_StripMapName & " " & sMapCount
771:   pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldNumInSeries)) = lPageNumber
772:   pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldMapAngle)) = dRotationAngle 'degrees
773:   If Len(m_FldScale) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldScale)) = m_dMapScale
774:   If m_FldFromStation <> "" Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldFromStation)) = dFromM
775:   If m_FldToStation <> "" Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldToStation)) = dToM
776:   If m_FldInGrid <> "" Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldInGrid)) = sSegmentFitsInsideGrid

  Exit Sub
eh:
780:   MsgBox "Error in GenerateIndexPolygonFeature2_GenerateFromStationing - " & Erl & " - " & Err.Description
End Sub

'This function determines if a polyline travels through a grid polygon.
'Returns true if the polyline intersects the polygon in exactly 2 places.
'Also returns true if the polyline intersects the polygon in 1 place,
'becase that means that one of the ends points of the polyline resides
'within the polygon.
Private Function DoesSegmentFitInsideGrid(pPolyline As IPolyline, pPolygon As IPolygon, pFromPoint As IPoint, pToPoint As IPoint) As Boolean
On Error GoTo eh
  Dim pTopoOpt As ITopologicalOperator
  Dim pGeoCol As IGeometryCollection
  Dim pSubCurve As ICurve
  Dim dFromDistance As Double, dToDistance As Double
  Dim pOutPoint As IPoint, dFC As Double, bRS As Boolean
  
  'Get the distance along the curve of the FROM and TO points.
797:   pPolyline.QueryPointAndDistance esriNoExtension, pFromPoint, False, pOutPoint, dFromDistance, dFC, bRS
798:   pPolyline.QueryPointAndDistance esriNoExtension, pToPoint, False, pOutPoint, dToDistance, dFC, bRS
  
  'Get the sub-curve between the FROM and TO points.
801:   pPolyline.GetSubcurve dFromDistance, dToDistance, False, pSubCurve
  
  'Determine if the subcurve intersects the polygon.  If so, return false
804:   Set pTopoOpt = pSubCurve
805:   Set pGeoCol = pTopoOpt.Intersect(pPolygon, esriGeometry0Dimension)
806:   If (pGeoCol.GeometryCount > 0) Then
807:     DoesSegmentFitInsideGrid = False
    Exit Function
809:   End If
  
  'Determine if the polyline intersects the polygon in at least one place.
  'If so, return true.  Otherwise, return false.
  'If it does not intersect, than the page is in the wrong location.
814:   Set pTopoOpt = pPolyline
815:   Set pGeoCol = pTopoOpt.Intersect(pPolygon, esriGeometry0Dimension)
816:   If (pGeoCol.GeometryCount > 0) Then
817:     DoesSegmentFitInsideGrid = True
818:   Else
819:     DoesSegmentFitInsideGrid = False
820:   End If

  Exit Function
eh:
824:   MsgBox "Error in SegmentFitsInsideGrid - " & Erl & " - " & Err.Description
End Function

Private Sub GenerateStripMapHelper(pFC As IFeatureClass, _
pProgress As frmProgress, pWorkspaceEdit As IWorkspaceEdit, pPolyline As IPolyline)
    On Error GoTo eh
    Dim dHighest As Double
    Dim dHighestPrev As Double
    Dim pInsertFeatureBuffer As IFeatureBuffer
    Dim pInsertFeatureCursor As IFeatureCursor
    Dim vUserChoice As Variant
    Dim lPageNumber As Long
        
    'Initialize the page number
838:     lPageNumber = 1
    'Initialize the feature cursor and feature buffer
840:     Set pInsertFeatureCursor = pFC.Insert(True)
841:     Set pInsertFeatureBuffer = pFC.CreateFeatureBuffer
    'Initialize the progress bar
843:     m_pProgress.Visible = True
        
    'Loop through each of the parts of the polyline
    Dim pGeometryCollection As IGeometryCollection
    Dim iGeomCount As Integer
    Dim pPath As IPath
    Dim pSinglePartPolyline As IPolyline
    Dim pSinglePartGeometry As IGeometryCollection
    Dim iPartNum As Integer
852:     Set pGeometryCollection = pPolyline
853:     iGeomCount = pGeometryCollection.GeometryCount
    If iGeomCount = 0 Then Exit Sub
    'Loop through each of the polyline parts
856:     For iPartNum = 0 To iGeomCount - 1
857:         Set pPath = pGeometryCollection.Geometry(iPartNum)
        'Generate a single part polyline from the path
859:         Set pSinglePartPolyline = New esriGeometry.Polyline
860:         Set pSinglePartGeometry = pSinglePartPolyline
861:         pSinglePartGeometry.AddGeometry pPath
        
        'Set the "From Point" of the first index polygon
        Dim pFromPoint As IPoint
        Dim pToPoint As IPoint
866:         Set pFromPoint = pSinglePartPolyline.FromPoint
867:         Set pToPoint = New esriGeometry.Point
        
        ' Initialize the highest previous value
870:         dHighestPrev = -1

872:         Do '***Start of loop Through Each polygon in a polyline part***.
            'Generate an Index Polygon Feature
874:             GenerateIndexPolygonFeature pFC, pFromPoint, pToPoint, dHighest, dHighestPrev, _
              pInsertFeatureBuffer, lPageNumber, pSinglePartPolyline
    
            'If dHighestPrev = dHighest the loop is stuck.  dHighest will never reach 100%.  Exit the Do loop.
878:             If dHighestPrev = dHighest Then Exit Do
    
            'Set the dHighestPrevious for the next time through the outer loop.
881:             dHighestPrev = dHighest
            'Set the "From Point" for the next time through the outer loop.
883:             Set pFromPoint = pToPoint
    
            'Insert the new Index Polygon Feature
886:             pInsertFeatureCursor.InsertFeature pInsertFeatureBuffer
    
            'Update the progress bar
889:             If dHighest <= pProgress.ProgressBar1.Max Then
890:                 pProgress.lblInformation.Caption = "Creating strip map...(Part " & iPartNum + 1 & " of " _
                     & iGeomCount & ") " & CInt(dHighest - 1) & "% complete"
892:                 pProgress.ProgressBar1.value = dHighest
893:             Else
894:                 pProgress.lblInformation.Caption = "Warning: Val > Max (" & pProgress.ProgressBar1.Max & ")"
895:                 pProgress.ProgressBar1.value = pProgress.ProgressBar1.Max
896:             End If
            'Allow the user interface some CPU time so that the user may click cancel.
898:             DoEvents
            'Commit the updates to the database.
900:             pInsertFeatureCursor.Flush
            'Refresh the progress bar
902:             pProgress.Refresh
            'Check if the user clicked cancel.
904:             If pProgress.Cancelled Then
905:                 pProgress.Cancelled = False       ' Reset the form
906:                 vUserChoice = MsgBox("Operation cancelled." _
                    & "  Save the edits made thus far?" & vbCrLf & vbCrLf _
                    & "(Click Cancel to continue processing)", _
                                vbYesNoCancel, "Generate Strip Map")
910:                 If vUserChoice <> vbCancel Then
911:                     GoTo CancelledGenerateGrids     'Sorry for GoTo usage - in a hurry
912:                 End If
913:             End If
            
            'Increment the page number
916:             lPageNumber = lPageNumber + 1
917:         Loop While dHighest < 100 '***End of loop Through Each polygon in a polyline part***
918:     Next iPartNum 'End of looping through polyline parts.

    'Flush any outstanding buffered writes to the database
921:     pInsertFeatureCursor.Flush
    'Hide the progress bar
923:     m_pProgress.Visible = False
    ' Stop editing
925:     pWorkspaceEdit.StopEditOperation
926:     pWorkspaceEdit.StopEditing True
    Exit Sub
    
CancelledGenerateGrids:
930:     m_pProgress.Visible = False
931:     If vUserChoice = vbYes Then
932:         pInsertFeatureCursor.Flush
933:         pWorkspaceEdit.StopEditOperation
934:         pWorkspaceEdit.StopEditing True
935:     Else
936:         pWorkspaceEdit.StopEditOperation
937:         pWorkspaceEdit.StopEditing False
939:     End If
    Exit Sub
    
942:     Resume
eh:
944:     MsgBox "Error in GenerateStripMapHelper - " & Erl & " - " & Err.Description
End Sub

Private Sub GenerateIndexPolygonFeature(pFC As IFeatureClass, ByVal pFromPoint As IPoint, ByRef pToPoint As IPoint, _
  ByRef dHighest As Double, ByVal dHighestPrev As Double, ByRef pInsertFeatureBuffer As IFeatureBuffer, _
  ByRef lPageNumber As Long, pPolyline As IPolyline)
  'ByRef bFirstRun As Boolean,
On Error GoTo eh
    Dim pCirclePoly As IPolygon
    Dim pGridPoly As IPolygon
    Dim pCircularArc As IConstructCircularArc
    Dim pSegmentCollection As ISegmentCollection
    Dim pTopoOpt As ITopologicalOperator
    Dim pGeoCol As IGeometryCollection
    Dim lLoop2 As Long
    Dim lHighestRef As Long
    Dim bContinue As Boolean
    Dim dGridAngle As Double
    Dim bReducedRadius As Boolean
    Dim dHighestThisTurn As Double
    Dim pArc As ICurve
    Dim dCircleRadius As Double
    Dim colIntersects As Collection
    Dim dIntersect As Double
    Dim pPointBeforeFromPoint As IPoint
    Dim pPointAfterToPoint As IPoint

    'Query Interface from IPolyline to ICurve
972:     Set pArc = pPolyline
973:     dCircleRadius = m_GridWidth
    
975:     bReducedRadius = False
976:     Do '***Start Inner Loop.***
        ' Create the search circle
978:         Set pCircularArc = New CircularArc
979:         pCircularArc.ConstructCircle pFromPoint, dCircleRadius, False 'make it clockwise
980:         Set pCirclePoly = New Polygon
981:         Set pSegmentCollection = pCirclePoly
982:         pSegmentCollection.AddSegment pCircularArc
        
        ' Intersect the polyline and the circle
985:         Set pTopoOpt = pPolyline
986:         Set pGeoCol = New GeometryBag
987:         Set pGeoCol = pTopoOpt.Intersect(pCirclePoly, esriGeometry0Dimension)
        
989:         If pGeoCol.GeometryCount = 0 Then
990:             MsgBox "error - no geoms intersected"
            Exit Sub
992:         End If
993:         lHighestRef = -1
994:         dHighestThisTurn = 102 '(A fake percentage greater than 100%)
995:         For lLoop2 = 0 To pGeoCol.GeometryCount - 1
996:             Set pToPoint = pGeoCol.Geometry(lLoop2)
997:             dIntersect = ReturnPercentageAlong(pArc, pToPoint)
998:             If dIntersect > (dHighestPrev * 1.001) And dIntersect < dHighestThisTurn Then
999:                 dHighest = dIntersect
1000:                 dHighestThisTurn = dIntersect
1001:                 lHighestRef = lLoop2
1002:             End If
1003:         Next
        ' If no intersection higher than our previous run, we are at the end.
1005:         If lHighestRef < 0 Then
1006:             dHighest = 101 '(A fake percentage greater than 100%)
            ' Need to extend the end (tangent) to get intersection
            Dim pExtendedToPoint As IPoint
1009:             Set pExtendedToPoint = IntersectPointExtendedTo(pPolyline, pCirclePoly)
1010:             If Not (pExtendedToPoint Is Nothing) Then
1011:                 Set pToPoint = pExtendedToPoint
1012:             Else
1013:                 MsgBox "An error occured while attempting to generate an index Grid Polygon.", vbExclamation, "Error in GenerateIndexPolygonFeature"
                Exit Sub
1015:             End If
        ' Otherwise, still in the middle somewhere
1017:         Else
1018:             Set pToPoint = pGeoCol.Geometry(lHighestRef)
1019:         End If
        ' Make our grid polygon, allowing for any 'shrunken' grids
1021:         If bReducedRadius Then
            Dim pTmpPLine As IPolyline
            Dim pTmpCPoly As IPolygon
            Dim pTmpIntPoint As IPoint
1025:             Set pCircularArc = New CircularArc
1026:             pCircularArc.ConstructCircle pFromPoint, m_GridWidth, False 'make it clockwise
1027:             Set pTmpCPoly = New Polygon
1028:             Set pSegmentCollection = pTmpCPoly
1029:             pSegmentCollection.AddSegment pCircularArc
            
1031:             Set pTmpPLine = createPolylineM
1032:             pTmpPLine.FromPoint = pFromPoint
1033:             pTmpPLine.ToPoint = pToPoint
1034:             Set pTmpIntPoint = IntersectPointExtendedTo(pTmpPLine, pTmpCPoly)
1035:             CreateAngledGridPolygon pFromPoint, pTmpIntPoint, pGridPoly, dGridAngle, m_GridHeight
1036:         Else
1037:             CreateAngledGridPolygon pFromPoint, pToPoint, pGridPoly, dGridAngle, m_GridHeight
1038:         End If
        ' Now, we potentially need to reprocess if the route dips out of our grid
1040:         Set pTopoOpt = pGridPoly
1041:         Set pGeoCol = New GeometryBag
1042:         Set pGeoCol = pTopoOpt.Intersect(pPolyline, esriGeometry0Dimension)
1043:         bContinue = True
1044:         If pGeoCol.GeometryCount > 2 Then
1045:             Set colIntersects = New Collection
1046:             For lLoop2 = 0 To pGeoCol.GeometryCount - 1
1047:                 colIntersects.Add ReturnPercentageAlong(pArc, pGeoCol.Geometry(lLoop2))
1048:             Next
1049:             For lLoop2 = 1 To colIntersects.count
1050:                 If colIntersects.Item(lLoop2) > (dHighestPrev * 1.001) And colIntersects.Item(lLoop2) < (dHighest * 0.999) Then
1051:                     bContinue = False
1052:                     dHighest = dHighestPrev
1053:                     dCircleRadius = dCircleRadius - (m_GridWidth * 0.1)
1054:                     bReducedRadius = True
1055:                     If dCircleRadius <= 0 Then
1056:                         bContinue = True
1057:                     End If
1058:                     Exit For
1059:                 End If
1060:             Next
1061:         End If
        ' If all OK and a reduced radius, look for a quick jump ahead
1063:         If bContinue And bReducedRadius Then 'And pGeoCol.GeometryCount <= 2 Then
            Dim dTmpHighest As Double
1065:             lHighestRef = -1
1066:             dTmpHighest = -1
1067:             For lLoop2 = 0 To pGeoCol.GeometryCount - 1
1068:                 Set pToPoint = pGeoCol.Geometry(lLoop2)
1069:                 dIntersect = ReturnPercentageAlong(pArc, pToPoint)
1070:                 If dIntersect > dTmpHighest Then
1071:                     dTmpHighest = dIntersect
1072:                     lHighestRef = lLoop2
1073:                 End If
1074:             Next
1075:             If lHighestRef >= 0 Then Set pToPoint = pGeoCol.Geometry(lHighestRef)
1076:             dHighest = dTmpHighest
1077:         End If
1078:     Loop Until bContinue '***End of Inner Loop***
    
    ' All OK to create our grid feature now (hopefully, anyway)
    
    'CreateGridFeaturesAsGraphics pGridPoly, lPageNumber, dGridAngle, Application 'AAA
            
    'Get the page number and left-pad it with zeros.
    Dim sMapCount As String
1086:     sMapCount = CStr(lPageNumber)
1087:     While (Len(sMapCount) < 4) '4 digits supports up to 9999 pages.
1088:       sMapCount = "0" + sMapCount
1089:     Wend
    
    ' Create new grid cell feature
1092:     Set pInsertFeatureBuffer.Shape = pGridPoly
1093:     pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldStripName)) = m_StripMapName & " " & sMapCount
1094:     pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldNumInSeries)) = lPageNumber
1095:     pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldMapAngle)) = dGridAngle 'degrees
1096:     If Len(m_FldScale) > 0 Then pInsertFeatureBuffer.value(pFC.Fields.FindField(m_FldScale)) = m_dMapScale

    'Set the Starting Station and Ending Station Attributes
    Dim atEnd As Boolean
1100:     atEnd = (dHighest = 102) '102 is a fake percentage value greater than 100%
1101:     GetStationEndPoints pInsertFeatureBuffer, pGridPoly, pPolyline, atEnd
    
  Exit Sub
eh:
1105:   MsgBox "Error in GenerateIndexPolygonFeature - " & Erl & " - " & Err.Description
End Sub

Private Sub CreateGridFeaturesAsGraphics(pGridPolygon As IPolygon, lIndex As Long, dAngle As Double, pApp As IApplication)
    Dim pPntColl As IPointCollection
    Dim pArea As IArea
    Dim pCentroid As IPoint
    Dim lLoop As Long
    
    ' Create graphics (TEST PHASE)
1115:     Set pPntColl = pGridPolygon
1116:     For lLoop = 0 To pPntColl.PointCount - 2
1117:         Perm_DrawLineFromPoints pPntColl.Point(lLoop), pPntColl.Point(lLoop + 1), pApp
1118:     Next
1119:     Perm_DrawLineFromPoints pPntColl.Point(0), pPntColl.Point(pPntColl.PointCount - 1), pApp
1120:     Set pArea = pGridPolygon
1121:     Set pCentroid = pArea.Centroid
1122:     Perm_DrawTextFromPoint pCentroid, CStr(lIndex), pApp, , , , , 8
1123:     pCentroid.Y = pCentroid.Y - (m_GridWidth / 3)
1124:     Perm_DrawTextFromPoint pCentroid, Format(dAngle / cPI * 180, "(#0.0)"), pApp, , , , , 8

End Sub

'Input:
'pArc - a polyline curve
'pPoint - a point along the curve.
'Returns: a double representing how far the point is along the curve, as a percentage (0-100).
Private Function ReturnPercentageAlong(ByVal pArc As ICurve, ByVal pPoint As IPoint) As Double
    Dim GeoCount As Long
    Dim pDistAlong As Double
    Dim pDist As Double
    Dim pRightSide As Boolean
    Dim pOutPt As IPoint
    Dim CompareDist As Double
    
    On Error GoTo ErrorHandler
    
1142:     CompareDist = 0
    'Find the distance along curve
1144:     Set pOutPt = New esriGeometry.Point
1145:     pArc.QueryPointAndDistance esriNoExtension, pPoint, True, pOutPt, _
                        pDistAlong, pDist, pRightSide
    
1148:     ReturnPercentageAlong = (pDistAlong * 100)
    Exit Function
ErrorHandler:
1151:     Err.Raise Err.Number, "ReturnPercentageAlong", "Error in ReturnPercentageAlong." _
        & vbCrLf & "Err " & Err.Number & ": " & Erl & " - " & Err.Description
End Function

'This function generates a rectangular grid polygon.
'Inputs:
'p1 and p2 - "From" and "To" point geometries.
'Outputs:
'ReturnedGrid - the generated grid polygon
'ReturnedAngleInDegrees - the angle of the generated polygon
'Module Level Variables: m_GridHeight - The height of the grid
'Global constants: cPI - The PI constant.
Private Sub CreateAngledGridPolygon(ByVal p1 As IPoint, ByVal p2 As IPoint, _
            ByRef ReturnedGrid As IPolygon, ByRef ReturnedAngleInDegrees As Double, ByVal pGridHeight As Double)
    Dim pPointColl As IPointCollection
    Dim pPointStart As IPoint
    Dim pPoint As IPoint
    Dim dAngleInRadians As Double
    Dim pLine As ILine
    
    On Error GoTo eh
    
    ' Init
1174:     Set pLine = New esriGeometry.Line
1175:     pLine.FromPoint = p1
1176:     pLine.ToPoint = p2
1177:     dAngleInRadians = pLine.Angle
1178:     If dAngleInRadians = 0 Then
1179:       ReturnedAngleInDegrees = 0
1180:     ElseIf dAngleInRadians > 0 Then
1181:       ReturnedAngleInDegrees = 360 - ((dAngleInRadians / cPI) * 180)
1182:     Else
1183:       ReturnedAngleInDegrees = Abs((dAngleInRadians / cPI) * 180)
1184:     End If
1185:     Set ReturnedGrid = New Polygon
1186:     Set pPointColl = ReturnedGrid
    ' POINT 1 -------------------------------------------
1188:     Set pPoint = New esriGeometry.Point
1189:     pPoint.PutCoords p1.X + (Sin(dAngleInRadians) * (pGridHeight / 2)), _
                     p1.Y - (Cos(dAngleInRadians) * (pGridHeight / 2))
1191:     pPointColl.AddPoint pPoint
1192:     Set pPointStart = pPoint
    ' POINT 2 -------------------------------------------
1194:     Set pPoint = New esriGeometry.Point
1195:     pPoint.PutCoords p1.X - (Sin(dAngleInRadians) * (pGridHeight / 2)), _
                     p1.Y + (Cos(dAngleInRadians) * (pGridHeight / 2))
1197:     pPointColl.AddPoint pPoint
    ' POINT 3 -------------------------------------------
1199:     Set pPoint = New esriGeometry.Point
1200:     pPoint.PutCoords p2.X - Sin(dAngleInRadians) * pGridHeight / 2, _
                     p2.Y + Cos(dAngleInRadians) * pGridHeight / 2
1202:     pPointColl.AddPoint pPoint
    ' POINT 4 -------------------------------------------
1204:     Set pPoint = New esriGeometry.Point
1205:     pPoint.PutCoords p2.X + Sin(dAngleInRadians) * pGridHeight / 2, _
                     p2.Y - Cos(dAngleInRadians) * pGridHeight / 2
1207:     pPointColl.AddPoint pPoint
    ' JOIN BACK TO FIRST (CLOSE POLYGON) ----------------
1209:     pPointColl.AddPoint pPointStart
    
    Exit Sub
eh:
1213:     Err.Raise Err.Number, Err.source, "Error in CreateAngledGridPolygon." _
        & "Err " & Err.Number & ": " & Erl & " - " & Err.Description
End Sub

'This function generates a rectangular grid polygon.
'This function differs from CreateAngledGridPolygon because it takes
'a Grid Width as input.
'Inputs:
'p1 and p2 - "From" and "To" point geometries.
'Outputs:
'ReturnedGrid - the generated grid polygon
'ReturnedAngleInDegrees - the angle of the generated polygon
'pGridHeight - The height of the grid
'pGridWidth - The width of the grid
'Global constants: cPI - The PI constant.
Private Sub CreateAngledGridPolygon2_Buffered(ByVal p1 As IPoint, ByVal p2 As IPoint, _
            ByRef ReturnedGrid As IPolygon, ByRef ReturnedAngleInDegrees As Double, _
            ByVal pGridHeight As Double, ByVal pGridWidth As Double)
    Dim pPointColl As IPointCollection
    Dim pPointStart As IPoint
    Dim pPoint As IPoint
    Dim dAngleInRadians As Double
    Dim pLine As ILine
    Dim pBufferedLine As ILine
    Dim pB1 As IPoint, pB2 As IPoint
    
    On Error GoTo eh
        
'1123:     MsgBox "P1(x,y): " & Str(p1.X) & ", " & Str(p1.Y) & vbNewLine _
    & "P2(x,y): " & Str(p2.X) & ", " & Str(p2.Y)
        
    'If the first and second point are the same, move the second point slightly to avoid a crash.
1245:     If p1.X = p2.X And p1.Y = p2.Y Then
1246:       p2.X = p2.X + 10
1247:       MsgBox "Warning, the From Point and To Point are the same:" & vbNewLine _
    & "P1(x,y): " & Str(p1.X) & ", " & Str(p1.Y) & vbNewLine _
    & "P2(x,y): " & Str(p2.X) & ", " & Str(p2.Y)
1250:     End If
        
    ' Init the line segment
1253:     Set pLine = New esriGeometry.Line
1254:     pLine.FromPoint = p1
1255:     pLine.ToPoint = p2

    'Find the angle
1258:     dAngleInRadians = pLine.Angle
1259:     If dAngleInRadians = 0 Then
1260:       ReturnedAngleInDegrees = 0
1261:     ElseIf dAngleInRadians > 0 Then
1262:       ReturnedAngleInDegrees = 360 - ((dAngleInRadians / cPI) * 180)
1263:     Else
1264:       ReturnedAngleInDegrees = Abs((dAngleInRadians / cPI) * 180)
1265:     End If
    
    'Find the buffer width
    Dim dBufferWidth As Double
1269:     dBufferWidth = (pGridWidth - pLine.length) \ 2
    
    'Buffer the line on both ends.  Buffer the From Point and the To Point
1272:     Set pB1 = New Point
1273:     Set pB2 = New Point
1274:     pLine.QueryPoint esriExtendTangents, 0 - dBufferWidth, False, pB1
1275:     pLine.QueryPoint esriExtendTangents, pLine.length + dBufferWidth, False, pB2

    'Find the horizontal offset
    Dim dHorizontalOffset As Double
    Dim dLessHalfPageHeight As Double
1280:     dHorizontalOffset = GetHorizontalOffset(m_Polyline, pLine)

'1196: MsgBox "Horizontal Offset: " & Str(dHorizontalOffset) & vbNewLine _
'    & "P1(x,y,m): " & Str(p1.X) & ", " & Str(p1.Y) & ", " & Str(p1.m) & vbNewLine _
'    & "P2(x,y,m): " & Str(p2.X) & ", " & Str(p2.Y) & ", " & Str(p2.m)

1286:     dLessHalfPageHeight = (pGridHeight / 2) * 0.95
1287:     If dHorizontalOffset > dLessHalfPageHeight Then dHorizontalOffset = dLessHalfPageHeight
1288:     If dHorizontalOffset < (dLessHalfPageHeight * -1) Then dHorizontalOffset = (dLessHalfPageHeight * -1)
    
    'Find the height of the portion of the page above and below the polyline
    Dim dUpperHeight As Double
    Dim dLowerHeight As Double

1294:     dUpperHeight = (pGridHeight / 2) + dHorizontalOffset
1295:     dLowerHeight = (pGridHeight / 2) - dHorizontalOffset

'1228: MsgBox "pGridWidth: " & pGridWidth & vbNewLine _
'& "pLine.length:" & pLine.length & vbNewLine _
'& "dBufferWidth:" & dBufferWidth & vbNewLine _
'& "pLine.length + dBufferWidth: " & pLine.length + dBufferWidth

    'Init the grid polygon
1303:     Set ReturnedGrid = New Polygon
1304:     Set pPointColl = ReturnedGrid
    ' POINT 1 -------------------------------------------
1306:     Set pPoint = New esriGeometry.Point
1307:     pPoint.PutCoords pB1.X + (Sin(dAngleInRadians) * dLowerHeight), _
                     pB1.Y - (Cos(dAngleInRadians) * dLowerHeight)
1309:     pPointColl.AddPoint pPoint
1310:     Set pPointStart = pPoint
    ' POINT 2 -------------------------------------------
1312:     Set pPoint = New esriGeometry.Point
1313:     pPoint.PutCoords pB1.X - (Sin(dAngleInRadians) * dUpperHeight), _
                     pB1.Y + (Cos(dAngleInRadians) * dUpperHeight)
1315:     pPointColl.AddPoint pPoint
    ' POINT 3 -------------------------------------------
1317:     Set pPoint = New esriGeometry.Point
1318:     pPoint.PutCoords pB2.X - Sin(dAngleInRadians) * dUpperHeight, _
                     pB2.Y + Cos(dAngleInRadians) * dUpperHeight
1320:     pPointColl.AddPoint pPoint
    ' POINT 4 -------------------------------------------
1322:     Set pPoint = New esriGeometry.Point
1323:     pPoint.PutCoords pB2.X + Sin(dAngleInRadians) * dLowerHeight, _
                     pB2.Y - Cos(dAngleInRadians) * dLowerHeight

'    ' POINT 1 -------------------------------------------
'1149:     Set pPoint = New esriGeometry.Point
'1150:     pPoint.PutCoords pB1.X + (Sin(dAngleInRadians) * (pGridHeight / 2)), _
'                     pB1.Y - (Cos(dAngleInRadians) * (pGridHeight / 2))
'1152:     pPointColl.AddPoint pPoint
'1153:     Set pPointStart = pPoint
'    ' POINT 2 -------------------------------------------
'1155:     Set pPoint = New esriGeometry.Point
'1156:     pPoint.PutCoords pB1.X - (Sin(dAngleInRadians) * (pGridHeight / 2)), _
'                     pB1.Y + (Cos(dAngleInRadians) * (pGridHeight / 2))
'1158:     pPointColl.AddPoint pPoint
'    ' POINT 3 -------------------------------------------
'1160:     Set pPoint = New esriGeometry.Point
'1161:     pPoint.PutCoords pB2.X - Sin(dAngleInRadians) * pGridHeight / 2, _
'                     pB2.Y + Cos(dAngleInRadians) * pGridHeight / 2
'1163:     pPointColl.AddPoint pPoint
'    ' POINT 4 -------------------------------------------
'1165:     Set pPoint = New esriGeometry.Point
'1166:     pPoint.PutCoords pB2.X + Sin(dAngleInRadians) * pGridHeight / 2, _
'                     pB2.Y - Cos(dAngleInRadians) * pGridHeight / 2

1347:     pPointColl.AddPoint pPoint
    ' JOIN BACK TO FIRST (CLOSE POLYGON) ----------------
1349:     pPointColl.AddPoint pPointStart

'1198: MsgBox "Point 1(x,y): " & pPointColl.Point(0).X & ", " & pPointColl.Point(0).Y & vbNewLine _
'& "Point 2(x,y): " & pPointColl.Point(1).X & ", " & pPointColl.Point(1).Y & vbNewLine _
'& "Point 3(x,y): " & pPointColl.Point(2).X & ", " & pPointColl.Point(2).Y & vbNewLine _
'& "Point 4(x,y): " & pPointColl.Point(3).X & ", " & pPointColl.Point(3).Y

    Exit Sub
eh:
1358:     Err.Raise Err.Number, Err.source, "Error in CreateAngledGridPolygon." _
        & "Err " & Err.Number & ": " & Erl & " - " & Err.Description
End Sub

Public Function GetHorizontalOffset(pPolyline As IPolyline, pStraightLineSegment As ILine)
On Error GoTo eh
  Dim pPointCollection As IPointCollection
  Dim pEnumVertex As IEnumVertex2
  Dim pOutVertex As IPoint
  Dim lOutPart As Long
  Dim lOutVertex As Long
  Dim dFromM As Double
  Dim dToM As Double
  Dim dMaxLeft As Double
  Dim dMaxRight As Double
  Dim pCurve As ICurve
  
1375:   dMaxLeft = 0
1376:   dMaxRight = 0
  
1378:   dFromM = pStraightLineSegment.FromPoint.m
1379:   dToM = pStraightLineSegment.ToPoint.m
  
1381:   Set pCurve = pStraightLineSegment
1382:   Set pPointCollection = pPolyline
1383:   Set pEnumVertex = pPointCollection.EnumVertices
1384:   Set pOutVertex = New esriGeometry.Point 'Create a new point.  This may not be a required step.
  
  'Reset the Enum
1387:   pEnumVertex.Reset
  'Get the first vertex
1389:   pEnumVertex.Next pOutVertex, lOutPart, lOutVertex
  
1391:   Do While Not pOutVertex Is Nothing
1392:     If (pOutVertex.m > dFromM) And (pOutVertex.m < dToM) Then
      Dim pOutPoint As IPoint
      Dim dDistanceAlongCurve As Double
      Dim dDistanceFromCurve As Double
      Dim bRightSide As Boolean
      
      'Find the distance from the vertex to the straigh line curve segment.  Also find which side it is on.
1399:       pCurve.QueryPointAndDistance esriNoExtension, pOutVertex, True, pOutPoint, dDistanceAlongCurve, dDistanceFromCurve, bRightSide
      'Compate the distance from the curve to the previous points to find the maximum distance from the curve on either side.
1401:       If bRightSide Then
1402:         If dDistanceFromCurve > dMaxRight Then dMaxRight = dDistanceFromCurve
1403:       Else
1404:         If dDistanceFromCurve > dMaxLeft Then dMaxLeft = dDistanceFromCurve
1405:       End If
'      'If M value is greater than the M value of the TO point, exit the loop
'      If pOutVertex.m > dToM Then Exit Do
1408:     End If
    
    'Increment to the next vertex.
1411:     pEnumVertex.Next pOutVertex, lOutPart, lOutVertex
1412:   Loop

  'Return the difference between the left and right offsets.
1415:   GetHorizontalOffset = dMaxLeft + (dMaxRight * -1)

'1265:   MsgBox "Horrizontal Offset: " & GetHorizontalOffset & vbNewLine _
'  & "From: " & dFromM & vbNewLine _
'  & "To: " & dToM

  Exit Function
eh:
1423:   Err.Raise Err.Number, Err.source, "Error in GetHorizontalOffset." _
        & "Err " & Err.Number & ": " & Erl & " - " & Err.Description
End Function


Public Sub Perm_DrawPoint(ByVal pPoint As IPoint, Application As IApplication, _
            Optional sElementName As String = "DEMO_TEMPORARY", _
            Optional dRed As Double = 255, Optional dGreen As Double = 0, _
            Optional dBlue As Double = 0, Optional dSize As Double = 6)
' Add a permanent graphic dot on the display at the given point location
    Dim pColor As IRgbColor
    Dim pMarker As ISimpleMarkerSymbol
    Dim pGLayer As IGraphicsLayer
    Dim pGCon As IGraphicsContainer
    Dim pElement As IElement
    Dim pMarkerElement As IMarkerElement
    Dim pElementProp As IElementProperties
    Dim pMx As IMxDocument
    
    ' Init
1443:     Set pMx = Application.Document
1444:     Set pGLayer = pMx.FocusMap.BasicGraphicsLayer
1445:     Set pGCon = pGLayer
1446:     Set pElement = New MarkerElement
1447:     pElement.Geometry = pPoint
1448:     Set pMarkerElement = pElement
    
    ' Set the symbol
1451:     Set pColor = New RgbColor
1452:     pColor.Red = dRed
1453:     pColor.Green = dGreen
1454:     pColor.Blue = dBlue
1455:     Set pMarker = New SimpleMarkerSymbol
1456:     With pMarker
1457:         .Color = pColor
1458:         .Size = dSize
1459:     End With
1460:     pMarkerElement.Symbol = pMarker
    
    ' Add the graphic
1463:     Set pElementProp = pElement
1464:     pElementProp.Name = sElementName
1465:     pGCon.AddElement pElement, 0
End Sub

Public Sub Perm_DrawLineFromPoints(ByVal pFromPoint As IPoint, ByVal pToPoint As IPoint, _
            Application As IApplication, _
            Optional sElementName As String = "DEMO_TEMPORARY", _
            Optional dRed As Double = 0, Optional dGreen As Double = 0, _
            Optional dBlue As Double = 255, Optional dSize As Double = 1)
' Add a permanent graphic line on the display, using the From and To points supplied
    Dim pLnSym As ISimpleLineSymbol
    Dim pLine1 As ILine
    Dim pSeg1 As ISegment
    Dim pPolyline As ISegmentCollection
    Dim myColor As IRgbColor
    Dim pSym As ISymbol
    Dim pLineSym As ILineSymbol
    Dim pGLayer As IGraphicsLayer
    Dim pGCon As IGraphicsContainer
    Dim pElement As IElement
    Dim pLineElement As ILineElement
    Dim pElementProp As IElementProperties
    Dim pMx As IMxDocument
    
    ' Init
1489:     Set pMx = Application.Document
1490:     Set pGLayer = pMx.FocusMap.BasicGraphicsLayer
1491:     Set pGCon = pGLayer
1492:     Set pElement = New LineElement
    
    ' Set the line symbol
1495:     Set pLnSym = New SimpleLineSymbol
1496:     Set myColor = New RgbColor
1497:     myColor.Red = dRed
1498:     myColor.Green = dGreen
1499:     myColor.Blue = dBlue
1500:     pLnSym.Color = myColor
1501:     pLnSym.Width = dSize
    
    ' Create a standard polyline (via 2 points)
1504:     Set pLine1 = New esriGeometry.Line
1505:     pLine1.PutCoords pFromPoint, pToPoint
1506:     Set pSeg1 = pLine1
1507:     Set pPolyline = createPolylineM
1508:     pPolyline.AddSegment pSeg1
1509:     pElement.Geometry = pPolyline
1510:     Set pLineElement = pElement
1511:     pLineElement.Symbol = pLnSym
    
    ' Add the graphic
1514:     Set pElementProp = pElement
1515:     pElementProp.Name = sElementName
1516:     pGCon.AddElement pElement, 0
End Sub

Public Sub Perm_DrawTextFromPoint(pPoint As IPoint, sText As String, Application As IApplication, _
            Optional sElementName As String = "DEMO_TEMPORARY", _
            Optional dRed As Double = 50, Optional dGreen As Double = 50, _
            Optional dBlue As Double = 50, Optional dSize As Double = 10)
' Add permanent graphic text on the display at the given point location
    Dim myTxtSym As ITextSymbol
    Dim myColor As IRgbColor
    Dim pGLayer As IGraphicsLayer
    Dim pGCon As IGraphicsContainer
    Dim pElement As IElement
    Dim pTextElement As ITextElement
    Dim pElementProp As IElementProperties
    Dim pMx As IMxDocument
    
    ' Init
1534:     Set pMx = Application.Document
1535:     Set pGLayer = pMx.FocusMap.BasicGraphicsLayer
1536:     Set pGCon = pGLayer
1537:     Set pElement = New TextElement
1538:     pElement.Geometry = pPoint
1539:     Set pTextElement = pElement
    
    ' Create the text symbol
1542:     Set myTxtSym = New TextSymbol
1543:     Set myColor = New RgbColor
1544:     myColor.Red = dRed
1545:     myColor.Green = dGreen
1546:     myColor.Blue = dBlue
1547:     myTxtSym.Color = myColor
1548:     myTxtSym.Size = dSize
1549:     myTxtSym.HorizontalAlignment = esriTHACenter
1550:     pTextElement.Symbol = myTxtSym
1551:     pTextElement.Text = sText
    
    ' Add the graphic
1554:     Set pElementProp = pElement
1555:     pElementProp.Name = sElementName
1556:     pGCon.AddElement pElement, 0
End Sub

Public Sub RemoveGraphicsByName(pMxDoc As IMxDocument, _
            Optional sPrefix As String = "DEMO_TEMPORARY")
' Delete all graphics with our prefix from ArcScene
    Dim pElement As IElement
    Dim pElementProp As IElementProperties
    Dim sLocalPrefix As String
    Dim pGLayer As IGraphicsLayer
    Dim pGCon As IGraphicsContainer
    Dim lCount As Long
    
    On Error GoTo ErrorHandler
    
    ' Init and switch OFF the updating of the TOC
1572:     pMxDoc.DelayUpdateContents = True
1573:     Set pGLayer = pMxDoc.FocusMap.BasicGraphicsLayer
1574:     Set pGCon = pGLayer
1575:     pGCon.Next
    
    ' Delete all the graphic elements that we created (identify by the name prefix)
1578:     pGCon.Reset
1579:     Set pElement = pGCon.Next
1580:     While Not pElement Is Nothing
1581:         If TypeOf pElement Is IElement Then
1582:             Set pElementProp = pElement
1583:             If (Left(pElementProp.Name, Len(sPrefix)) = sPrefix) Then
1584:                 pGCon.DeleteElement pElement
1585:             End If
1586:         End If
1587:         Set pElement = pGCon.Next
1588:     Wend
    
    ' Switch ON the updating of the TOC, refresh
1591:     pMxDoc.DelayUpdateContents = False
1592:     pMxDoc.ActiveView.Refresh
    
    Exit Sub
ErrorHandler:
1596:     MsgBox "Error in RemoveGraphicsByName: " & Erl & " - " & Err.Description, , "RemoveGraphicsByName"
End Sub

'Input: pPolyline - a Polyline geometry
'pCirclePolygon - a Polygon geometry containing a circle.
'Returns: a Point geometry.
'The point geometry is created by first creating a line geometry that is tangent to the polyline.
'A new polyline geometry is created based on the FromPoint and ToPoint of the line geometry.
'The new polyline geometry is intersected with pCirclePolygon.
'The first intersection point is returned.
'Purpose: This function is used to help generate the last page on the map strip by providing a second
'Point past the end of the Polyline that intersects the Circle Polygon.
Private Function IntersectPointExtendedTo(pPolyline As IPolyline, pCirclePoly As IPolygon) As IPoint
On Error GoTo ErrorHandler
    Dim pCurve As ICurve
    Dim pLine As ILine
    Dim pPLine As IPolyline
    Dim pTopoOpt As ITopologicalOperator
    Dim pGeoCol As IGeometryCollection
    ' Need to extend the end (creates an ILine object)
1616:     Set pCurve = pPolyline
1617:     Set pLine = New esriGeometry.Line
1618:     pCurve.QueryTangent esriExtendTangentAtTo, 1, True, _
                        CDbl(m_GridWidth) * 1.1, pLine
    ' Convert ILine to an IPolyline
1621:     Set pPLine = createPolylineM
1622:     pPLine.FromPoint = pLine.FromPoint
1623:     pPLine.ToPoint = pLine.ToPoint
    ' Intersect the polyline with the circle
1625:     Set pTopoOpt = pPLine
1626:     Set pGeoCol = New GeometryBag
1627:     Set pGeoCol = pTopoOpt.Intersect(pCirclePoly, esriGeometry0Dimension)
1628:     If pGeoCol.GeometryCount = 0 Then
1629:         Set IntersectPointExtendedTo = Nothing
1630:     Else
1631:         Set IntersectPointExtendedTo = pGeoCol.Geometry(0)
1632:     End If
    Exit Function
ErrorHandler:
1635:     MsgBox "Error in IntersectPointExtendedTo: " & Erl & " - " & Err.Description, , "IntersectPointExtendedTo"
End Function

Private Function AddPathToPolyLine(pPolyline As IPolyline, pPath As IPath) As IPolyline
  Dim pGCol As IGeometryCollection
  Dim pGeom As IGeometry
    
1642:   If pPolyline Is Nothing Then
1643:     Set pPolyline = createPolylineM
1644:   End If
1645:   Set pGCol = pPolyline
1646:   Set pGeom = pPath
1647:   pGCol.AddGeometry pGeom
1648:   Set AddPathToPolyLine = pGCol

End Function

'Inputs:
'A feature buffer
'A pageGeometry.
'A polyline (Must be M-Aware)
'And a boolean, atEnd which appears to not be used.
'This subroutine sets the "From Station" and "To Station" attributes of the feature buffer.
Private Sub GetStationEndPoints(feat As IFeatureBuffer, pageGeom As ITopologicalOperator, routeLine As IPolyline, atEnd As Boolean)
    On Error GoTo ErrHand
        If m_FldFromStation = "" Then Exit Sub
        If m_FldToStation = "" Then Exit Sub
    
        Dim intersectionGeom As IGeometry
        Dim fromField As IField
        Dim toField As IField
        
        'Set pageGeom = feat.Shape

             'Set pageGeom = m_pPageShape
             Dim ma As IMAware
1671:              Set ma = routeLine
             'MsgBox "MA " & ma.maware


1675:         Set intersectionGeom = pageGeom.Intersect(routeLine, esriGeometry0Dimension)
        
        Dim intersectionPoints As IPointCollection
        
1679:         Set intersectionPoints = intersectionGeom

             'MsgBox "intersectionPoints.PointCount: " & intersectionPoints.PointCount

1683:         If intersectionPoints.PointCount = 2 Then
            Dim m1 As Double, m2 As Double
1685:             m1 = GetClosestPointStation(intersectionPoints.Point(0), routeLine)
1686:             m2 = GetClosestPointStation(intersectionPoints.Point(1), routeLine)
1687:                 If m1 < m2 Then
1688:                     feat.value(feat.Fields.FindField(m_FldFromStation)) = m1
1689:                     feat.value(feat.Fields.FindField(m_FldToStation)) = m2
1690:                 Else
1691:                     feat.value(feat.Fields.FindField(m_FldFromStation)) = m2
1692:                     feat.value(feat.Fields.FindField(m_FldToStation)) = m1
1693:                 End If
1694:             ElseIf intersectionPoints.PointCount = 1 Then
                    Dim m As Double
                    Dim routePoints As IPointCollection
                    
1698:                     Set routePoints = routeLine
1699:                     m = GetClosestPointStation(intersectionPoints.Point(0), routeLine)
1700:                     If pageGeom.Intersect(routePoints.Point(0), esriGeometry0Dimension).IsEmpty Then
                    
1702:                         feat.value(feat.Fields.FindField(m_FldFromStation)) = m
1703:                    feat.value(feat.Fields.FindField(m_FldToStation)) = routePoints.Point(routePoints.PointCount - 1).m
1704:                     Else
1705:                         feat.value(feat.Fields.FindField(m_FldFromStation)) = routePoints.Point(0).m
1706:                    feat.value(feat.Fields.FindField(m_FldToStation)) = m
1707:                     End If
                    
1709:         End If
        'dataGraph.XAxisMin = m_pPageShape pFeatLayer

  Exit Sub
ErrHand:
1714:   MsgBox "GetStationEndPoints - " & Erl & " - " & Err.Description
End Sub

'This function takes a geometry and a point as input.
'It treats the geometry as a collection of points,
'and returns the point that is closest to the point that was pased in.
Private Function GetClosestPointStation(pPoint As IPoint, geom As IGeometry) As Double
  On Error GoTo ErrorHandler

    Dim pointGeom As IPointCollection
    Dim i As Integer
1725:     Set pointGeom = geom
    Dim closestD As Double
    Dim closestIndex As Long
    Dim closePoint As IPoint
    Dim maware As IMAware
    
1731:     Set maware = pointGeom.Point(0)
    'MsgBox maware.maware
1733:     closestD = 9999999# * 9999999#
1734:     For i = 0 To pointGeom.PointCount - 1
        Dim geomPoint As IPoint
        Dim d As Double
        
1738:         Set geomPoint = pointGeom.Point(i)
1739:         d = (geomPoint.X - pPoint.X) * (geomPoint.X - pPoint.X) + (geomPoint.Y - pPoint.Y) * (geomPoint.Y - pPoint.Y)
1740:         If d < closestD Then
1741:             closestD = d
1742:             closestIndex = i
1743:         End If
1744:     Next i
         
1746:     Set closePoint = pointGeom.Point(closestIndex)
1747:     GetClosestPointStation = closePoint.m
    
'MsgBox closePoint.X & " " & closePoint.Y & " Station = " & closePoint.m & " distance = " & closestD & " index = " & closestIndex & " pCount = " & pointGeom.PointCount
  Exit Function
ErrorHandler:
1752:   MsgBox "GetClosestPointStation - " & Erl & " - " & Err.Description
End Function

Private Function createPolylineM() As IPolyline
  On Error GoTo ErrorHandler

    Dim rt As IPolyline
    Dim rt2 As IMAware
    
1761:     Set rt = New Polyline
1762:     Set rt2 = rt
1763:     rt2.maware = True
    
1765:     Set createPolylineM = rt

  Exit Function
ErrorHandler:
  HandleError False, "createPolylineM " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.source, Err.Description, 4
End Function
