VERSION 5.00
Begin VB.Form frmAnnotationDialog 
   BorderStyle     =   1  'Fixed Single
   Caption         =   "Convert Labels To Annotation..."
   ClientHeight    =   4860
   ClientLeft      =   45
   ClientTop       =   435
   ClientWidth     =   4740
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   4860
   ScaleWidth      =   4740
   StartUpPosition =   3  'Windows Default
   Begin VB.ListBox lstLayers 
      Height          =   4155
      ItemData        =   "frmAnnotationDialog.frx":0000
      Left            =   120
      List            =   "frmAnnotationDialog.frx":0002
      MultiSelect     =   2  'Extended
      Sorted          =   -1  'True
      TabIndex        =   2
      Top             =   120
      Width           =   4455
   End
   Begin VB.CommandButton cmdOK 
      Caption         =   "&OK"
      Height          =   345
      Left            =   2400
      TabIndex        =   1
      Top             =   4440
      Width           =   1125
   End
   Begin VB.CommandButton cmdCancel 
      Cancel          =   -1  'True
      Caption         =   "&Cancel"
      Height          =   345
      Left            =   3480
      TabIndex        =   0
      Top             =   4440
      Width           =   1125
   End
   Begin VB.Label lblMultiSelect 
      Caption         =   "Hold down Ctrl or Shift to select multiple layers."
      Height          =   375
      Left            =   120
      TabIndex        =   3
      Top             =   4320
      Width           =   2295
   End
End
Attribute VB_Name = "frmAnnotationDialog"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

'Author: Sam Falchook
'USACE, MVN
'Creation Date: 10/19/2008
'
'Purpose: This form provides the capability to convert labels in a map series into page-specific annotation layers.

Option Explicit

Public m_pApp As IApplication
Private m_pProgress As IModelessFrame

Private Sub cmdCancel_Click()
13:   Unload Me
End Sub

'When the user clicks OK, the application generates page-specific annotation for each of the selected layers.
Private Sub cmdOK_Click()
On Error GoTo ErrHand:
  'Get the MapBook
  Dim pMapBook As IDSMapBook
21:   Set pMapBook = GetMapBookExtension(m_pApp)
22:   If pMapBook Is Nothing Then
23:     MsgBox "Warning: MapBook is nothing."
    Exit Sub
25:   End If
  'Get the MapSeries
  Dim pMapSeries As IDSMapSeries
28:   If pMapBook.ContentCount = 0 Then
29:     MsgBox "Warning: The MapBook does not contain any map series."
    Exit Sub
31:   End If
32:   Set pMapSeries = pMapBook.ContentItem(0)
33:   If pMapSeries Is Nothing Then
    Exit Sub
35:   End If

  'Count the pages
  Dim iPageCount As Integer
39:   iPageCount = pMapSeries.PageCount
  
  'Get the focus map
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
45:   Set pMxDoc = m_pApp.Document
46:   Set pActiveView = pMxDoc.ActiveView
47:   Set pMap = FindDataFrame2(m_pApp) 'get the main Map Book data frame
  
  'Grab an enumeration of feature layers from the map (recursively searches group layers)
  Dim pEnumLayer As IEnumLayer
  Dim pId As New UID
52:   pId = FEATURE_LAYER_UID 'UID for IFeatureLayer
53:   Set pEnumLayer = pMap.Layers(pId, True)
  
  'Initialize the progress bar
  Dim pProgress As frmProgress
57:   Set pProgress = New frmProgress
58:   m_pProgress.Create pProgress
59:   pProgress.ProgressBar1.Min = 0
60:   pProgress.ProgressBar1.Max = iPageCount
61:   pProgress.ProgressBar1.value = 0
62:   pProgress.lblInformation.Caption = "Creating Annotation Groups (0 of " & iPageCount & ")"
63:   pProgress.cmdCancel.Visible = True        ' User can cancel at any time
64:   m_pProgress.Visible = True
65:   pProgress.Refresh
  
  'Set mouse pointer
68:   Screen.MousePointer = vbArrowHourglass
  
  'Loop through each page in the map series.
  Dim lLoopCounter As Long
  Dim pMapPage As IDSMapPage
  Dim lPageNumber As Long
  Dim sPageNumber As String
  Dim pSeriesProps As IDSMapSeriesProps
76:   Set pSeriesProps = pMapSeries
  
78:   For lLoopCounter = 0 To (iPageCount - 1)
79:     Set pMapPage = pMapSeries.Page(lLoopCounter)
80:     lPageNumber = lLoopCounter + pSeriesProps.StartNumber
81:     sPageNumber = CStr(lPageNumber)
    
    'Display the page
84:     DrawPage m_pApp, pMapSeries, lLoopCounter, False

    'Annotate the page.
87:     AnnotatePage m_pApp, pMap, pEnumLayer, sPageNumber
    
    'Refresh the progress bar
90:     pProgress.ProgressBar1.value = lLoopCounter + 1
91:     pProgress.lblInformation.Caption = "Creating Annotation Groups (" _
    & (lLoopCounter + 1) & " of " & iPageCount & ")"

94:     DoEvents 'Share some CPU time with the user interface to all user-events to be handled.
95:     pProgress.Refresh

97:     If pProgress.Cancelled Then
98:         pProgress.Cancelled = False 'Reset the form
        Dim vUserChoice As Variant
100:         vUserChoice = MsgBox("Operation cancelled." & "  Save the Annotation generated thus far?" & vbCrLf & vbCrLf _
        & "(Click Cancel to continue generating annotation)", _
        vbYesNoCancel, "Generate Annotation")
        
        Select Case vUserChoice
        Case vbYes 'Stop generating annotation, but keep the annotation that has been generated so far.
106:             Exit For
        Case vbNo 'Stop generating Annotation, and delete the annotation that has been generated thus far.
108:             DeleteAnnotationLayersAfterPartialCreation
109:             Exit For
        Case vbCancel 'Continue generating Annotation
        Case Else
112:             MsgBox "Error, you must select either yes, no or cancel."
113:         End Select
114:     End If

116:   Next lLoopCounter

  'Hide the progress bar
119:   m_pProgress.Visible = False
  'Restore the mouse pointer
121:   Screen.MousePointer = vbDefault
  'After generating annotation for each page,
  'return to the current page.
  Dim lPageIndex As Long
125:   lPageIndex = (pMapSeries.CurrentPageNumber - pSeriesProps.StartNumber)
  
127:   ViewPage m_pApp, pMapSeries, lPageIndex
128:   Unload Me
  
  Exit Sub
ErrHand:
132:   MsgBox "cmdOK_Click - " & Erl & " - " & Err.Description
End Sub

'This subroutine deletes annotation layers after the user decides to cancel creating the annotation layers
Private Sub DeleteAnnotationLayersAfterPartialCreation()
On Error GoTo ErrHand:
  'Get a collection of selected layer names
  Dim layerNames As Collection
140:   Set layerNames = New Collection
  
  'Loop through each of the selected layer names in the listbox and add them to a collection
  Dim iLCount1 As Integer
  Dim bSelected As Boolean
145:   For iLCount1 = 0 To (Me.lstLayers.ListCount - 1)
146:     bSelected = Me.lstLayers.Selected(iLCount1)
147:     If bSelected Then
148:         layerNames.Add Me.lstLayers.List(iLCount1)
149:     End If
150:   Next iLCount1 'End of looping through selected layer names in listbox (Outer Loop)

152:   DeletePageSpecificAnnotationByLayerNames layerNames, m_pApp

  Exit Sub
ErrHand:
156:   MsgBox "DeleteAnnotationLayersAfterPartialCreation - " & Erl & " - " & Err.Description
End Sub

'This subroutine generates page-specific annotation for the current specific page.
Private Sub AnnotatePage(pApp As IApplication, pMap As IMap, pEnumLayer As IEnumLayer, sPageNumber As String)
On Error GoTo ErrHand:
  Dim pLayer As ILayer
  
  'Use this extra counter for troubleshooting purposes.
  Dim lExtraCounter As Long
166:   lExtraCounter = 0
  
  'Look through the selected layer names in the listbox
  'Get the layer(s) to annotate from the layer names in the listbox
  Dim iLCount1 As Integer
171:   For iLCount1 = 0 To (Me.lstLayers.ListCount - 1)
    Dim bSelected As Boolean
173:     bSelected = Me.lstLayers.Selected(iLCount1)
174:     If bSelected Then
      'Get a selected layer name from the listbox
      Dim sLayerNameFromListbox As String
177:       sLayerNameFromListbox = Me.lstLayers.List(iLCount1)
      
      'Loop through the map layers to find a layer with a matching name
180:       pEnumLayer.Reset
181:       Set pLayer = pEnumLayer.Next
182:       Do While Not pLayer Is Nothing
        'Use this extra counter for troubleshooting purposes.
184:         lExtraCounter = lExtraCounter + 1
      
        Dim sLayerNameFromMap As String
187:         sLayerNameFromMap = pLayer.Name
        'If the names match, convert the labels to annotation
189:         If sLayerNameFromMap = sLayerNameFromListbox Then
190:           CreatePageSpecificAnnotation pApp, pLayer, sPageNumber
191:         End If
192:         Set pLayer = pEnumLayer.Next
193:       Loop
194:     End If
195:   Next iLCount1
  Exit Sub
ErrHand:
198:   MsgBox "AnnotatePage - " & Erl & " - " & Err.Description
End Sub

Private Sub Form_Initialize()
202:   Set m_pProgress = New ModelessFrame
End Sub

Private Sub Form_Load()
On Error GoTo ErrHand:
207:   InitializeLayerList
  Exit Sub
ErrHand:
210:   MsgBox "frmAnnotationDialog_Load - " & Erl & " - " & Err.Description
End Sub

'Populates the list box with names of feature layers in the map
Private Sub InitializeLayerList()
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
  Dim pEnumLayer As IEnumLayer
  Dim pLayer As ILayer
  Dim pId As New UID
  Dim AnnotatedLayers As Collection
  
224:   Set AnnotatedLayers = GetAnnotatedLayers()
  
226:   Set pMxDoc = m_pApp.Document
227:   Set pActiveView = pMxDoc.ActiveView
228:   Set pMap = FindDataFrame2(m_pApp) 'get the main Map Book data frame
229:   pId = FEATURE_LAYER_UID 'UID for IFeatureLayer
  'Grab all feature layers from the map (and recursively searches group layers)
231:   Set pEnumLayer = pMap.Layers(pId, True)
  'Loop through the layers
233:   pEnumLayer.Reset
234:   Set pLayer = pEnumLayer.Next
235:   Do While Not pLayer Is Nothing
    Dim sLayerName As String
237:     sLayerName = pLayer.Name
    
239:     If ContainsString(AnnotatedLayers, sLayerName) = False Then
240:       Me.lstLayers.AddItem sLayerName
241:     End If
    
    'Increment the loop
244:     Set pLayer = pEnumLayer.Next
245:   Loop
  Exit Sub
ErrHand:
248:   MsgBox "InitializeLayerList - " & Erl & " - " & Err.Description
End Sub

'This function takes a collection of strings and a string as input.
'If the string is found in the collection, it returns true.
Private Function ContainsString(StringCollection As Collection, s As String) As Boolean
On Error GoTo ErrHand:
  Dim vValue As Variant
  Dim sValue As String
  Dim bMatch As Boolean
258:   bMatch = False
259:   For Each vValue In StringCollection
260:     sValue = vValue
261:     If sValue = s Then
262:       bMatch = True
263:       Exit For
264:     End If
265:   Next vValue
266:   ContainsString = bMatch
  Exit Function
ErrHand:
269:   MsgBox "ContainsString - " & Erl & " - " & Err.Description
End Function

'This function returns a collection of layer names of annotated layers.
Private Function GetAnnotatedLayers() As Collection
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
  Dim AnnotatedLayers As Collection
279:   Set pMxDoc = m_pApp.Document
280:   Set pActiveView = pMxDoc.ActiveView
281:   Set pMap = FindDataFrame2(m_pApp) 'get the main Map Book data frame
282:   Set AnnotatedLayers = New Collection

  'Get the 'Basic Graphics Layer' which is a container for the annotation groups.
  Dim pBasicGraphLyr As ICompositeLayer
286:   Set pBasicGraphLyr = pMap.BasicGraphicsLayer
  
  'Loop through each annotation group
  Dim lngLoopCount As Long
290:   For lngLoopCount = 0 To (pBasicGraphLyr.count - 1)
    Dim pAnnotationLayer As ILayer
    Dim sAnnotationLayerName As String
293:     Set pAnnotationLayer = pBasicGraphLyr.Layer(lngLoopCount)
294:     sAnnotationLayerName = pAnnotationLayer.Name
'MsgBox "pAnnotationLayer.Name: " & CStr(sAnnotationLayerName)
    
    'Determine if Annotation Layer name contains the MAPBOOK_ANNO_CONST constant.
    'This will determine if the Annotation Layer was created by JAMB
    Dim lSubStrLocation As Long
300:     lSubStrLocation = InStrRev(sAnnotationLayerName, MAPBOOK_ANNO_CONST)
301:     If (lSubStrLocation <> 0) Then 'If the annotation layer name does not contain the MAPBOOK_ANNO_CONST constant.
      'Get the associated feature layer
      Dim pGraphicsLayer As IGraphicsLayer
      Dim pAssociatedLayer As ILayer
305:       Set pGraphicsLayer = pAnnotationLayer
306:       Set pAssociatedLayer = pGraphicsLayer.AssociatedLayer
  
308:       If pAssociatedLayer Is Nothing Then
309:       Else
        Dim sMapLayerName As String
311:         sMapLayerName = pAssociatedLayer.Name
        'Add the layer name to the collection
313:         AnnotatedLayers.Add sMapLayerName
314:       End If
315:     End If 'End of "annotation layer name does not contain the MAPBOOK_ANNO_CONST constant"

317:   Next lngLoopCount
  'Return a collection of layer names
319:   Set GetAnnotatedLayers = AnnotatedLayers

  Exit Function
ErrHand:
323:   MsgBox "GetAnnotatedLayers - " & Erl & " - " & Err.Description
324:   Set GetAnnotatedLayers = AnnotatedLayers
End Function

Private Sub Form_Terminate()
328:   Set m_pProgress = New ModelessFrame
End Sub
