Attribute VB_Name = "modGeneralFunctions"
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

' Copyright 2006 ESRI
'
' All rights reserved under the copyright laws of the United States
' and applicable international laws, treaties, and conventions.
'
' You may freely redistribute and use this sample code, with or
' without modification, provided you include the original copyright
' notice and use restrictions.
'
' See use restrictions at /arcgis/developerkit/userestrictions.


Option Explicit

Public Const c_DefaultFld_Shape = "SHAPE"
Public Const cPI = 3.14159265358979
Public Const MAPBOOK_ANNO_CONST = "_Anno_Page_"

'Grid Index Field Name Constants
Public Const c_DefaultFld_GridID = "GRID_ID"
Public Const c_DefaultFld_ColNum = "COL_NUM"
Public Const c_DefaultFld_RowNum = "ROW_NUM"
Public Const c_DefaultFld_Scale = "PLOTSCALE"
'Strip Map Grid Field Name Constants
Public Const c_DefaultFld_StripMapName = "SMAP_NAME"
Public Const c_DefaultFld_SeriesNum = "SMAP_NUM"
Public Const c_DefaultFld_MapAngle = "SMAP_ANGLE"
Public Const c_DefaultFld_MapScale = "SMAP_SCALE"
Public Const c_DefaultFld_ToStat = "TO_STAT"
Public Const c_DefaultFld_FromStat = "FROM_STAT"
Public Const c_DefaultFld_FitsInsideGrid = "IN_GRID"



' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\development\Source\DSMapBook\modGeneralFunctions.bas"

'Private variables
Private m_pConvertLabelsToAnnotation As IConvertLabelsToAnnotation

Public Function GetUnitsDescription(pUnits As esriUnits) As String
    Select Case pUnits
        Case esriInches: GetUnitsDescription = "Inches"
        Case esriPoints: GetUnitsDescription = "Points"
        Case esriFeet: GetUnitsDescription = "Feet"
        Case esriYards: GetUnitsDescription = "Yards"
        Case esriMiles: GetUnitsDescription = "Miles"
        Case esriNauticalMiles: GetUnitsDescription = "Nautical miles"
        Case esriMillimeters: GetUnitsDescription = "Millimeters"
        Case esriCentimeters: GetUnitsDescription = "Centimeters"
        Case esriMeters: GetUnitsDescription = "Meters"
        Case esriKilometers: GetUnitsDescription = "Kilometers"
        Case esriDecimalDegrees: GetUnitsDescription = "Decimal degrees"
        Case esriDecimeters: GetUnitsDescription = "Decimeters"
        Case esriUnknownUnits: GetUnitsDescription = "Unknown"
        Case Else: GetUnitsDescription = "Unknown"
58:     End Select
End Function

Public Function GetActiveDataFrameName(pApp As IApplication) As String
    Dim pMx As IMxDocument
    Dim pFocusMap As IMap
    
65:     Set pMx = pApp.Document
66:     Set pFocusMap = pMx.FocusMap
    
68:     GetActiveDataFrameName = pFocusMap.Name
End Function

Public Function GetDataFrameElement(sDataFramName As String, pApp As IApplication) As IElement
' Get the data frame element by name
    Dim pGraphicsContainer As IGraphicsContainer
    Dim pElementProperties As IElementProperties
    Dim pElement As IElement
    Dim pMx As IMxDocument
    Dim pFE As IFrameElement
    Dim pElProps As IElementProperties
    
    On Error GoTo ErrorHandler
    
    ' Init
83:     Set pMx = pApp.Document
    ' Loop through the elements (in the layout)
85:     Set pGraphicsContainer = pMx.PageLayout
86:     pGraphicsContainer.Reset
87:     Set pElement = pGraphicsContainer.Next
88:     While Not pElement Is Nothing
        ' If type of element is an IFrameElement
90:         If TypeOf pElement Is IFrameElement Then
91:             Set pElProps = pElement
            ' If Name matches
93:             If UCase(pElProps.Name) = UCase(sDataFramName) Then
                ' Return element
95:                 Set GetDataFrameElement = pElement
96:                 Set pElement = Nothing
97:             Else
98:                 Set pElement = pGraphicsContainer.Next
99:             End If
100:         Else
101:             Set pElement = pGraphicsContainer.Next
102:         End If
103:     Wend
    
    Exit Function
ErrorHandler:
107:     Err.Raise Err.Number, Err.source, "Error in GetDataFrameElement:" _
        & vbCrLf & Erl & " - " & Err.Description
End Function

Public Function FindFeatureLayerByDS(DatasetName As String, pApp As IApplication) As IFeatureLayer
  
    On Error GoTo ErrorHandler
  
    Dim pMxDoc As IMxDocument
    Dim pMap As IMap
    Dim pFeatureLayer As IFeatureLayer
    Dim pDataset As IDataset
    Dim i As Integer
    
121:     Set pMxDoc = pApp.Document
122:     Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
  
124:     With pMap
125:         For i = 0 To .LayerCount - 1
126:             If TypeOf .Layer(i) Is IFeatureLayer Then
127:                 Set pFeatureLayer = .Layer(i)
128:                 Set pDataset = pFeatureLayer.FeatureClass
129:                 If UCase(pDataset.Name) = UCase(DatasetName) Then
130:                     Set FindFeatureLayerByDS = pFeatureLayer
131:                     Exit For
132:                 End If
133:             End If
134:         Next i
135:     End With
  
137:     If pFeatureLayer Is Nothing Then
138:         Err.Raise vbObjectError, "FindFeatureLayerByDS", "Error in " _
            & "FindFeatureLayerByDS:" & vbCrLf & "Could not locate " _
            & "layer with a dataset name of '" & DatasetName & "'."
141:     End If
  
    Exit Function
ErrorHandler:
145:     Err.Raise Err.Number, Err.source, "Error in routine: FindFeatureLayerByDS" _
        & vbCrLf & Erl & " - " & Err.Description
End Function

Public Function FindFeatureLayerByName(FLName As String, pApp As IApplication) As IFeatureLayer
    On Error GoTo ErrorHandler
    Dim pMxDoc As IMxDocument
    Dim pMap As IMap
    Dim pFeatureLayer As IFeatureLayer
154:     Set pMxDoc = pApp.Document
155:     Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame

'    If pMap Is Nothing Then
'        MsgBox "pMap is nothing"
'    Else
'        MsgBox "pMap is not nothing"
'    End If

163:     Set pFeatureLayer = GetFeatureLayerByName(FLName, pMap)

165:     Set FindFeatureLayerByName = pFeatureLayer

    Exit Function
ErrorHandler:
169:     Err.Raise Err.Number, Err.source, "Error in routine: FindFeatureLayerByName" _
        & vbCrLf & Erl & " - " & Err.Description
End Function


'This function takes a dataset name string and an application pointer as input.
'and returns a dataset from the map.
'If no dataset is found with a matching name, it returns Nothing.
Public Function FindDatasetByName(sDatasetName As String, pApp As IApplication) As IDataset
  On Error GoTo ErrorHandler
  'Initialize
180:   Set FindDatasetByName = Nothing
  
  'Get an enumeration of datasets
  Dim pMx As IMxDocument
  Dim pDocDatasets As IDocumentDatasets
  Dim pEnumDataset As IEnumDataset
  Dim pDataset As IDataset, pFoundDataset As IDataset
  Dim bFound As Boolean
188:   bFound = False
189:   Set pMx = pApp.Document
190:   Set pDocDatasets = pMx
191:   Set pEnumDataset = pDocDatasets.Datasets
192:   pEnumDataset.Reset
  'Loop through the data sets to get the selected dataset
194:   Set pDataset = pEnumDataset.Next
195:   Do While Not pDataset Is Nothing
'MsgBox "pDataset.Name: " & pDataset.Name
197:     If pDataset.Name = sDatasetName Then
198:       Set pFoundDataset = pDataset
199:       bFound = True
'MsgBox "Matching Dataset Found"
201:       Exit Do
202:     End If
    'Increment the loop
204:     Set pDataset = pEnumDataset.Next
205:   Loop

  'Return a Dataset
208:   If bFound = True Then
209:     Set FindDatasetByName = pFoundDataset
210:   Else
211:     Set FindDatasetByName = Nothing
212:   End If
  
  Exit Function
ErrorHandler:
216:     MsgBox "Error in FindDatasetByName: " & Erl & " - " & Err.Description, , "FindDatasetByName"
217:     Set FindDatasetByName = Nothing
End Function

Public Function GetValidExtentForLayer(pFL As IFeatureLayer) As IEnvelope
    Dim pGeoDataset As IGeoDataset
    Dim pMx As IMxDocument
    Dim pW As IWorkspace
    Dim pWSR As IWorkspaceSpatialReferenceInfo
    Dim pEnumSRI As IEnumSpatialReferenceInfo
    Dim pSR As ISpatialReference
    Dim dX1 As Double, dY1 As Double
    Dim dX2 As Double, dY2 As Double
    Dim pP As IPoint
    
231:     If Not pFL Is Nothing Then
232:         If Not pFL.FeatureClass Is Nothing Then
233:             If TypeOf pFL.FeatureClass Is IGeoDataset Then
234:                 If pFL.FeatureClass.FeatureDataset Is Nothing Then
235:                     dX1 = -1000000000
236:                     dY1 = -1000000000
237:                     dX2 = 1000000000
238:                     dY2 = 1000000000
239:                 Else
240:                     Set pW = pFL.FeatureClass.FeatureDataset.Workspace
241:                     Set pWSR = pW
242:                     Set pEnumSRI = pWSR.SpatialReferenceInfo
243:                     Set pSR = pEnumSRI.Next(0)
244:                     pSR.GetDomain dX1, dX2, dY1, dY2
245:                 End If
246:                 Set pP = New esriGeometry.Point
247:                 Set GetValidExtentForLayer = New Envelope
248:                 pP.PutCoords dX1, dY1
249:                 GetValidExtentForLayer.LowerLeft = pP
250:                 pP.PutCoords dX2, dY2
251:                 GetValidExtentForLayer.UpperRight = pP
252:             Else
253:                 Err.Raise vbObjectError, "GetValidExtentForLayer", _
                    "The 'FeatureClass' property for the IFeatureLayer parameter is not an IGeoDataset"
255:             End If
256:         Else
257:             Err.Raise vbObjectError, "GetValidExtentForLayer", _
                "The IFeatureLayer parameter does not have a valid FeatureClass property"
259:         End If
260:     Else
261:         Err.Raise vbObjectError, "GetValidExtentForLayer", _
            "The IFeatureLayer parameter is set to Nothing"
263:     End If
End Function

Public Function DoesShapeFileExist(pPath As String) As Boolean
  Dim pTruncPath As String
268:   If InStr(1, pPath, ".shp") > 0 Then
269:     pTruncPath = Left(pPath, InStr(1, pPath, ".shp") - 1)
270:   Else
271:     pTruncPath = pPath
272:   End If
      
  'Make sure the specified file does not exist
  Dim fs As Object
276:   Set fs = CreateObject("Scripting.FileSystemObject")
277:   If fs.fileexists(pTruncPath & ".shp") Or fs.fileexists(pTruncPath & ".dbf") Or _
   fs.fileexists(pTruncPath & ".shx") Then
279:     DoesShapeFileExist = True
280:   Else
281:     DoesShapeFileExist = False
282:   End If
End Function

Private Function DoesFeatureClassExist(location As IGxObject, newObjectName As String) As Boolean
On Error GoTo ErrHand:
  Dim pFeatClass As IFeatureClass
  Dim pFeatDataset As IGxDataset
289:   Set pFeatDataset = location
  Dim pFeatClassCont As IFeatureClassContainer, pData As IFeatureDataset
291:   Set pData = pFeatDataset.Dataset
292:   Set pFeatClassCont = pData
  Dim pEnumClass As IEnumFeatureClass, pDataset As IDataset
294:   Set pEnumClass = pFeatClassCont.Classes
295:   Set pFeatClass = pEnumClass.Next
296:   While Not pFeatClass Is Nothing
297:     Set pDataset = pFeatClass
298:     If UCase(pDataset.Name) = UCase(newObjectName) Then
299:       DoesFeatureClassExist = True
      Exit Function
301:     End If
      
303:     Set pFeatClass = pEnumClass.Next
304:   Wend
305:   DoesFeatureClassExist = False
  
  Exit Function
ErrHand:
309:   MsgBox Err.Description
End Function

Public Function NewAccessFile(pDatabase As String, _
 pNewDataSet As String, pNewFile As String, Optional pMoreFields As IFields) As IFeatureClass
On Error GoTo ErrorHandler

    Dim pName As IName
    Dim pOutShpWspName As IWorkspaceName
    Dim pShapeWorkspace As IWorkspace
    Dim pOutputFields As IFields
    Dim pFieldChecker As IFieldChecker
    Dim pErrorEnum As IEnumFieldError
    Dim pNewFields As IFields, pField As IField
    Dim pClone As IClone, pCloneFields As IFields
    Dim pFeatureWorkspace As IFeatureWorkspace
    Dim pDataset As IFeatureDataset
    Dim shapeFieldName As String
    Dim pNewFeatClass As IFeatureClass
    Dim pFieldsEdit As IFieldsEdit
    Dim newFieldEdit As IFieldEdit
    Dim pGeomDef As IGeometryDef
    Dim pGeomDefEdit As IGeometryDefEdit
    Dim pGeoDataset As IGeoDataset
    Dim i As Integer
  
335:     Set pOutShpWspName = New WorkspaceName

337:     pOutShpWspName.PathName = pDatabase
338:     pOutShpWspName.WorkspaceFactoryProgID = "esriDataSourcesGDB.AccessWorkspaceFactory"
339:     Set pName = pOutShpWspName
340:     Set pShapeWorkspace = pName.Open
341: i = 1
    'Open the dataset
343:     Set pFeatureWorkspace = pShapeWorkspace
344:     Set pDataset = pFeatureWorkspace.OpenFeatureDataset(pNewDataSet)
345: i = 2
    ' Add the SHAPE field (based on the dataset)
347:     Set pFieldsEdit = pMoreFields
348:     Set pField = New Field
349:     Set newFieldEdit = pField
350:     With newFieldEdit
351:         .Name = c_DefaultFld_Shape
352:         .Type = esriFieldTypeGeometry
353:         .IsNullable = True
354:         .Editable = True
355:     End With
356:     Set pGeomDef = New GeometryDef
357:     Set pGeomDefEdit = pGeomDef
358:     With pGeomDefEdit
359:         .GeometryType = esriGeometryPolygon
360:         If TypeOf pDataset Is IGeoDataset Then
361:             Set pGeoDataset = pDataset
362:             Set .SpatialReference = pGeoDataset.SpatialReference
363:         Else
364:             Set .SpatialReference = New UnknownCoordinateSystem
365:         End If
366:         .GridCount = 1
367:         .GridSize(0) = 200
368:         .HasM = False
369:         .HasZ = False
370:         .AvgNumPoints = 4
371:     End With
372:     Set newFieldEdit.GeometryDef = pGeomDef
373:     pFieldsEdit.AddField pField
    ' Check the fields
375:     Set pFieldChecker = New FieldChecker
376:     Set pFieldChecker.ValidateWorkspace = pShapeWorkspace
377:     Set pNewFields = pMoreFields
378: i = 3
379:     Set pClone = pNewFields
380:     Set pCloneFields = pClone.Clone
381:     pFieldChecker.Validate pCloneFields, pErrorEnum, pOutputFields
      
  ' Create the output featureclass
  Dim pUID As New UID
385:   pUID = "{52353152-891A-11D0-BEC6-00805F7C4268}"
386:     shapeFieldName = c_DefaultFld_Shape
387: i = 4
388:     Set pNewFeatClass = pDataset.CreateFeatureClass(pNewFile, pOutputFields, pUID, Nothing, esriFTSimple, shapeFieldName, "")
389: i = 5
390:     Set NewAccessFile = pNewFeatClass
  
    Exit Function
  
ErrorHandler:
395:     MsgBox Err.Number & " " & Erl & " - " & Err.Description, vbCritical, "Error in NewAccessFile " & i
End Function

Public Function NewShapeFile(pNewFile As String, pMap As IMap, _
            Optional pMoreFields As IFields) As IFeatureClass
  On Error GoTo ErrorHandler

    
    Dim pOutShpWspName As IWorkspaceName
    Dim pName As IName
    Dim pShapeWorkspace As IWorkspace
    Dim pOutputFields As IFields
    Dim pFieldChecker As IFieldChecker
    Dim pErrorEnum As IEnumFieldError
    Dim pNewFields As IFields, pField As IField
    Dim pClone As IClone, pCloneFields As IFields
    Dim featureclassName As String, pNewFeatClass As IFeatureClass
    Dim pFeatureWorkspace As IFeatureWorkspace
    Dim pUID As IUID
    Dim shapeFieldName As String
    Dim pFieldsEdit As IFieldsEdit
    Dim newFieldEdit As IFieldEdit
    Dim pGeomDef As IGeometryDef
    Dim pGeomDefEdit As IGeometryDefEdit
    
'MsgBox pNewFile
    ' Open the workspace for the new shapefile
422:     Set pOutShpWspName = New WorkspaceName
423:     pOutShpWspName.PathName = EntryName(pNewFile)
424:     pOutShpWspName.WorkspaceFactoryProgID = "esriDataSourcesFile.ShapefileWorkspaceFactory.1"
425:     Set pName = pOutShpWspName
426:     Set pShapeWorkspace = pName.Open
    ' Add the SHAPE field (based on the Map)
428:     Set pFieldsEdit = pMoreFields
429:     Set pField = New Field
430:     Set newFieldEdit = pField
431:     newFieldEdit.Name = c_DefaultFld_Shape
432:     newFieldEdit.Type = esriFieldTypeGeometry
433:     Set pGeomDef = New GeometryDef
434:     Set pGeomDefEdit = pGeomDef
435:     With pGeomDefEdit
436:         .GeometryType = esriGeometryPolygon
437:         Set .SpatialReference = pMap.SpatialReference
438:     End With
439:     Set newFieldEdit.GeometryDef = pGeomDef
440:     pFieldsEdit.AddField pField
    ' Validate field names
442:     Set pFieldChecker = New FieldChecker
443:     Set pFieldChecker.ValidateWorkspace = pShapeWorkspace
444:     Set pNewFields = pMoreFields
445:     Set pClone = pNewFields
446:     Set pCloneFields = pClone.Clone
447:     pFieldChecker.Validate pCloneFields, pErrorEnum, pOutputFields
    ' Create the output featureclass
449:     shapeFieldName = c_DefaultFld_Shape
450:     featureclassName = Mid(pNewFile, Len(pOutShpWspName.PathName) + 2)
451:     Set pFeatureWorkspace = pShapeWorkspace
452:     Set pNewFeatClass = pFeatureWorkspace.CreateFeatureClass(featureclassName, pOutputFields, _
                            Nothing, Nothing, esriFTSimple, shapeFieldName, "")
    ' Return
455:     Set NewShapeFile = pNewFeatClass
  
    Exit Function
  


  Exit Function
ErrorHandler:
  HandleError True, "NewShapeFile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), Err.Number, Err.source, Err.Description, 4
End Function

Public Function EntryName(sFile As String) As String
  ' work from the right side to the first file delimeter
  Dim iLength As Integer
469:   iLength = Len(sFile)
  Dim iCounter As Integer
  Dim sDelim As String
472:   sDelim = "\"
  Dim sRight As String
  
475:   For iCounter = iLength To 0 Step -1
    
477:     If Mid$(sFile, iCounter, 1) = sDelim Then
478:       EntryName = Mid$(sFile, 1, (iCounter - 1))
479:       Exit For
480:     End If
  
482:   Next
  
End Function

Public Sub TurnOffClipping(pSeriesProps As IDSMapSeriesProps, pApp As IApplication)
On Error GoTo ErrHand:
  Dim pMap As IMap, pDoc As IMxDocument
  'Find the data frame
490:   Set pDoc = pApp.Document
491:   Set pMap = FindDataFrame(pDoc, pSeriesProps.DataFrameName)
  If pMap Is Nothing Then Exit Sub
  
494:   pMap.ClipGeometry = Nothing

  Exit Sub
ErrHand:
498:   MsgBox "TurnOffClipping - " & Erl & " - " & Err.Description
End Sub

'Public Sub RemoveIndicators(pApp As IApplication)
'On Error GoTo ErrHand:
'  Dim lLoop As Long, pDoc As IMxDocument, pDelColl As Collection
'  Dim pPage As IPageLayout, pGraphCont As IGraphicsContainer
'  Dim pElem As IElement, pMapFrame As IMapFrame
'508:   Set pDoc = pApp.Document
'509:   Set pPage = pDoc.PageLayout
'510:   Set pDelColl = New Collection
'511:   Set pGraphCont = pPage
'512:   pGraphCont.Reset
'513:   Set pElem = pGraphCont.Next
'514:   Do While Not pElem Is Nothing
'515:     If TypeOf pElem Is IMapFrame Then
'516:       Set pMapFrame = pElem
'517:       If pMapFrame.Map.Name = "Local Indicator" Or _
'       pMapFrame.Map.Name = "Global Indicator" Then
'519:         pDelColl.Add pMapFrame
'520:       End If
'521:     End If
'
'523:     Set pElem = pGraphCont.Next
'524:   Loop
'
'526:   For lLoop = 1 To pDelColl.count
'527:     pGraphCont.DeleteElement pDelColl.Item(lLoop)
'528:   Next lLoop
'
'  Exit Sub
'ErrHand:
'532:   MsgBox "RemoveIndicators - " & Erl & " - " & Err.Description
'End Sub

Public Sub RemoveLabels(pDoc As IMxDocument)
On Error GoTo ErrHand:
  Dim pGraphicsCont As IGraphicsContainer
  Dim pTempColl As Collection, pElemProps As IElementProperties, lLoop As Long
  'Remove any previous neighbor labels.
538:   Set pGraphicsCont = pDoc.PageLayout
539:   pGraphicsCont.Reset
540:   Set pTempColl = New Collection
541:   Set pElemProps = pGraphicsCont.Next
542:   Do While Not pElemProps Is Nothing
543:     If pElemProps.Name = "DSMAPBOOK TEXT" Then
544:       pTempColl.Add pElemProps
545:     End If
546:     Set pElemProps = pGraphicsCont.Next
547:   Loop
548:   For lLoop = 1 To pTempColl.count
549:     pGraphicsCont.DeleteElement pTempColl.Item(lLoop)
550:   Next lLoop
551:   Set pTempColl = Nothing

  Exit Sub
ErrHand:
555:   MsgBox "RemoveLabels - " & Erl & " - " & Err.Description
End Sub

Public Sub RemoveClipElement(pDoc As IMxDocument)
On Error GoTo ErrHand:
  Dim pGraphs As IGraphicsContainer, pElemProps As IElementProperties
  
  'Search for an existing clip element and delete it when found
563:   Set pGraphs = pDoc.FocusMap
564:   pGraphs.Reset
565:   Set pElemProps = pGraphs.Next
566:   Do While Not pElemProps Is Nothing
567:     If TypeOf pElemProps Is IPolygonElement Then
568:       If UCase(pElemProps.Name) = "DSMAPBOOK CLIP ELEMENT" Then
569:         pGraphs.DeleteElement pElemProps
570:         Exit Do
571:       End If
572:     End If
573:     Set pElemProps = pGraphs.Next
574:   Loop
575:   pDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing

  Exit Sub
ErrHand:
579:   MsgBox "RemoveClipElement - " & Erl & " - " & Err.Description
End Sub

'This routine draws the page at the specified index, but does not set it as te current page.
Public Sub DrawPage(pApp As IApplication, pMapSeries As IDSMapSeries, lPageIndex As Long, Optional bRefreshFlag As Boolean = False)
On Error GoTo ErrHand:
  Dim pMapPage As IDSMapPage
  Dim pSeriesProps As IDSMapSeriesProps
587:   If pMapSeries Is Nothing Then
588:     MsgBox "Error: pMapSeries is Nothing", vbInformation, "Error in modGeneralFunctions.DrawPage"
    Exit Sub
590:   End If
591:   If lPageIndex < 0 Or lPageIndex >= pMapSeries.PageCount Then
592:     MsgBox "Error: '" & lPageIndex & "' is not a valid page index in the current map series.", vbInformation, "Error in modGeneralFunctions.DrawPage"
    Exit Sub
594:   End If
  
596:   Set pMapPage = pMapSeries.Page(lPageIndex)
597:   Set pSeriesProps = pMapSeries
598:   pMapPage.DrawPage pApp.Document, pMapSeries, bRefreshFlag

  'Show page-specific annotation
601:   ShowPageSpecificAnnotation pApp, (lPageIndex + pSeriesProps.StartNumber)
  'Rotate the hatch labels
603:   RotateHatchLabels pApp, (lPageIndex + pSeriesProps.StartNumber)

  Exit Sub
ErrHand:
607:   MsgBox "DrawPage - " & Erl & " - " & Err.Description
End Sub

'This routine sets the current page in the map series and displays the page.
Public Sub ViewPage(pApp As IApplication, pMapSeries As IDSMapSeries, lPageIndex As Long)
On Error GoTo ErrHand:
  Dim pSeriesOpts As IDSMapSeriesOptions, pSeriesOpts2 As IDSMapSeriesOptions2
  Dim pMapPage As IDSMapPage
  Dim pSeriesProps As IDSMapSeriesProps
'MsgBox "lPageIndex: " + CStr(lPageIndex)
617:   Set pSeriesOpts = pMapSeries
618:   Set pSeriesOpts2 = pSeriesOpts
619:   Set pSeriesProps = pMapSeries
  
621:   Set pMapPage = pMapSeries.Page(lPageIndex)

623:   If pMapPage Is Nothing Then
624:     MsgBox "Warning:  Map page is nothing." & vbNewLine & "lPageIndex=" & Str(lPageIndex) & vbNewLine & "start#=" & pSeriesProps.StartNumber _
, vbOKOnly, "In modGeneralFunctions.ViewPage"
626:   End If

628:   pMapPage.DrawPage pApp.Document, pMapSeries, True
629:   If pSeriesOpts2.ClipData > 0 Then
630:     g_bClipFlag = True
631:   End If
632:   If pSeriesOpts.RotateFrame Then
633:     g_bRotateFlag = True
634:   End If
635:   If pSeriesOpts.LabelNeighbors Then
636:     g_bLabelNeighbors = True
637:   End If
  'Set the current page number
639:   pMapSeries.CurrentPageNumber = lPageIndex + pSeriesProps.StartNumber 'lPageIndex is a zero-based index, so add the starting page# to get true page#
'MsgBox "Page Number: " & CStr(pMapSeries.CurrentPageNumber)
  
  'Show page-specific annotation
643:   ShowPageSpecificAnnotation pApp, (lPageIndex + pSeriesProps.StartNumber)
  'Rotate the hatch labels
645:   RotateHatchLabels pApp, (lPageIndex + pSeriesProps.StartNumber)
  'Display the current page as bold in the table of contents.
647:   ShowCurrentPageAsBoldInTOC pMapSeries
  Exit Sub
ErrHand:
650:   MsgBox "ViewPage - " & Erl & " - " & Err.Description
End Sub

'This sub-routine displays the current page as bold in the table of contents.
Public Sub ShowCurrentPageAsBoldInTOC(pMapSeries As IDSMapSeries)
  Dim lPage As Long 'zero based page index
  Dim pSeriesProps As IDSMapSeriesProps
657:   Set pSeriesProps = pMapSeries
658:   lPage = pMapSeries.CurrentPageNumber - pSeriesProps.StartNumber
  
  'Remove the bold from the other pages.
  Dim pNode As Node
  Dim lLoop As Long
663:   For lLoop = 0 To pMapSeries.PageCount - 1
664:     Set pNode = g_pFrmMapSeries.tvwMapBook.Nodes.Item(lLoop + 3)
665:     pNode.Bold = False
'    pNode.BackColor = vbWindowBackground
'    pNode.ForeColor = vbWindowText
668:   Next lLoop

  'Make the current page number bold:
  Dim pNode2 As Node
672:   Set pNode2 = g_pFrmMapSeries.tvwMapBook.Nodes.Item(lPage + 3)
673:   pNode2.Bold = True
'  pNode2.BackColor = vbGreen
'  pNode2.ForeColor = vbMagenta

End Sub

''This function uses lazy evaluation to retrieve a ConvertLabelsToAnnotation
''object or create one if it does not already exist.
'Private Function Get_ConvertLabelsToAnnotation_Object(pMap As IMap, pTC As ITrackCancel) As IConvertLabelsToAnnotation
'586:   If m_pConvertLabelsToAnnotation Is Nothing Then
'587:     Set m_pConvertLabelsToAnnotation = New ConvertLabelsToAnnotation
'588:     m_pConvertLabelsToAnnotation.Initialize pMap, esriMapAnnotation, esriVisibleFeatures, True, pTC
'589:   End If
'590:   Set Get_ConvertLabelsToAnnotation_Object = m_pConvertLabelsToAnnotation
'End Function

'Converts Labels on the current page into an annotation group
Public Sub CreatePageSpecificAnnotation(pApp As IApplication, pLayer As ILayer, sPageNumber As String)
On Error GoTo ErrHand:
'MsgBox "start CreatePageSpecificAnnotation"
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
  Dim pTC As ITrackCancel
  Dim pConvertLabelsToAnnotation As IConvertLabelsToAnnotation
698:   Set pMxDoc = pApp.Document
699:   Set pActiveView = pMxDoc.ActiveView
700:   Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
701:   Set pTC = New CancelTracker
702:   pTC.CancelOnClick = False
703:   pTC.CancelOnKeyPress = False

  'Initialize the "Convert Labels To Annotation" object
  'Settings include:
  '1. Store annotation in map
  '2. Create annotation only for visible features
  '3. Generate Unplaced Annotation

'613:   Set pConvertLabelsToAnnotation = Get_ConvertLabelsToAnnotation_Object(pMap, pTC)
712:   Set pConvertLabelsToAnnotation = New ConvertLabelsToAnnotation
713:   pConvertLabelsToAnnotation.Initialize pMap, esriMapAnnotation, esriVisibleFeatures, True, pTC

'MsgBox "1"

'MsgBox "2"
718:   If Not TypeOf pLayer Is IFeatureLayer Then
719:     MsgBox "Warning, the following layer: '" & pLayer.Name & "' is not a feature layer"
    Exit Sub
721:   End If

  'Insert code here to check if labeling is turned on for the layer.
  'If not, then turn it on.

  Dim pFeatureLayer As IFeatureLayer
  Dim pGeoFeatureLayer As IGeoFeatureLayer
728:   Set pFeatureLayer = pLayer
'MsgBox "3"
  'Add the layer information to the converter object.
  'Since this is just map annotation, specify only the layer and the annotation group name
  '(explore some of the optional parameters later)
733:   pConvertLabelsToAnnotation.AddFeatureLayer pFeatureLayer, pFeatureLayer.Name & MAPBOOK_ANNO_CONST & sPageNumber
'MsgBox "4"

  'After setting everything up, now perform the conversion
737:   pConvertLabelsToAnnotation.ConvertLabels
'MsgBox "5"
 'Turn the labels for the converted layer off
740:   Set pGeoFeatureLayer = pLayer
741:   pGeoFeatureLayer.DisplayAnnotation = False
742:   pActiveView.ContentsChanged
'MsgBox "6"
  'Show the overflow dialog
'736:   pApp.ShowDialog esriMxDlgOverflowLabels, True

'  'Turn off the newly created annotation.  Pass -1 as page number so all pages are hidden.
'631:   ShowPageSpecificAnnotation pApp, -1

''Save the map document
'634: pApp.SaveDocument

'MsgBox "end CreatePageSpecificAnnotation"
  Exit Sub
ErrHand:
756:   MsgBox "CreatePageSpecificAnnotation - " & Erl & " - " & Err.Description
End Sub

'Turn Page-Specific Annotation Layers on or off when a new page is selected.
Public Sub ShowPageSpecificAnnotation(pApp As IApplication, lPage As Long)
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
765:   Set pMxDoc = pApp.Document
766:   Set pActiveView = pMxDoc.ActiveView
767:   Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
  
  'Get the 'Basic Graphics Layer' which is a container for the annotation groups.
  'Dim pBasicGraphLyr As ICompositeGraphicsLayer
  Dim pBasicGraphLyr As ICompositeLayer
772:   Set pBasicGraphLyr = pMap.BasicGraphicsLayer
  
  'Loop through each annotation group
  Dim lngLoopCount As Long
776:   For lngLoopCount = 0 To (pBasicGraphLyr.count - 1)
    Dim pLayer As ILayer
    Dim sLayerName As String
    
780:     Set pLayer = pBasicGraphLyr.Layer(lngLoopCount)
781:     sLayerName = pLayer.Name
'MsgBox "pLayer.Name: " & CStr(sLayerName) & "   pLayer.Visible: " & pLayer.Visible
  
    Dim lSubStrLocation As Long
    Dim sSubStr As String
    Dim sPageNumber As String
    Dim lPageNumber As Long
    Dim bCurrentPage As Boolean
789:     lSubStrLocation = InStrRev(sLayerName, MAPBOOK_ANNO_CONST)
  
    'Look for the special substring that indicates this is a mapbook annotation layer.
792:     If (lSubStrLocation <> 0) Then
      'Get the substring
794:       sSubStr = Mid(sLayerName, lSubStrLocation)
      'Get the page number
796:       sPageNumber = Mid(sSubStr, Len(MAPBOOK_ANNO_CONST) + 1)
797:       lPageNumber = val(sPageNumber)
798:       bCurrentPage = (lPageNumber = lPage)
'MsgBox "sSubStr: " & sSubStr & "   lPageNumber: " & CStr(lPageNumber) _
      & "   lPage: " & CStr(lPage) & "   bCurrentPage: " & CStr(bCurrentPage)
      
      'Show or hide the annotation group
803:       pLayer.Visible = bCurrentPage
804:     End If
      
806:   Next lngLoopCount

  Exit Sub
ErrHand:
810:   MsgBox "ShowPageSpecificAnnotation - " & Erl & " - " & Err.Description
End Sub

'When a page is deleted from the map series, call this function to delete the page-specific annotation.
'It takes an application pointer and a page number as input.
Public Sub DeletePageSpecificAnnotation(pApp As IApplication, ByVal sPageNumber As String)
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
  Dim pBasicGraphLyr As ICompositeLayer
  Dim pCompositeGraphicsLayer As ICompositeGraphicsLayer
822:   Set pMxDoc = pApp.Document
823:   Set pActiveView = pMxDoc.ActiveView
824:   Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
825:   Set pBasicGraphLyr = pMap.BasicGraphicsLayer
826:   Set pCompositeGraphicsLayer = pBasicGraphLyr

  'Loop through each of the annotation groups
  'to find annotation groups for the specified page.
  Dim lLoop As Long
831:   For lLoop = (pBasicGraphLyr.count - 1) To 0 Step -1
    'Get the name of the annotation layer/group
    Dim pGraphicAnnotationLayer As ILayer
    Dim sGraphicAnnotationLayerName As String
835:     Set pGraphicAnnotationLayer = pBasicGraphLyr.Layer(lLoop)
836:     sGraphicAnnotationLayerName = pGraphicAnnotationLayer.Name
'MsgBox "sGraphicAnnotationLayerName: " + sGraphicAnnotationLayerName
     
    'Parse out the page number from the name of the annotation layer.
    Dim lSubStrLocation As Long
    Dim sSubStr As String
    Dim sCurrPageNum As String
843:     lSubStrLocation = InStrRev(sGraphicAnnotationLayerName, MAPBOOK_ANNO_CONST)
    'Look for the special substring that indicates this is a mapbook annotation layer.
845:     If (lSubStrLocation <> 0) Then
      'Get the substring
847:       sSubStr = Mid(sGraphicAnnotationLayerName, lSubStrLocation)
      'Get the page number
849:       sCurrPageNum = Mid(sSubStr, Len(MAPBOOK_ANNO_CONST) + 1)
850:       If sCurrPageNum = sPageNumber Then 'If the page number matches, then delete the annotation group/layer
851:         pCompositeGraphicsLayer.DeleteLayer sGraphicAnnotationLayerName
852:       End If
853:     End If
854:   Next lLoop 'End of loop through annotation groups
  Exit Sub
ErrHand:
857:   MsgBox "DeletePageSpecificAnnotation - " & Erl & " - " & Err.Description
End Sub

'When a page is deleted from the map series, call this function to delete the page-specific map elements.
'It takes an application pointer and a page number as input.
Public Sub DeletePageSpecificMapElements(pApp As IApplication, ByVal sPageNumber As String)
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pGraphCont As IGraphicsContainer
  Dim pElement As IElement
  Dim pElementProp As IElementProperties
  Dim ToDelete As Collection
  Dim varItem As Variant
870:   Set pMxDoc = pApp.Document
871:   Set pGraphCont = pMxDoc.PageLayout
872:   Set ToDelete = New Collection
  
874:   pGraphCont.Reset
875:   Set pElement = pGraphCont.Next
876:   While Not pElement Is Nothing
877:     Set pElementProp = pElement
    'If this is a page specific map element, and belongs to the right page
879:     If (pElementProp.Name = TAG_CONST_THISPAGEONLY) _
    And (pElementProp.Type = sPageNumber) Then
      'Mark the map element for deletion.
882:       ToDelete.Add pElement
883:     End If
884:     Set pElement = pGraphCont.Next
885:   Wend
  'After looping through all of the map elements,
  'delete the ones that are marked for deletion.
888:   For Each varItem In ToDelete
889:     Set pElement = varItem
890:     pGraphCont.DeleteElement pElement
891:   Next varItem
  
  Exit Sub
ErrHand:
895:   MsgBox "DeletePageSpecificMapElements - " & Erl & " - " & Err.Description
End Sub

'When a page is deleted from the map series, call this function to re-number the page-specific annotation.
'It takes an application pointer and a page number as input.
'It adjusts the page numbers of items that belong to the pages that come after the deleted page.
'Page numbers are shifted down by 1.
Public Sub ReNumberPageSpecificAnnotation(pApp As IApplication, ByVal sPageNumber As String)
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
  Dim pBasicGraphLyr As ICompositeLayer
  Dim pCompositeGraphicsLayer As ICompositeGraphicsLayer
  Dim lPageNumber As Long
910:   Set pMxDoc = pApp.Document
911:   Set pActiveView = pMxDoc.ActiveView
912:   Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
913:   Set pBasicGraphLyr = pMap.BasicGraphicsLayer
914:   Set pCompositeGraphicsLayer = pBasicGraphLyr
915:   lPageNumber = val(sPageNumber)

  'Loop through each of the annotation groups
  'to find annotation groups for the specified page.
  Dim lLoop As Long
920:   For lLoop = (pBasicGraphLyr.count - 1) To 0 Step -1
    'Get the name of the annotation layer/group
    Dim pGraphicAnnotationLayer As ILayer
    Dim sGraphicAnnotationLayerName As String
924:     Set pGraphicAnnotationLayer = pBasicGraphLyr.Layer(lLoop)
925:     sGraphicAnnotationLayerName = pGraphicAnnotationLayer.Name
'MsgBox "sGraphicAnnotationLayerName: " + sGraphicAnnotationLayerName
     
    'Parse out the page number from the name of the annotation layer.
    Dim lSubStrLocation As Long
    Dim sSubStr As String
    Dim sCurrPageNum As String
    Dim lCurrPageNum As Long
933:     lSubStrLocation = InStrRev(sGraphicAnnotationLayerName, MAPBOOK_ANNO_CONST)
    'Look for the special substring that indicates this is a mapbook annotation layer.
935:     If (lSubStrLocation <> 0) Then
      'Get the substring
937:       sSubStr = Mid(sGraphicAnnotationLayerName, lSubStrLocation)
      'Get the page number
939:       sCurrPageNum = Mid(sSubStr, Len(MAPBOOK_ANNO_CONST) + 1)
940:       lCurrPageNum = val(sCurrPageNum)
      'If the page number is after the deleted page, renumber the page.
942:       If lCurrPageNum > lPageNumber Then
'MsgBox "Old Name: " + sGraphicAnnotationLayerName
        Dim sNewPageNumber As String
        Dim sNewName As String
946:         sNewPageNumber = CStr(lCurrPageNum - 1)
947:         sNewName = Mid(sGraphicAnnotationLayerName, 1, lSubStrLocation - 1) + MAPBOOK_ANNO_CONST + sNewPageNumber
'MsgBox "New Name: " + sNewName
949:         pGraphicAnnotationLayer.Name = sNewName
950:       End If
951:     End If
952:   Next lLoop 'End of loop through annotation groups

  Exit Sub
ErrHand:
956:   MsgBox "ReNumberPageSpecificAnnotation - " & Erl & " - " & Err.Description
End Sub


'When a page is deleted from the map series, call this function to re-number the page-specific map elements.
'It takes an application pointer and a page number as input.
'It adjusts the page numbers of items that belong to the pages that come after the deleted page.
'Page numbers are shifted down by 1.
Public Sub ReNumberPageSpecificMapElements(pApp As IApplication, ByVal sPageNumber As String)
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pGraphCont As IGraphicsContainer
  Dim pElement As IElement
  Dim pElementProp As IElementProperties
  Dim ToReNumber As Collection
  Dim varItem As Variant
  Dim lPageNumber As Long
  Dim sCurrentPageNumber As String
  Dim lCurrentPageNumber As Long
  
976:   Set pMxDoc = pApp.Document
977:   Set pGraphCont = pMxDoc.PageLayout
978:   Set ToReNumber = New Collection
  
980:   pGraphCont.Reset
981:   Set pElement = pGraphCont.Next
982:   While Not pElement Is Nothing
983:     Set pElementProp = pElement
    'If this is a page specific map element, and belongs to the right page
985:     If (pElementProp.Name = TAG_CONST_THISPAGEONLY) Then
986:       lPageNumber = val(sPageNumber)
987:       sCurrentPageNumber = pElementProp.Type
988:       lCurrentPageNumber = val(sCurrentPageNumber)
989:       If (lCurrentPageNumber > lPageNumber) Then
        'Mark the map element for renumbering.
991:         ToReNumber.Add pElement
992:       End If
993:     End If
994:     Set pElement = pGraphCont.Next
995:   Wend
  'After looping through all of the map elements,
  'renumber the ones that are marked for renumbering.
998:   For Each varItem In ToReNumber
999:     Set pElement = varItem
1000:     Set pElementProp = pElement
1001:     sCurrentPageNumber = pElementProp.Type
1002:     lCurrentPageNumber = val(sCurrentPageNumber)
1003:     lCurrentPageNumber = lCurrentPageNumber - 1
1004:     pElementProp.Type = CStr(lCurrentPageNumber)
1005:   Next varItem

  Exit Sub
ErrHand:
1009:   MsgBox "ReNumberPageSpecificMapElements - " & Erl & " - " & Err.Description
End Sub

'This function determines if a file already exists and if so, returns true.
Public Function CheckFileExists(FileName As String) As Boolean
  Dim fso
1015:   Set fso = CreateObject("Scripting.FileSystemObject")
1016:   CheckFileExists = fso.fileexists(FileName)
End Function

'This function determines if a geodatabase table already exists and if so, returns true.
Public Function CheckGdbTableExists(sWorkspace As String, sTable As String) As Boolean
  'this will fail to find an existing sde table because
  'the code will not read the username and concatenate
  'with the name of the table for sde table names
  On Error Resume Next
  Dim pAccessFactory As IWorkspaceFactory
1026:   Set pAccessFactory = New AccessWorkspaceFactory
  Dim pAccessWorkspace As IWorkspace
1028:   Set pAccessWorkspace = pAccessFactory.OpenFromFile(sWorkspace, 0)
  Dim pFeatureWorkspace As IFeatureWorkspace
1030:   Set pFeatureWorkspace = pAccessWorkspace

  Dim pTable As ITable
1033:   Set pTable = pFeatureWorkspace.OpenTable(sTable)

1035:   If pTable Is Nothing Then
1036:     CheckGdbTableExists = False
1037:   Else
1038:     CheckGdbTableExists = True
1039:   End If
End Function

'This function finds all of the feature layers in the map that have hatches and rotates the hatch labels
'to (page_roation * -1).
Public Sub RotateHatchLabels(pApp As IApplication, lPage As Long)
On Error GoTo ErrHand:
  'Find the page rotation angle of the current page
  Dim dPageRotationAngle As Double
1048:   dPageRotationAngle = GetPageRotation(pApp, lPage)
  
  Dim pMap As IMap
1051:   Set pMap = FindDataFrame2(pApp)
  
  'Set pMap = pMxDoc.Maps.Item(0) 'This method of getting the data frame is not reliable.
1054:   If pMap Is Nothing Then
1055:     MsgBox "Could not find the DataFrame in the RotateHatchLabels routine!"
    Exit Sub
1057:   End If
  
  Dim pEnumLayer As IEnumLayer
  Dim pLayer As ILayer
  Dim pId As New UID
  
  Dim bFoundHatchLayer As Boolean
1064:   bFoundHatchLayer = False
  
1066:   pId = "{40A9E885-5533-11d0-98BE-00805F7CED21}" ' IFeatureLayer

1068:   Set pEnumLayer = pMap.Layers(pId, True)
1069:   pEnumLayer.Reset
1070:   Set pLayer = pEnumLayer.Next
1071:   Do While Not pLayer Is Nothing
    Dim pLayerExt As ILayerExtensions
    Dim pHatchLayerExt As IHatchLayerExtension
    
1075:     Set pLayerExt = pLayer
    
    'Loop through each of the "layer extensions" to try and find a "Hatch Layer" extension.
1078:     Set pHatchLayerExt = Nothing
1079:     If pLayerExt.ExtensionCount > 0 Then
        Dim i As Integer
1081:         For i = 0 To pLayerExt.ExtensionCount - 1
1082:             If TypeOf pLayerExt.Extension(i) Is IHatchLayerExtension Then
1083:               Set pHatchLayerExt = pLayerExt.Extension(i)
1084:               Exit For
1085:             End If
1086:         Next i
1087:     End If
    'If a hatch layer was found, show message box.
1089:     If pHatchLayerExt Is Nothing Then
1090:     Else
        'MsgBox pLayer.Name & " is a hatch layer."
1092:         bFoundHatchLayer = True
        
1094:         RotateHatchLabelsHelper pHatchLayerExt, dPageRotationAngle
1095:     End If
    
    'Increment the layers loop.
1098:     Set pLayer = pEnumLayer.Next
1099:   Loop
  
  'If bFoundHatchLayer = False Then MsgBox "No hatch layers found."
  
  Exit Sub
ErrHand:
1105:   MsgBox "RotatHatchLabels - " & Erl & " - " & Err.Description
End Sub

'This function is called by the RotateHatchLabels subroutine.
'This function rotates the hatch labels in a particular layer to
'correspond with the page rotation angle.
Public Sub RotateHatchLabelsHelper(pHatchLayerExt As IHatchLayerExtension, dPageRotationAngle As Double)
On Error GoTo ErrHand:
    Dim dHatchRotationAngle As Double
1114:     dHatchRotationAngle = dPageRotationAngle * -1
    
    'Loop through each hatch class in the IHatchLayerExtension
    Dim X As Integer
1118:     For X = 0 To pHatchLayerExt.HatchClassCount - 1
        Dim pHatchClass As IHatchClass
1120:         Set pHatchClass = pHatchLayerExt.HatchClass(pHatchLayerExt.HatchClassNames(X))
        
        'Get the Hatch Definition enumeration
        Dim pHatchTemplate As IHatchTemplate
        Dim pEnumHatchDefinition As IEnumHatchDefinition
        'Dim pTextSymbol As ITextSymbol
1126:         Set pHatchTemplate = pHatchClass.HatchTemplate
1127:         Set pEnumHatchDefinition = pHatchTemplate.EnumHatchDefinitions
        
        'Loop through the Hatch Definition enumeration
        Dim pHatchDefinition As IHatchDefinition, lHatchInterval As Long
1131:         pEnumHatchDefinition.Reset
1132:         pEnumHatchDefinition.Next lHatchInterval, pHatchDefinition
1133:         Do Until pHatchDefinition Is Nothing
            'Modify the hatch definition here by changing the angle of rotation of the text symbol.
1135:             ModifyHatchDefinition pHatchDefinition, dHatchRotationAngle
            'Increment the loop
1137:             pEnumHatchDefinition.Next lHatchInterval, pHatchDefinition
1138:         Loop
        
        'Now perform the same logic on the EndHatchDefinition
1141:         Set pHatchDefinition = pHatchTemplate.EndHatchDefinition
1142:         If Not pHatchDefinition Is Nothing Then
            'Modify the hatch definition here by changing the angle of rotation of the text symbol.
1144:             ModifyHatchDefinition pHatchDefinition, dHatchRotationAngle
1145:         End If
1146:     Next
  Exit Sub
ErrHand:
1149:   MsgBox "RotatHatcheLabelsHelper - " & Erl & " - " & Err.Description
End Sub

'This function modifies the hatch definition here by changing the angle of rotation of the text symbol.
Public Sub ModifyHatchDefinition(ByRef pHatchDefinition As IHatchDefinition, dHatchRotationAngle As Double)
On Error GoTo ErrHand:
    Dim pTextSymbol As ITextSymbol
    'Do not modify if labeling is turned off.
1157:     If pHatchDefinition.TextDisplay <> esriHatchTDNone Then
1158:         Set pTextSymbol = pHatchDefinition.TextSymbol
1159:         If Not pTextSymbol Is Nothing Then
1160:             pTextSymbol.Angle = dHatchRotationAngle
1161:             Set pHatchDefinition.TextSymbol = pTextSymbol
1162:         End If
1163:     End If
    Exit Sub
ErrHand:
1166:     MsgBox "ModifyHatchDefinition - " & Erl & " - " & Err.Description
End Sub

'This function returns the angle of rotation of the current page.
Public Function GetPageRotation(pApp As IApplication, lPage As Long) As Double
On Error GoTo ErrHand:
    Dim pMapBook As IDSMapBook
    Dim pSeries As IDSMapSeries
    Dim pMapPage As IDSMapPage
    Dim pSeriesProps As IDSMapSeriesProps
    Dim lPageIndex As Long
1177:     Set pMapBook = GetMapBookExtension(pApp)
1178:     Set pSeries = pMapBook.ContentItem(0)
1179:     Set pSeriesProps = pSeries
1180:     lPageIndex = lPage - pSeriesProps.StartNumber
1181:     Set pMapPage = pSeries.Page(lPageIndex)
1182:     GetPageRotation = pMapPage.PageRotation
  Exit Function
ErrHand:
1185:   MsgBox "GetPageRotation - " & Erl & " - " & Err.Description
End Function


Public Function FindDataFrame2(pApp As IApplication) As IMap
On Error GoTo ErrHand:
  Dim pMxDoc As IMxDocument
  Dim pMap As IMap
  Dim pSeriesProps As IDSMapSeriesProps
  Dim pMapBook As IDSMapBook

1196:   Set pMap = Nothing
1197:   Set FindDataFrame2 = pMap
  'Get the data frame.
1199:   Set pMxDoc = pApp.Document
1200:   Set pMapBook = GetMapBookExtension(pApp)
1201:   If pMapBook Is Nothing Then GoTo NoMainDataFrame
1202:   If pMapBook.ContentCount = 0 Then GoTo NoMainDataFrame
1203:   Set pSeriesProps = pMapBook.ContentItem(0)
1204:   Set pMap = FindDataFrame(pMxDoc, pSeriesProps.DataFrameName)
  
1206:   If pMap Is Nothing Then
NoMainDataFrame: 'If there is no main Map Book DataFrame, use the focus map.
1208:     Set pMap = pMxDoc.FocusMap
1209:   End If
1210:   Set FindDataFrame2 = pMap
  Exit Function
ErrHand:
1213:   MsgBox "FindDataFrame2 - " & Erl & " - " & Err.Description
End Function

Public Function GetMapBookExtension(pApp As IApplication) As IDSMapBook
On Error GoTo ErrHand:
  Dim pMapBookExt As DSMapBookExt, pMapBook As IDSMapBook
1219:   Set pMapBookExt = pApp.FindExtensionByName("USACE_JAMB_MapBook")
1220:   If pMapBookExt Is Nothing Then
1221:     MsgBox "USACE JAMB Extension is not installed properly.  Please re-install USACE JAMB.", , "USACE JAMB Extension Not Found!!!"
1222:     Set GetMapBookExtension = Nothing
    Exit Function
1224:   End If
  
1226:   Set GetMapBookExtension = pMapBookExt.MapBook

  Exit Function
ErrHand:
1230:   MsgBox "GetMapBookExtension - " & Erl & " - " & Err.Description
End Function

'This function takes a list of layer names as input and deletes all off the associated page-specific annotation layers.
Public Sub DeletePageSpecificAnnotationByLayerNames(layerNames As Collection, pApp As IApplication)
On Error GoTo ErrHand:
  'Get the focus map
  Dim pMxDoc As IMxDocument
  Dim pActiveView As IActiveView
  Dim pMap As IMap
1240:   Set pMxDoc = pApp.Document
1241:   Set pActiveView = pMxDoc.ActiveView
1242:   Set pMap = FindDataFrame2(pApp) 'get the main Map Book data frame
  'Get the 'Basic Graphics Layer' which is a container for the annotation groups.
  Dim pBasicGraphLyr As ICompositeLayer
  Dim pCompositeGraphicsLayer As ICompositeGraphicsLayer
1246:   Set pBasicGraphLyr = pMap.BasicGraphicsLayer
1247:   Set pCompositeGraphicsLayer = pBasicGraphLyr

  'Loop through each of the layer names
  Dim vLayerName As Variant
1251:   For Each vLayerName In layerNames
    Dim sLayerNameFromListbox As String
1253:     sLayerNameFromListbox = vLayerName
      
    'Loop through each of the annotation groups to find annotation
    'groups that are linked the the names layer.(Inner Loop)
    Dim lInnerLCount As Long
1258:     For lInnerLCount = (pBasicGraphLyr.count - 1) To 0 Step -1
      Dim pGraphicAnnotationLayer As ILayer
      Dim sGraphicAnnotationLayerName As String
1261:       Set pGraphicAnnotationLayer = pBasicGraphLyr.Layer(lInnerLCount)
1262:       sGraphicAnnotationLayerName = pGraphicAnnotationLayer.Name
   
      'Determine if the Graphic layer name contains the special keyword constant.
      Dim lSubStrLocation As Long
1266:       lSubStrLocation = InStrRev(sGraphicAnnotationLayerName, MAPBOOK_ANNO_CONST)
1267:       If (lSubStrLocation <> 0) Then
        'Get the associated feature layer name
        Dim pGraphicsLayer As IGraphicsLayer
        Dim pAssociatedLayer As ILayer
        Dim sMapLayerName As String
1272:         Set pGraphicsLayer = pGraphicAnnotationLayer
1273:         Set pAssociatedLayer = pGraphicsLayer.AssociatedLayer
        
1275:         If pAssociatedLayer Is Nothing Then
1276:         Else
1277:           sMapLayerName = pAssociatedLayer.Name
          'If the associated feature layer name matches the selected
          'layer name from the listbox, delete the annotation group
1280:           If sMapLayerName = sLayerNameFromListbox Then
1281:             pCompositeGraphicsLayer.DeleteLayer sGraphicAnnotationLayerName
1282:           End If
1283:         End If
1284:       End If 'End of Determine if the Graphic layer name contains the special keyword constant.
1285:     Next lInnerLCount 'End of looping through each annotation group (Inner Loop)
1286:   Next vLayerName 'End of looping through the layer names

  'Refresh the map
1289:    pActiveView.Refresh

  Exit Sub
ErrHand:
1293:   MsgBox "DeletePageSpecificAnnotationByLayerNames - " & Erl & " - " & Err.Description
End Sub

'This function determines if a table is empty.
'It takes a table name and an application object as input.
'It returns true if the table has no rows, false otherwise.
'If the table does not exist in the map, a warning is displayed.
Public Function isTableEmpty(strTableName As String, pApp As IApplication) As Boolean
On Error GoTo ErrorHandler
1302:     If GetNumberOfRows(strTableName, pApp) = 0 Then
1303:         isTableEmpty = True
1304:     Else
1305:         isTableEmpty = False
1306:     End If
    Exit Function
ErrorHandler:
1309:     MsgBox "Error in isTableEmpty: " & Erl & " - " & Err.Description, , "isTableEmpty"
End Function

'This function determines the number of rows in a table.
'It takes a table name and an application object as input.
'It returns a long integer with the number or rows.
'If the table does not exist in the map, a warning is displayed.
Public Function GetNumberOfRows(strTableName As String, pApp As IApplication) As Long
On Error GoTo ErrorHandler
    Dim pDataset As IDataset
    Dim pTable As ITable
    Dim lNumRows As Long
    Dim pQueryFilter As IQueryFilter
1322:     Set pDataset = FindDatasetByName(strTableName, pApp)
1323:     If pDataset Is Nothing Then
1324:         MsgBox "Warning: The '" & strTableName & "' dataset was not found.", vbExclamation, "Warning in 'GetNumberOfRows' function."
        Exit Function
1326:     End If
    
1328:     If pDataset.Type = esriDTFeatureClass Or pDataset.Type = esriDTTable Then
1329:         Set pTable = pDataset
1330:     Else
1331:         MsgBox "Warning: The '" & strTableName & "' dataset is not a feature class or table dataset.", vbExclamation, "Warning in 'GetNumberOfRows' function."
        Exit Function
1333:     End If

1335:     Set pQueryFilter = New QueryFilter
1336:     pQueryFilter.WhereClause = ""
1337:     Err.Clear
On Error Resume Next
1339:     lNumRows = pTable.RowCount(pQueryFilter)
1340:     If Err.Number <> 0 Then
1341:         MsgBox "Warning: An error occured while attempting to read the '" & strTableName & "' table or feature class.", vbExclamation, "Warning in 'GetNumberOfRows' function."
        Exit Function
1343:     End If
On Error GoTo ErrorHandler
1345:     GetNumberOfRows = lNumRows
    Exit Function
ErrorHandler:
1348:     MsgBox "Error in GetNumberOfRows: " & Erl & " - " & Err.Description, , "GetNumberOfRows"
End Function

'This subroutine deletes all of the rows from a table or feature class
'It takes a table name and an application object as input
'If the table does not exist in the map, a warning is displayed.
Public Sub DeleteAllRowsFromTable(strTableName As String, pApp As IApplication)
On Error GoTo ErrorHandler
    Dim pDataset As IDataset
    Dim pTable As ITable
    Dim pQueryFilter As IQueryFilter
1359:     Set pDataset = FindDatasetByName(strTableName, pApp)
1360:     If pDataset Is Nothing Then
1361:         MsgBox "Warning: The '" & strTableName & "' dataset was not found.", vbExclamation, "Warning in 'DeleteAllRowsFromTable' subroutine."
        Exit Sub
1363:     End If

1365:     If pDataset.Type = esriDTFeatureClass Or pDataset.Type = esriDTTable Then
1366:         Set pTable = pDataset
1367:     Else
1368:         MsgBox "Warning: The '" & strTableName & "' dataset is not a feature class or table dataset.", vbExclamation, "Warning in 'DeleteAllRowsFromTable' subroutine."
        Exit Sub
1370:     End If

1372:     Set pQueryFilter = New QueryFilter
1373:     pQueryFilter.WhereClause = ""
1374:     Err.Clear
On Error Resume Next
1376:     pTable.DeleteSearchedRows pQueryFilter 'Delete all of the rows from the table.
1377:     If Err.Number <> 0 Then
1378:         MsgBox "Warning: An error occured while attempting to delete all rows from the '" & strTableName & "' table or feature class.", vbExclamation, "Warning in 'DeleteAllRowsFromTable' subroutine."
        Exit Sub
1380:     End If
On Error GoTo ErrorHandler
    Exit Sub
ErrorHandler:
1384:     MsgBox "Error in DeleteAllRowsFromTable: " & Erl & " - " & Err.Description, , "DeleteAllRowsFromTable"
End Sub

