Attribute VB_Name = "modSharedFunctions"
'-----------------------------------------------------------------------------
'    USACE JAMB (Just Another Map Book) Extension for ArcGIS
'    Copyright (C) 2010  U.S. Army Corps of Engineers - New Orleans District
'
'    This library is free software; you can redistribute it and/or
'    modify it under the terms of the GNU Lesser General Public
'    License as published by the Free Software Foundation; either
'    version 2.1 of the License, or (at your option) any later version.
'
'    This library is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'    Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public
'    License along with this library; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'-----------------------------------------------------------------------------
''

'Author: Sam Falchook
'USACE, MVN
'Creation Date: 05/13/2009
'
'Purpose: This module contains functions and subroutines that are shared by multiple projects in the group project.

Option Explicit

Public Function GetFeatureLayerByName(sLayerName As String, pMap As IMap) As IFeatureLayer
On Error GoTo ErrorHandler
  'Grab an enumeration of feature layers from the map (recursively searches group layers)
  Dim pEnumLayer As IEnumLayer
  Dim pId As New UID
14:   pId = FEATURE_LAYER_UID 'UID for IFeatureLayer
15:   Set pEnumLayer = pMap.Layers(pId, True)

  'Loop through the map layers to find a layer with a matching name
  Dim pLayer As IFeatureLayer
19:   pEnumLayer.Reset
20:   Set pLayer = pEnumLayer.Next
21:   Do While Not pLayer Is Nothing
    Dim sLayerNameFromMap As String
23:     sLayerNameFromMap = pLayer.Name

    'If the names match, return the feature layer
26:     If UCase(sLayerNameFromMap) = UCase(sLayerName) Then
27:       If pLayer.FeatureClass Is Nothing Then
28:         MsgBox "Warning, the index layer '" & sLayerName & "' is not connected to the underlying data source. " _
            & "Please re-connect the data source and try again.", vbExclamation
30:         Set GetFeatureLayerByName = Nothing
31:       Else
32:         Set GetFeatureLayerByName = pLayer
33:       End If

      Exit Function
36:     End If
    'Increment to the next layer
38:     Set pLayer = pEnumLayer.Next
39:   Loop

41:   Set GetFeatureLayerByName = Nothing

43:   MsgBox "Warning, the index layer '" & sLayerName & "' was removed, renamed, over moved to a different data frame. " _
      & "Please add the index layer back into the main data frame.", vbExclamation
  Exit Function
ErrorHandler:
47:   MsgBox "GetFeatureLayerByName - " & Erl & " - " & Err.Description
End Function

Public Function FindDataFrame(pDoc As IMxDocument, sFrameName As String) As IMap
On Error GoTo ErrHand:
  Dim lLoop As Long, pMap As IMap
  
  'Find the data frame
55:   For lLoop = 0 To pDoc.Maps.count - 1
56:     If pDoc.Maps.Item(lLoop).Name = sFrameName Then
57:       Set pMap = pDoc.Maps.Item(lLoop)
58:       Exit For
59:     End If
60:   Next lLoop
61:   If Not pMap Is Nothing Then
62:     Set FindDataFrame = pMap
63:   Else
64:     MsgBox "Error, the main data frame was removed or renamed. " _
    & "Please rename the main data frame name back to its original name:" & vbNewLine & sFrameName, vbExclamation
66:     Set FindDataFrame = Nothing
67:   End If

  Exit Function
ErrHand:
71:   MsgBox "FindDataFrame - " & Erl & " - " & Err.Description
End Function

